@@ -130,8 +130,8 @@ static int ali_saveregs(unsigned char regs[])
     int tmp;
     ali_unlock();
     tmp = GETB(GRA_I, 0x0c);
-    regs[EXT + 7] = inb(0x3d6);
-    regs[EXT + 8] = inb(0x3d7);
+    regs[EXT + 7] = port_in(0x3d6);
+    regs[EXT + 8] = port_in(0x3d7);
     outb(0x3d6, 0);
     regs[EXT + 0] = GETB(__svgalib_CRT_I, 0x19);
     regs[EXT + 1] = GETB(__svgalib_CRT_I, 0x1a);
@@ -139,12 +139,12 @@ static int ali_saveregs(unsigned char regs[])
     regs[EXT + 3] = GETB(__svgalib_CRT_I, 0x2a);
     regs[EXT + 4] = GETB(GRA_I, 0x0b);
     regs[EXT + 5] = tmp;
-    inb(0x3c8);
-    inb(0x3c6);
-    inb(0x3c6);
-    inb(0x3c6);
-    inb(0x3c6);
-    regs[EXT + 6] = inb(0x3c6);
+    port_in(0x3c8);
+    port_in(0x3c6);
+    port_in(0x3c6);
+    port_in(0x3c6);
+    port_in(0x3c6);
+    regs[EXT + 6] = port_in(0x3c6);
     return 9;			/* 9 additional registers */
 }
 
@@ -167,11 +167,11 @@ static void ali_setregs(const unsigned char regs[], int mode)
     SETB(GRA_I, 0x0c, regs[EXT + 5]);
     SETB(__svgalib_CRT_I, 0x28, regs[EXT + 2]);
     SETB(__svgalib_CRT_I, 0x2a, regs[EXT + 3]);
-    inb(0x3c8);
-    inb(0x3c6);
-    inb(0x3c6);
-    inb(0x3c6);
-    inb(0x3c6);
+    port_in(0x3c8);
+    port_in(0x3c6);
+    port_in(0x3c6);
+    port_in(0x3c6);
+    port_in(0x3c6);
     outb(0x3c6, regs[EXT + 6]);
     outb(0x3d6, regs[EXT + 7]);
     outb(0x3d7, regs[EXT + 8]);
@@ -273,11 +273,11 @@ static int cwg(void)
 {
     int t, t1, t2, t3;
 
-    t = inb(GRA_I);
+    t = port_in(GRA_I);
     t1 = GETB(GRA_I, 0x0b);
     t2 = t1 & 0x30;
     outb(GRA_D, t2 ^ 0x30);
-    t3 = ((inb(GRA_D) & 0x30) ^ 0x30);
+    t3 = ((port_in(GRA_D) & 0x30) ^ 0x30);
     outb(GRA_D, t1);
     outb(GRA_I, t);
     return (t2 == t3);
@@ -296,7 +296,7 @@ static int ali_test(void)
 
     ali_unlock();
 
-    tmp = inb(__svgalib_CRT_I);
+    tmp = port_in(__svgalib_CRT_I);
     ov = GETB(__svgalib_CRT_I, 0x1a);
     outb(__svgalib_CRT_D, ov & 0xef);
     if (cwg()) {
@@ -444,7 +444,7 @@ static int GETB(int port, int idx)
 {
     if (idx != -1)
 	outb(port, idx);
-    return inb(port + (idx != -1));
+    return port_in(port + (idx != -1));
 }
 
 /* Function table (exported) */
