@@ -71,7 +71,7 @@ static void outlXR(int index,unsigned long d)
 static unsigned char inXR(int index)
 {
     __svgalib_outseq(0x1d,index>>2);
-    return inb(apm_xbase);
+    return port_in(apm_xbase);
 }
 
 #if 0
@@ -108,13 +108,12 @@ return apm_is_linear;
 
 static void apm_getmodeinfo(int mode, vga_modeinfo *modeinfo)
 {
-
     if(modeinfo->colors==16)return;
 
-	modeinfo->maxpixels = apm_memory*1024/modeinfo->bytesperpixel;
+    modeinfo->maxpixels = apm_memory*1024/modeinfo->bytesperpixel;
     modeinfo->maxlogicalwidth = 4088;
-	modeinfo->startaddressrange = apm_memory * 1024 - 1;
-	modeinfo->haveblit = 0;
+    modeinfo->startaddressrange = apm_memory * 1024 - 1;
+    modeinfo->haveblit = 0;
     modeinfo->flags &= ~HAVE_RWPAGE;
 
     if (modeinfo->bytesperpixel >= 1) {
@@ -201,12 +200,8 @@ static int apm_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
-    if((modeinfo->bitsPerPixel==24)&&(apm_chiptype==AT24))return 0;
   
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
@@ -215,6 +210,11 @@ static int apm_modeavailable(int mode)
 
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
+    if((modeinfo->bitsPerPixel==24)&&(apm_chiptype==AT24)) {
+	free(modeinfo);
+	return 0;
+    }
+
     modetiming = malloc(sizeof(ModeTiming));
     if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
@@ -234,7 +234,8 @@ static unsigned comp_lmn(unsigned clock) ;
 
 static void apm_initializemode(unsigned char *moderegs,
 			    ModeTiming * modetiming, ModeInfo * modeinfo, int mode)
-{ long k;
+{
+    long k;
   
     apm_saveregs(moderegs);
     __svgalib_setup_VGA_registers(moderegs, modetiming, modeinfo);
@@ -369,8 +370,7 @@ static int apm_setmode(int mode, int prv_mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    if ((mode < G640x480x256 /*&& mode != G320x200x256*/)
-	|| mode == G720x348x2) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
 
 /*	__svgalib_clear_accelspecs(__svgalib_driverspecs->accelspecs);
 */
@@ -500,7 +500,7 @@ static void apm_setdisplaystart(int address)
    * the appropriate extended registers.
    */
    outb(0x3d4,0x1c);
-   i=(inb(0x3d5)&0xf0)|((address & 0x0f0000) >> 16);
+   i=(port_in(0x3d5)&0xf0)|((address & 0x0f0000) >> 16);
    __svgalib_outCR(0x1c,i);
 /*  modinx(vgaIOBase + 4, 0x1c, 0x0f, (address & 0x0f0000) >> 16); */
 
@@ -517,7 +517,7 @@ static void apm_setlogicalwidth(int width)
 static int apm_linear(int op, int param)
 {
 if (op==LINEAR_ENABLE || op==LINEAR_DISABLE){ apm_is_linear=1-apm_is_linear; return 0;}
-if (op==LINEAR_QUERY_BASE) {__svgalib_outSR(0x1d,0x193>>2); return inb(apm_xbase+3)<<24 ;}
+if (op==LINEAR_QUERY_BASE) {__svgalib_outSR(0x1d,0x193>>2); return port_in(apm_xbase+3)<<24 ;}
 if (op == LINEAR_QUERY_RANGE || op == LINEAR_QUERY_GRANULARITY) return 0;		/* No granularity or range. */
     else return -1;		/* Unknown function. */
 }
@@ -571,6 +571,8 @@ static int apm_init(int force, int par1, int par2)
     char    idstring[]  = "       ";
     int found=0;
     unsigned long buf[64];
+    unsigned char *p;
+    int db, d9;
 
     apm_unlock();
 
@@ -583,11 +585,21 @@ static int apm_init(int force, int par1, int par2)
     };
 
     __svgalib_linear_mem_base=buf[4]&0xffffff00;
-    VGAMap=mmap(0,0x1000,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,
+    
+    p=mmap(0,0x1000, PROT_READ|PROT_WRITE, MAP_SHARED, __svgalib_mem_fd,
+       		__svgalib_linear_mem_base+0xffe000);
+    db=p[0xcdb];
+    d9=p[0xcd9];
+    p[0xcdb] = (db & 0xf4) | 0x0a ;
+    p[0xcd9] = (d9 & 0xcf) | 0x20 ;
+    
+    munmap(p, 0x1000);
+    
+    __svgalib_vgammbase=mmap(0,0x1000,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,
                 __svgalib_linear_mem_base+0xfff000);
-/* not working
-    mm_io_mapio();
-*/
+
+    __svgalib_mm_io_mapio();
+
     apm_memory=__svgalib_inseq(0x20)*64-34; /* maybe will support accel some day */
     if (__svgalib_driver_report) {
 	printf("Using Alliance driver, %.7s, %iKB.\n",idstring, apm_memory);
@@ -604,7 +616,7 @@ static int apm_init(int force, int par1, int par2)
     cardspecs->maxPixelClock24bpp = 75000;
     cardspecs->maxPixelClock32bpp = 94500;
     cardspecs->flags = CLOCK_PROGRAMMABLE | INTERLACE_DIVIDE_VERT;
-    cardspecs->maxHorizontalCrtc = 2040;
+    cardspecs->maxHorizontalCrtc = 4088;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->clocks = NULL;
@@ -614,7 +626,7 @@ static int apm_init(int force, int par1, int par2)
     __svgalib_driverspecs = &__svgalib_apm_driverspecs;
     __svgalib_banked_mem_base=0xa0000;
     __svgalib_banked_mem_size=0x10000;
-    __svgalib_linear_mem_size=apm_memory*0x400;
+    __svgalib_linear_mem_size=apm_memory*1024;
     return 0;
 }
 
