@@ -92,7 +92,7 @@
 static int ark_chip;
 static int ark_memory;
 static int ark_bus;
-static int ark_baseaddress;	/* PCI base address. */
+static int ark_baseaddress, is_linear;	/* PCI base address. */
 static CardSpecs *cardspecs;
 static DacMethods *dac_used;
 static int dac_speed = 0;
@@ -123,10 +123,10 @@ static void ark_unlock(void)
 static void ark_getmodeinfo(int mode, vga_modeinfo * modeinfo)
 {
 #ifdef SVGA_STYLE_320x200x256
-    if ((mode < G640x480x256 || mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	&& mode != G320x200x256) {
 #else
-    if (mode < G640x480x256 || mode == G720x348x2) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
 #endif
 	__svgalib_vga_driverspecs.getmodeinfo(mode, modeinfo);
 	return;
@@ -159,10 +159,10 @@ static int ark_modeavailable(int mode)
     ModeTiming *modetiming;
 
 #ifdef SVGA_STYLE_320x200x256
-    if ((mode < G640x480x256 || mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	&& mode != G320x200x256)
 #else
-    if (mode < G640x480x256 || mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 #endif
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
@@ -210,8 +210,8 @@ static int ark_saveregs(unsigned char regs[])
     regs[ARK_CR46] = __svgalib_inCR(0x46);
     regs[ARK_CR50] = __svgalib_inCR(0x50);
 
-    inb(0x3C8);
-    regs[ARK_PELMASK] = inb(0x3C6);
+    port_in(0x3C8);
+    regs[ARK_PELMASK] = port_in(0x3C6);
 
     dac_used->saveState(regs + ARK_DAC_OFFSET);
     return ARK_DAC_OFFSET - VGA_TOTAL_REGS + dac_used->stateSize;
@@ -244,7 +244,7 @@ static void ark_setregs(const unsigned char regs[], int mode)
     __svgalib_outCR(0x42, regs[ARK_CR42]);
     __svgalib_outCR(0x44, regs[ARK_CR44]);
 
-    inb(0x3C8);
+    port_in(0x3C8);
     outb(0x3C6, regs[ARK_PELMASK]);
     dac_used->restoreState(regs + ARK_DAC_OFFSET);
 
@@ -261,6 +261,9 @@ static void ark_initializemode(unsigned char *moderegs,
 {
 
     /* Get current values. */
+
+
+    is_linear=0;
     ark_saveregs(moderegs);
 
     /* Set up the standard VGA registers for a generic SVGA. */
@@ -283,8 +286,12 @@ static void ark_initializemode(unsigned char *moderegs,
 	moderegs[ARK_VIDEO_CLOCK_SELECT] |= 0x6;
     if (modeinfo->bytesPerPixel == 2)
 	moderegs[ARK_VIDEO_CLOCK_SELECT] |= 0xA;
-    if (modeinfo->bytesPerPixel == 3)
+    if (modeinfo->bytesPerPixel == 3) {
 	moderegs[ARK_VIDEO_CLOCK_SELECT] |= 0x6;
+//        moderegs[ARK_DPMS_CONTROL] &=0xe7;
+//        moderegs[ARK_DPMS_CONTROL] |=0x18;
+        
+    }
     if (modeinfo->bytesPerPixel == 4)
 	moderegs[ARK_VIDEO_CLOCK_SELECT] |= 0xE;
 
@@ -314,6 +321,7 @@ static void ark_initializemode(unsigned char *moderegs,
 	    val |= threshold;
 	}
 	if (ark_chip == ARK2000PV) {
+            threshold=12;
 	    val &= 0x40;
 	    val |= 0x10;	/* 32-deep FIFO. */
 	    val |= (threshold & 0x0E) >> 1;
@@ -396,8 +404,11 @@ static void ark_initializemode(unsigned char *moderegs,
 	if (dac16)
 	    moderegs[ARK_PIXEL_CLOCK_CONTROL] |= 0x04;	/* 16-bit */
     }
-    if (modetiming->flags & USEPROGRCLOCK)
-	moderegs[VGA_MISCOUTPUT] |= 0x0C;	/* External clock select. */
+    if (modetiming->flags & USEPROGRCLOCK) 
+        if(dac_used==&__svgalib_ICS_GENDAC_methods) {
+	    moderegs[VGA_MISCOUTPUT] &= ~0x0C;
+	    moderegs[VGA_MISCOUTPUT] |= 0x08;
+        } else moderegs[VGA_MISCOUTPUT] |= 0x0C;	/* External clock select. */
     else {
 	/* Program clock select. */
 	moderegs[VGA_MISCOUTPUT] &= ~0x0C;
@@ -438,10 +449,10 @@ static int ark_setmode(int mode, int prv_mode)
     unsigned char *moderegs;
 
 #ifdef SVGA_STYLE_320x200x256
-    if ((mode < G640x480x256 || mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	&& mode != G320x200x256)
 #else
-    if (mode < G640x480x256 || mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 #endif
 	/* Let the standard VGA driver set standard VGA modes. */
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
@@ -505,9 +516,9 @@ static void ark_setdisplaystart(int address)
 {
     outw(CRT_IC, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
     outw(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
-    inb(0x3da);			/* set ATC to addressing mode */
+    port_in(0x3da);			/* set ATC to addressing mode */
     outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
-    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
+    outb(ATT_IW, (port_in(ATT_R) & 0xf0) | ((address & 3) << 1));
     /* write sa0-1 to bits 1-2 */
     outb(__svgalib_CRT_I, 0x40);
     outb(__svgalib_CRT_D, (address & 0x1C0000) >> 18);	/* sa18-sa20 */
@@ -521,7 +532,7 @@ static void ark_setlogicalwidth(int width)
 {
     outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
     outb(__svgalib_CRT_I, 0x41);
-    outb(__svgalib_CRT_D, (inb(__svgalib_CRT_D) & ~0x08) | ((width >> 3) & 0x100));
+    outb(__svgalib_CRT_D, (port_in(__svgalib_CRT_D) & ~0x08) | ((width >> 3) & 0x100));
 }
 
 static int ark_linear(int op, int param)
@@ -542,12 +553,14 @@ static int ark_linear(int op, int param)
 	    size = 3;
 	}
 	__svgalib_outSR(0x12, (__svgalib_inSR(0x12) & ~0x03) | size);
+        is_linear=1;
 	return 0;
     }
     if (op == LINEAR_DISABLE) {
 	__svgalib_outSR(0x12, __svgalib_inSR(0x12) & ~0x03);	/* 64K map. */
 	__svgalib_outSR(0x13, 0xA0);	/* Revert to 0xA0000. */
 	__svgalib_outSR(0x14, 0x00);
+        is_linear=0;
 	return 0;
     }
     if (op == LINEAR_QUERY_BASE) {
@@ -572,9 +585,151 @@ static int ark_linear(int op, int param)
 	return -1;		/* Unknown function. */
 }
 
+static struct {
+    unsigned char c8;
+    unsigned short c15;
+    unsigned short c16;
+    unsigned int c32;
+} cursor_colors[16*2];
+
+static int pal=1, palette[768];
+
+static int findcolor(int rgb) {
+   int i,j,k,l=0;
+
+   if(pal)vga_getpalvec(0,256,palette);
+   pal=0;
+   k=0xffffff;
+   for(i=0;i<256;i++) {
+      j=((rgb&0xff)-(palette[i*3+2]<<2))*((rgb&0xff)-(palette[i*3+2]<<2))+
+        (((rgb>>8)&0xff)-(palette[i*3+1]<<2))*(((rgb>>8)&0xff)-(palette[i*3+1]<<2))+
+        (((rgb>>16)&0xff)-(palette[i*3]<<2))*(((rgb>>16)&0xff)-(palette[i*3]<<2));
+      if(j==0) {
+         return i;
+      }
+      if(j<k) {
+         k=j;
+         l=i;
+      }
+   }
+   return l;
+}
+
+static int cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
+    int i, j;
+    unsigned char *b3, *b4;
+    
+    switch(cmd){
+        case CURSOR_INIT:
+            return 0; /* does not work */
+        case CURSOR_HIDE:
+            __svgalib_outseq(0x20, __svgalib_inseq(0x20) & 0xe7);
+            break;
+        case CURSOR_SHOW:
+            switch(CI.colors) {
+                case 256:
+                    __svgalib_outseq(0x20, 0x18);
+                    break;
+                case 32768: case 65536:
+                    __svgalib_outseq(0x20, 0x19);
+                    break;
+                case 1<<24:
+                    __svgalib_outseq(0x20, 0x1a);
+                    break;
+            }
+            __svgalib_outseq(0x20, __svgalib_inseq(0x20) | 0x18);
+            break;
+        case CURSOR_POSITION:
+            __svgalib_outseq(0x2c, 0);
+            __svgalib_outseq(0x2d, 0);
+            __svgalib_outseq(0x22, p1&0xff);
+            __svgalib_outseq(0x21, (p1>>8)&0xff);
+            __svgalib_outseq(0x24, p2&0xff);
+            __svgalib_outseq(0x23, (p2>>8)&0xff);
+            break;
+        case CURSOR_SELECT:
+            i=ark_memory-(p1+1);
+            switch(CI.colors) {
+                case 256:
+                   __svgalib_outseq(0x26,cursor_colors[p1*2+1].c8);
+                   __svgalib_outseq(0x29,cursor_colors[p1*2].c8);
+                   break;
+                case 32768:
+                   __svgalib_outseq(0x27,cursor_colors[p1*2+1].c15>>8);
+                   __svgalib_outseq(0x26,cursor_colors[p1*2+1].c15&0xff);
+                   __svgalib_outseq(0x2a,cursor_colors[p1*2].c15>>8);
+                   __svgalib_outseq(0x29,cursor_colors[p1*2].c15&0xff);
+                   break;
+                case 65536:
+                   __svgalib_outseq(0x27,cursor_colors[p1*2+1].c16>>8);
+                   __svgalib_outseq(0x26,cursor_colors[p1*2+1].c16&0xff);
+                   __svgalib_outseq(0x2a,cursor_colors[p1*2].c16>>8);
+                   __svgalib_outseq(0x29,cursor_colors[p1*2].c16&0xff);
+                   break;
+                case (1<<24):
+                   __svgalib_outseq(0x28,cursor_colors[p1*2+1].c32>>16);
+                   __svgalib_outseq(0x27,(cursor_colors[p1*2+1].c32>>8)&0xff);
+                   __svgalib_outseq(0x26,cursor_colors[p1*2+1].c32&0xff);
+                   __svgalib_outseq(0x2b,cursor_colors[p1*2].c32>>16);
+                   __svgalib_outseq(0x2a,(cursor_colors[p1*2].c32>>8)&0xff);
+                   __svgalib_outseq(0x29,cursor_colors[p1*2].c32&0xff);
+                   break;
+            }
+            __svgalib_outseq(0x25, (i>>8)&0x3f);
+            __svgalib_outseq(0x25, 63);
+            break;
+        case CURSOR_IMAGE:
+            i=ark_memory*1024-(p1+1)*1024;
+            i=2048*1024-256;
+            b3=(unsigned char *)p5;
+            b4=LINEAR_POINTER+i;
+            if (!is_linear){
+	        int size;
+	        __svgalib_outSR(0x13, ark_baseaddress >> 16);
+	        __svgalib_outSR(0x14, ark_baseaddress >> 24);
+	        switch (ark_memory) {
+	        case 1024:
+	            size = 1;
+	            break;
+	        case 2048:
+	            size = 2;
+	            break;
+	        case 4096:
+	        default:
+	            size = 3;
+	        }
+	        __svgalib_outSR(0x12, (__svgalib_inSR(0x12) & ~0x03) | size);
+            };
+            cursor_colors[p1*2].c8=findcolor(p3);
+            cursor_colors[p1*2].c32=p3;
+            cursor_colors[p1*2].c16=((p3&0xf80000)>>8)|((p3&0xfc00)>>5)|((p3&0xf8)>>3);
+            cursor_colors[p1*2].c15=((p3&0xf80000)>>9)|((p3&0xf800)>>5)|((p3&0xf8)>>3);
+            cursor_colors[p1*2+1].c8=findcolor(p4);
+            cursor_colors[p1*2+1].c32=p4;
+            cursor_colors[p1*2+1].c16=((p4&0xf80000)>>8)|((p4&0xfc00)>>5)|((p4&0xf8)>>3);
+            cursor_colors[p1*2+1].c15=((p4&0xf80000)>>9)|((p4&0xf800)>>5)|((p4&0xf8)>>3);
+            switch(p2) {
+                case 0:
+                    for(j=0;j<32;j++) for(i=0;i<4;i++){
+                        *b4=*b3;
+                        b4++;
+                        *b4=*(b3+32);
+                        b4++;
+                        b3++;
+                    }
+                break;
+            }
+            if (!is_linear){
+	        __svgalib_outseq(0x12, __svgalib_inseq(0x12) & ~0x03);	/* 64K map. */
+	        __svgalib_outseq(0x13, 0xA0);	/* Revert to 0xA0000. */
+                __svgalib_outseq(0x14, 0x00);
+            };
+            break;
+    }
+    return 0;
+}       
 
 /* Function table (exported) */
-
 DriverSpecs __svgalib_ark_driverspecs =
 {
     ark_saveregs,		/* saveregs */
@@ -601,6 +756,7 @@ DriverSpecs __svgalib_ark_driverspecs =
     ark_linear,
     NULL,			/* accelspecs */
     NULL,                       /* Emulation */
+    cursor,
 };
 
 /* ARK-specific config file options. */
@@ -616,7 +772,7 @@ static char *ark_config_options[] =
     "clocks", "ramdac", "dacspeed", NULL
 };
 
-static char *ark_process_option(int option, int mode)
+static char *ark_process_option(int option, int mode, char** dummy)
 {
 /*
  * option is the number of the option string in ark_config_options,
@@ -648,6 +804,10 @@ static char *ark_process_option(int option, int mode)
 	DacMethods *old_dac_used;
 	ptr = strtok(NULL, " ");
 	old_dac_used = dac_used;
+#ifdef INCLUDE_ICS_GENDAC_DAC
+	if (strcasecmp(ptr, "ICSgendac") == 0)
+	    dac_used = &__svgalib_ICS_GENDAC_methods;
+#endif
 #ifdef INCLUDE_SIERRA_DAC
 	if (strcasecmp(ptr, "Sierra32K") == 0)
 	    dac_used = &__svgalib_Sierra_32K_methods;
@@ -682,6 +842,22 @@ static char *ark_process_option(int option, int mode)
 
 /* Initialize driver (called after detection) */
 
+static DacMethods *dacs_to_probe[] =
+{
+#ifdef INCLUDE_ICS_GENDAC_DAC_TEST
+    &__svgalib_ICS_GENDAC_methods,
+#endif
+#ifdef INCLUDE_ATT20C490_DAC_TEST
+    &__svgalib_ATT20C490_methods,
+#endif
+#ifdef INCLUDE_ATT20C498_DAC_TEST
+    &__svgalib_ATT20C498_methods,
+#endif
+#ifdef INCLUDE_SIERRA_DAC_TEST
+    &__svgalib_Sierra_32K_methods,
+#endif
+    NULL};
+
 static char *ark_chipname[] =
 {"ARK1000PV", "ARK2000PV"};
 
@@ -729,14 +905,20 @@ static int ark_init(int force, int par1, int par2)
     /* Process ARK-specific config file options. */
     __svgalib_read_options(ark_config_options, ark_process_option);
 
+
+    if (dac_used == NULL) {
+        dac_used=__svgalib_probeDacs(dacs_to_probe);
+    }
+
     if (dac_used == NULL) {
 	/* Not supported. */
-	printf("svgalib: ark: Assuming normal VGA DAC.\n");
+        printf("svgalib: ark: Assuming normal VGA DAC.\n");
+
 #ifdef INCLUDE_NORMAL_DAC
-	dac_used = &__svgalib_normal_dac_methods;
+        dac_used = &__svgalib_normal_dac_methods;
 #else
-	printf("svgalib: Alas, normal VGA DAC support is not compiled in, goodbye.\n");
-	return 1;
+        printf("svgalib: Alas, normal VGA DAC support is not compiled in, goodbye.\n");
+        return 1;
 #endif
     }
     dac_used->qualifyCardSpecs(cardspecs, dac_speed);
@@ -802,12 +984,14 @@ static int ark_init(int force, int par1, int par2)
 	}
 	printf("svgalib: Using ARK driver (%s, %dK, %s).",
 	       ark_chipname[ark_chip], ark_memory, bustype);
+#if 0
 	if (ark_bus == PCI)
 	    printf(" Base address = 0x%08X.", ark_baseaddress);
+#endif
 	printf("\n");
     }
     __svgalib_driverspecs = &__svgalib_ark_driverspecs;
-    __svgalib_mmio_base = 0xb80000;
+    __svgalib_mmio_base = 0xb8000;
     __svgalib_mmio_size = 4096;
     __svgalib_banked_mem_base=0xa0000;
     __svgalib_banked_mem_size=0x10000;
@@ -945,7 +1129,7 @@ static int ark_init(int force, int par1, int par2)
 #define SETCOMMAND(p) \
 	*(unsigned short *)(MMIO_POINTER + COMMAND) = p;
 
-#define COPISBUSY() (inb(0x3CB) & 0x40)
+#define COPISBUSY() (port_in(0x3CB) & 0x40)
 
 #define WAITUNTILFINISHED() \
 	for (;;) { \
