@@ -141,12 +141,8 @@ static int banshee_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
-    if(modeinfo->greenWeight==5) return 0;
-
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
@@ -155,6 +151,11 @@ static int banshee_modeavailable(int mode)
 
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
+    if((modeinfo->bitsPerPixel==16)&&(modeinfo->greenWeight==5)) {
+	free(modeinfo);
+        return 0;
+    }
+
     modetiming = malloc(sizeof(ModeTiming));
     if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
@@ -167,7 +168,7 @@ static int banshee_modeavailable(int mode)
     return SVGADRV;
 }
 
-static unsigned comp_lmn(unsigned clock) ;
+static unsigned comp_lmn(int freq);
 /* Set a mode */
 
 /* Local, called by banshee_setmode(). */
@@ -197,6 +198,7 @@ static void banshee_initializemode(unsigned char *moderegs,
                             ((hss&0x100)>>2) |
                             ((hse&0x20)<<2) ; 
 
+
     vd    = modetiming->CrtcVDisplay - 1;
     vt = modetiming->CrtcVTotal - 2;
     vbs  = modetiming->CrtcVSyncStart - 1;
@@ -271,8 +273,7 @@ static int banshee_setmode(int mode, int prv_mode)
     ModeTiming modetiming;
     ModeInfo *modeinfo;
 
-    if ((mode < G640x480x256 /*&& mode != G320x200x256*/)
-	|| mode == G720x348x2) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
 
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
@@ -325,6 +326,7 @@ static int banshee_test(void)
 
    found=(!__svgalib_pci_find_vendor_vga(0x121a,buf,0))&&
           (((buf[0]>>16)==0x0003)||
+          ((buf[0]>>16)==0x0009)||
           ((buf[0]>>16)==0x0005));
     
    if(found)banshee_init(0,0,0); 
@@ -337,13 +339,9 @@ static int banshee_test(void)
 
 static void banshee_setdisplaystart(int address)
 { 
-  address=address >> 2;
-  outw(CRT_IC, (address & 0x00FF00) | 0x0C);
-  outw(CRT_IC, ((address & 0x00FF) << 8) | 0x0D);
-  __svgalib_outcrtc(0x0c,(address & 0x00FF00)>>8);
-  __svgalib_outcrtc(0x0d,address & 0x00FF);
-  outl(banshee_io_base+0xe4,address>>2);
-
+  __svgalib_outcrtc(0x0c,((address>>2) & 0x00FF00)>>8);
+  __svgalib_outcrtc(0x0d,(address>>2) & 0x00FF);
+  outl(banshee_io_base+0xe4,address);
 }
 
 
@@ -418,16 +416,16 @@ static int banshee_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
                         /*change endianess */
                         l1=(l1<<24)|(l1>>24)|((l1>>8)&0xff00)|((l1<<8)&0xff0000);
                         l2=(l2<<24)|(l2>>24)|((l2>>8)&0xff00)|((l2<<8)&0xff0000);
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j)=l1;
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+4)=0;
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+8)=l2;
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+12)=0;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j)=l1;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+4)=0;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+8)=l2;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+12)=0;
                     }
                     for(j=32;j<64;j++) {
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j)=0;
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+4)=0;
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+8)=0;
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+12)=0;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j)=0;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+4)=0;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+8)=0;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+12)=0;
                     }
                 break;
             }
@@ -483,13 +481,14 @@ static int banshee_init(int force, int par1, int par2)
     
     found=(!__svgalib_pci_find_vendor_vga(0x121a,buf,0))&&
             (((buf[0]>>16)==0x0003)||
+            ((buf[0]>>16)==0x0009)||
             ((buf[0]>>16)==0x0005));
     
     if (found){
        banshee_linear_base=buf[5]&0xffffff00;
        banshee_io_base=buf[6]&0xff00;
-       io_reloc=banshee_io_base-0x300;
-       rel_io_mapio();
+       __svgalib_io_reloc=banshee_io_base-0x300;
+       __svgalib_rel_io_mapio();
     };
 
     if(banshee_memory==0) {
@@ -516,13 +515,13 @@ static int banshee_init(int force, int par1, int par2)
     
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = banshee_memory;
-    cardspecs->maxPixelClock4bpp = 250000;	
-    cardspecs->maxPixelClock8bpp = 250000;	
-    cardspecs->maxPixelClock16bpp = 250000;	
-    cardspecs->maxPixelClock24bpp = 250000;
-    cardspecs->maxPixelClock32bpp = 250000;
-    cardspecs->flags = INTERLACE_DIVIDE_VERT | CLOCK_PROGRAMMABLE;
-    cardspecs->maxHorizontalCrtc = 2040;
+    cardspecs->maxPixelClock4bpp = 270000;	
+    cardspecs->maxPixelClock8bpp = 270000;	
+    cardspecs->maxPixelClock16bpp = 270000;	
+    cardspecs->maxPixelClock24bpp = 270000;
+    cardspecs->maxPixelClock32bpp = 270000;
+    cardspecs->flags = CLOCK_PROGRAMMABLE;
+    cardspecs->maxHorizontalCrtc = 4088;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->mapClock = banshee_map_clock;
@@ -536,45 +535,39 @@ static int banshee_init(int force, int par1, int par2)
     return 0;
 }
 
-#define WITHIN(v,c1,c2) (((v) >= (c1)) && ((v) <= (c2)))
+#define REFFREQ 14318.18
 
 static unsigned
-comp_lmn(unsigned clock)
+comp_lmn(int freq)
 {
-  int     n, m, l;
-  double  fvco;
-  double  fout;
-  double  fmax;
-  double  fref;
-  double  fvco_goal;
+  int m, n, k, best_m, best_n, best_k, f_cur, best_error;
 
-    fmax = 162000.0;
-
-  fref = 14318.18; 
-
-  for (m = 1; m <= 61; m++)
-  {
-    for (l = 3; l >= 0; l--)
-    {
-      for (n = 1; n <= 253; n++)
-      {
-        fout = ((double)(n + 2) * fref)/((double)(m + 2) * (1 << l));
-        fvco_goal = (double)clock * (double)(1 << l);
-        fvco = fout * (double)(1 << l);
-        if (!WITHIN(fvco, 0.995*fvco_goal, 1.005*fvco_goal))
-          continue;
-#if 0
-        if (!WITHIN(fvco, 32000.0, fmax))
-          continue;
-#endif
-#if 0
-printf("clock=%i n=%i m=%i l=%i",clock,n,m,l);
-#endif
-        return (n << 8) | (m<<2) | l;
+  best_error=freq;
+  best_n=best_m=best_k=0;
+  for (n=1; n<256; n++) {
+    f_cur=REFFREQ*(n+2);
+    if (f_cur<freq) {
+      f_cur=f_cur/3;
+      if (freq-f_cur<best_error) {
+        best_error=freq-f_cur;
+        best_n=n;
+        best_m=1;
+        best_k=0;
+        continue;
+      }
+    }
+    for (m=1; m<64; m++) {
+      for (k=0; k<4; k++) {
+        f_cur=REFFREQ*(n+2)/(m+2)/(1<<k);
+        if (abs(f_cur-freq)<best_error) {
+          best_error=abs(f_cur-freq);
+          best_n=n;
+          best_m=m;
+          best_k=k;
+	}
       }
     }
   }
-printf("BANSHEE driver: Can't do clock=%i\n",clock);
-  return 0;
+  return (best_n << 8) | (best_m<<2) | best_k;
 }
 
