@@ -129,7 +129,7 @@
 static int cirrus_init(int, int, int);
 static void cirrus_unlock(void);
 
-void __svgalib_cirrusaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
+static void __svgalib_cirrusaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
 
 static int cirrus_memory;
 static int cirrus_chiptype;
@@ -283,7 +283,8 @@ static int cirrus_saveregs(unsigned char regs[])
 	regs[CIRRUS_MCLKREGISTER] = __svgalib_inSR(0x1F);
 
     /* Save Hicolor DAC register. */
-    if (CHIP_HAS_HIDDENDAC()) {
+    /* My 5446 works without this. With this 8 bpp and text modes do not work. - MZ */
+    if (CHIP_HAS_HIDDENDAC() && CHECKCHIPNOTEQUAL(CLGD5436)) {
 	outb(0x3c6, 0);
 	outb(0x3c6, 0xff);
 	inb(0x3c6);
@@ -391,16 +392,15 @@ static int cirrus_modeavailable(int mode)
     ModeInfo *modeinfo;
 
     /* CLGD7548 mode 13 is broken. Remove this if when supported */
-    if (mode == G320x200x256 && cirrus_chiptype == CLGD7548)
- return 0;
+    if (mode == G320x200x256 && cirrus_chiptype == CLGD7548) return 0;
 
-    if ((mode < G640x480x256 && mode != G320x200x256)
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode) && mode != G320x200x256)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
-    if (cirrus_memory * 1024 < info->ydim * info->xbytes)
+    if (cirrus_memory * 1024 < info->ydim * info->xbytes) {
 	return 0;
+    }
 
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
@@ -599,8 +599,7 @@ static int cirrus_setmode(int mode, int prv_mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    if ((mode < G640x480x256 && mode != G320x200x256)
-	|| mode == G720x348x2) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode) && mode != G320x200x256) {
 	__svgalib_clear_accelspecs(__svgalib_driverspecs->accelspecs);
 	/* Let the standard VGA driver set standard VGA modes */
 	/* But first reset an Cirrus extended register that */
@@ -1495,7 +1494,7 @@ static int cirrus_bitblt_pixelwidth;
 /* Foreground color is not preserved on 5420/2/4/6/8. */
 static int cirrus_accel_foreground_color;
 
-void __svgalib_cirrusaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels)
+static void __svgalib_cirrusaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels)
 {
     /* [Setup accelerator screen pitch] */
     /* [Prepare any required off-screen space] */
@@ -1567,7 +1566,7 @@ static void mmio_set_foreground_color(int fg)
 	if (__svgalib_accel_mode & BLITS_IN_BACKGROUND) \
 		MMIOWAITUNTILFINISHED();
 
-void __svgalib_cirrusaccel_FillBox(int x, int y, int width, int height)
+static void __svgalib_cirrusaccel_FillBox(int x, int y, int width, int height)
 {
     int destaddr;
     destaddr = BLTBYTEADDRESS(x, y);
@@ -1586,7 +1585,7 @@ void __svgalib_cirrusaccel_FillBox(int x, int y, int width, int height)
     SETFOREGROUNDCOLOR(0x00);
 }
 
-void __svgalib_cirrusaccel_mmio_FillBox(int x, int y, int width, int height)
+static void __svgalib_cirrusaccel_mmio_FillBox(int x, int y, int width, int height)
 {
     int destaddr;
     destaddr = BLTBYTEADDRESS(x, y);
@@ -1602,7 +1601,7 @@ void __svgalib_cirrusaccel_mmio_FillBox(int x, int y, int width, int height)
 	MMIOWAITUNTILFINISHED();
 }
 
-void __svgalib_cirrusaccel_ScreenCopy(int x1, int y1, int x2, int y2, int width,
+static void __svgalib_cirrusaccel_ScreenCopy(int x1, int y1, int x2, int y2, int width,
 			    int height)
 {
     int srcaddr, destaddr, dir;
@@ -1627,7 +1626,7 @@ void __svgalib_cirrusaccel_ScreenCopy(int x1, int y1, int x2, int y2, int width,
 	WAITUNTILFINISHED();
 }
 
-void __svgalib_cirrusaccel_mmio_ScreenCopy(int x1, int y1, int x2, int y2, int width,
+static void __svgalib_cirrusaccel_mmio_ScreenCopy(int x1, int y1, int x2, int y2, int width,
 				 int height)
 {
     int srcaddr, destaddr, dir;
@@ -1652,12 +1651,12 @@ void __svgalib_cirrusaccel_mmio_ScreenCopy(int x1, int y1, int x2, int y2, int w
 	MMIOWAITUNTILFINISHED();
 }
 
-void __svgalib_cirrusaccel_SetFGColor(int fg)
+static void __svgalib_cirrusaccel_SetFGColor(int fg)
 {
     cirrus_accel_foreground_color = fg;
 }
 
-void __svgalib_cirrusaccel_mmio_SetFGColor(int fg)
+static void __svgalib_cirrusaccel_mmio_SetFGColor(int fg)
 {
     MMIOFINISHBACKGROUNDBLITS();
     mmio_set_foreground_color(fg);
@@ -1672,19 +1671,19 @@ static unsigned char cirrus_rop_map[] =
     0x0B			/* ROP_INVERT */
 };
 
-void __svgalib_cirrusaccel_SetRasterOp(int rop)
+static void __svgalib_cirrusaccel_SetRasterOp(int rop)
 {
     FINISHBACKGROUNDBLITS();
     SETROP(cirrus_rop_map[rop]);
 }
 
-void __svgalib_cirrusaccel_mmio_SetRasterOp(int rop)
+static void __svgalib_cirrusaccel_mmio_SetRasterOp(int rop)
 {
     MMIOFINISHBACKGROUNDBLITS();
     MMIOSETROP(cirrus_rop_map[rop]);
 }
 
-void __svgalib_cirrusaccel_SetTransparency(int mode, int color)
+static void __svgalib_cirrusaccel_SetTransparency(int mode, int color)
 {
     FINISHBACKGROUNDBLITS();
     if (mode == DISABLE_TRANSPARENCY_COLOR) {
@@ -1707,7 +1706,7 @@ void __svgalib_cirrusaccel_SetTransparency(int mode, int color)
     __svgalib_accel_bitmaptransparency = 1;
 }
 
-void __svgalib_cirrusaccel_Sync(void)
+static void __svgalib_cirrusaccel_Sync(void)
 {
     WAITUNTILFINISHED();
 }
