@@ -171,7 +171,7 @@ static int setmode(int mode, int prv_mode)
 
 static void setdisplaystart(int address)
 {
-    inb(__svgalib_IS1_R);
+    port_in(__svgalib_IS1_R);
     __svgalib_delay();
     outb(ATT_IW, 0x13 + 0x20);
     __svgalib_delay();
@@ -195,11 +195,11 @@ static int ega_test(void)
     unsigned char save, back;
 
     /* Check if a DAC is present */
-    save = inb(PEL_IW);
+    save = port_in(PEL_IW);
     __svgalib_delay();
     outb(PEL_IW, ~save);
     __svgalib_delay();
-    back = inb(PEL_IW);
+    back = port_in(PEL_IW);
     __svgalib_delay();
     outb(PEL_IW, save);
     save = ~save;
