@@ -217,7 +217,7 @@ static ModeTable No_Modes = END_OF_MODE_TABLE;
 
 #else				/* !defined(DYNAMIC) */
 
-#include "et4000.regs"
+#include "config/et4000.regs"
 
 #ifdef DAC_TYPE
 static int et4000_dac = DAC_TYPE;
@@ -305,11 +305,11 @@ static int et4000_clocks(int clk)
     int res;
 
     /* get actual clock */
-    res = (inb(MIS_R) >> 2) & 3;	/* bit 0..1 */
+    res = (port_in(MIS_R) >> 2) & 3;	/* bit 0..1 */
     outb(__svgalib_CRT_I, 0x34);
-    res |= (inb(__svgalib_CRT_D) & 2) << 1;	/* bit 2 */
+    res |= (port_in(__svgalib_CRT_D) & 2) << 1;	/* bit 2 */
     outb(SEQ_I, 0x07);
-    temp = inb(SEQ_D);
+    temp = port_in(SEQ_D);
     if (temp & 0x41 == 0x41)
 	res |= 0x10;		/* bit 3..4 */
     else
@@ -319,13 +319,13 @@ static int et4000_clocks(int clk)
 	clk = CLOCKS - 1;
     if (clk >= 0) {
 	/* Set clock */
-	temp = inb(MIS_R) & 0xF3;
+	temp = port_in(MIS_R) & 0xF3;
 	outb(MIS_W, temp | ((clk & 3) << 2));
 	outb(__svgalib_CRT_I, 0x34);
-	temp = inb(__svgalib_CRT_D) & 0xFD;
+	temp = port_in(__svgalib_CRT_D) & 0xFD;
 	outb(__svgalib_CRT_D, temp | ((clk & 4) >> 1));
 	outb(SEQ_I, 0x07);
-	temp = inb(SEQ_D) & 0xBE;
+	temp = port_in(SEQ_D) & 0xBE;
 	temp |= (clk & 0x10 ? 0x41 : 0x00);
 	temp |= (clk & 0x08) << 3;
 	outb(SEQ_D, temp);
@@ -342,17 +342,17 @@ static int measure(int frames)
     unsigned counter;
     __asm__ volatile (
 			 "	xorl	%0,%0		\n"
-			 "__W1:	inb	%1,%%al		\n"
+			 "__W1:	port_in	%1,%%al		\n"
 			 "	testb	$8,%%al		\n"
 			 "	jne	__W1		\n"
-			 "__W2:	inb	%1,%%al		\n"
+			 "__W2:	port_in	%1,%%al		\n"
 			 "	testb	$8,%%al		\n"
 			 "	je	__W2		\n"
-			 "__L1:	inb	%1,%%al		\n"
+			 "__L1:	port_in	%1,%%al		\n"
 			 "	incl	%0		\n"
 			 "	testb	$8,%%al		\n"
 			 "	jne	__L1		\n"
-			 "__L2:	inb	%1,%%al		\n"
+			 "__L2:	port_in	%1,%%al		\n"
 			 "	incl	%0		\n"
 			 "	testb	$8,%%al		\n"
 			 "	je	__L2		\n"
@@ -377,7 +377,7 @@ static int measure_clk(int clk)
     if ((state = SCREENON))
 	vga_screenoff();
     if (act == 0) {
-	unsigned char save = inb(MIS_R);
+	unsigned char save = port_in(MIS_R);
 	outb(MIS_W, (save & 0xF3) | 0x04);
 	act = measure(FRAMES);
 	outb(MIS_W, save);
@@ -467,16 +467,16 @@ static int et4000_saveregs(unsigned char regs[])
 
     if (et4000_extdac) {
 	_ramdac_dactocomm();
-	regs[EXT + 13] = inb(PEL_MSK);
+	regs[EXT + 13] = port_in(PEL_MSK);
 	_ramdac_dactocomm();
 	outb(PEL_MSK, regs[EXT + 13] | 0x10);
 	_ramdac_dactocomm();
-	inb(PEL_MSK);
+	port_in(PEL_MSK);
 	outb(PEL_MSK, 3);		/* write index low */
 	outb(PEL_MSK, 0);		/* write index high */
-	regs[EXT + 14] = inb(PEL_MSK);	/* primary ext. pixel select */
-	regs[EXT + 15] = inb(PEL_MSK);	/* secondary ext. pixel select */
-	regs[EXT + 16] = inb(PEL_MSK);	/* PLL control register */
+	regs[EXT + 14] = port_in(PEL_MSK);	/* primary ext. pixel select */
+	regs[EXT + 15] = port_in(PEL_MSK);	/* secondary ext. pixel select */
+	regs[EXT + 16] = port_in(PEL_MSK);	/* PLL control register */
 	_ramdac_dactocomm();
 	outb(PEL_MSK, regs[EXT + 13]);
 	return 17;
@@ -544,7 +544,7 @@ static void et4000_setregs(const unsigned char regs[], int mode)
 	_ramdac_dactocomm();
 	outb(PEL_MSK, regs[EXT + 13] | 0x10);
 	_ramdac_dactocomm();
-	inb(PEL_MSK);
+	port_in(PEL_MSK);
 	outb(PEL_MSK, 3);		/* write index low */
 	outb(PEL_MSK, 0);		/* write index high */
 	outb(PEL_MSK, regs[EXT + 14]);	/* primary ext. pixel select */
@@ -630,7 +630,7 @@ static int et4000_setmode(int mode, int prv_mode)
 	/* least on some cards). */
 	et4000_unlock();
 	outb(__svgalib_CRT_I, 0x34);
-	i = inb(__svgalib_CRT_D);
+	i = port_in(__svgalib_CRT_D);
 	if ((i & 0x02) == 0x02)
 	    outb(__svgalib_CRT_D, (i & 0xfd)); /* DS */
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
@@ -685,9 +685,9 @@ static void et4000_setlinear(int addr)
     et4000_unlock();
     outb(CRT_IC, 0x36);
     if (addr)
-	outb(CRT_DC, inb(CRT_DC) | 16);	/* enable linear mode */
+	outb(CRT_DC, port_in(CRT_DC) | 16);	/* enable linear mode */
     else
-	outb(CRT_DC, inb(CRT_DC) & ~16);	/* disable linear mode */
+	outb(CRT_DC, port_in(CRT_DC) & ~16);	/* disable linear mode */
     outb(CRT_IC, 0x30);
     outb(CRT_DC, addr);
     et4000_lock();
@@ -745,7 +745,7 @@ static void et4000_setpage(int page)
     if (et4000_chiptype >= CHIP_ET4000W32i) {
 	/* Write page4-5 to bits 0-1 of ext. bank register, */
 	/* and to bits 4-5. */
-	outb(0x3cb, (inb(0x3cb) & ~0x33) | (page >> 4) | (page & 0x30));
+	outb(0x3cb, (port_in(0x3cb) & ~0x33) | (page >> 4) | (page & 0x30));
     }
 }
 
@@ -758,7 +758,7 @@ static void et4000_setrdpage(int page)
     port_out(last_page, SEG_SELECT);
     if (et4000_chiptype >= CHIP_ET4000W32i) {
 	/* Write page4-5 to bits 4-5 of ext. bank register. */
-	outb(0x3cb, (inb(0x3cb) & ~0x30) | (page & 0x30));
+	outb(0x3cb, (port_in(0x3cb) & ~0x30) | (page & 0x30));
     }
 }
 
@@ -770,7 +770,7 @@ static void et4000_setwrpage(int page)
     port_out(last_page, SEG_SELECT);
     if (et4000_chiptype >= CHIP_ET4000W32i) {
 	/* Write page4-5 to bits 0-1 of ext. bank register. */
-	outb(0x3cb, (inb(0x3cb) & ~0x03) | (page >> 4));
+	outb(0x3cb, (port_in(0x3cb) & ~0x03) | (page >> 4));
     }
 }
 
@@ -783,17 +783,17 @@ static void et4000_setdisplaystart(int address)
 {
     outw(CRT_IC, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
     outw(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
-    inb(0x3da);			/* set ATC to addressing mode */
+    port_in(0x3da);			/* set ATC to addressing mode */
     outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
-    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
+    outb(ATT_IW, (port_in(ATT_R) & 0xf0) | ((address & 3) << 1));
     /* write sa0-1 to bits 1-2 */
     outb(CRT_IC, 0x33);
     if (et4000_chiptype >= CHIP_ET4000W32i)
 	/* write sa18-21 to bit 0-3 */
-	outb(CRT_DC, (inb(CRT_DC) & 0xf0) | ((address & 0x3c0000) >> 18));
+	outb(CRT_DC, (port_in(CRT_DC) & 0xf0) | ((address & 0x3c0000) >> 18));
     else
 	/* write sa18-19 to bit 0-3 */
-	outb(CRT_DC, (inb(CRT_DC) & 0xfc) | ((address & 0xc0000) >> 18));
+	outb(CRT_DC, (port_in(CRT_DC) & 0xfc) | ((address & 0xc0000) >> 18));
 }
 
 
@@ -804,7 +804,7 @@ static void et4000_setlogicalwidth(int width)
 {
     outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
     outb(CRT_IC, 0x3f);
-    outb(CRT_DC, (inb(CRT_DC) & 0x7f)
+    outb(CRT_DC, (port_in(CRT_DC) & 0x7f)
 	 | ((width & 0x800) >> 5));	/* write lw12 to bit 7 */
 }
 
@@ -833,7 +833,7 @@ static int et4000_ext_set(unsigned what, va_list params)
 	    CRITICAL = 1;
 	    vga_lockvc();
 	    _ramdac_dactocomm();
-	    cmd = inb(PEL_MSK);
+	    cmd = port_in(PEL_MSK);
 	    _ramdac_dactocomm();
 	    if (ext_settings & VGA_CLUT8)
 		cmd |= 2;
@@ -926,20 +926,20 @@ static int testdac(void)
 
     retval = 0;			/* default to no fancy dac */
     _ramdac_dactopel();
-    x = inb(PEL_MSK);
+    x = port_in(PEL_MSK);
     do {			/* wait for repeat of value */
 	y = x;
-	x = inb(PEL_MSK);
+	x = port_in(PEL_MSK);
     } while (x != y);
     z = x;
-    inb(PEL_IW);
-    inb(PEL_MSK);
-    inb(PEL_MSK);
-    inb(PEL_MSK);
-    x = inb(PEL_MSK);
+    port_in(PEL_IW);
+    port_in(PEL_MSK);
+    port_in(PEL_MSK);
+    port_in(PEL_MSK);
+    x = port_in(PEL_MSK);
     y = 8;
     while ((x != 0x8e) && (y > 0)) {
-	x = inb(PEL_MSK);
+	x = port_in(PEL_MSK);
 	y--;
     }
     if (x == 0x8e) {
@@ -948,13 +948,13 @@ static int testdac(void)
 	_ramdac_dactopel();
     } else {
 	_ramdac_dactocomm();
-	oldcommreg = inb(PEL_MSK);
+	oldcommreg = port_in(PEL_MSK);
 	_ramdac_dactopel();
-	oldpelreg = inb(PEL_MSK);
+	oldpelreg = port_in(PEL_MSK);
 	x = oldcommreg ^ 0xFF;
 	outb(PEL_MSK, x);
 	_ramdac_dactocomm();
-	v = inb(PEL_MSK);
+	v = port_in(PEL_MSK);
 	if (v != x) {
 	    v = _ramdac_setcomm(x = oldcommreg ^ 0x60);
 	    /* We have a Sierra SC11486 */
@@ -962,11 +962,11 @@ static int testdac(void)
 
 	    if ((x & 0xe0) == (v & 0xe0)) {
 		/* We have a Sierra 32k/64k */
-		x = inb(PEL_MSK);
+		x = port_in(PEL_MSK);
 		_ramdac_dactopel();
 		retval = 1 | 2;
 
-		if (x == inb(PEL_MSK)) {
+		if (x == port_in(PEL_MSK)) {
 		    /* We have an ATT 20c491 or 20c492 */
 		    retval = 1 | 8;
 		    if (_ramdac_setcomm(0xFF) != 0xFF) {
@@ -977,7 +977,7 @@ static int testdac(void)
 			_ramdac_setcomm(oldcommreg | 0x10);
 			outb(PEL_MSK, 0);
 			outb(PEL_MSK, 0);
-			if (inb(PEL_MSK) == 0x44) {
+			if (port_in(PEL_MSK) == 0x44) {
 			    /* it's a 170x */
 			    /* Another read from PEL_MSK gets the chiptype, 0x02 == 1702. */
 			    retval = 1 | 64;
@@ -1024,9 +1024,9 @@ static int et4000_init(int force, int par1, int par2)
     /* Determine the ET4000 chip type. */
 
     /* Test for ET4000/W32. */
-    old = inb(0x3cb);
+    old = port_in(0x3cb);
     outb(0x3cb, 0x33);
-    new = inb(0x3cb);
+    new = port_in(0x3cb);
     outb(0x3cb, old);
     if (new != 0x33)
 	et4000_chiptype = CHIP_ET4000;
@@ -1038,7 +1038,7 @@ static int et4000_init(int force, int par1, int par2)
 	    et4000_chiptype = CHIP_ET4000W32;
 	else {
 	    outb(0x217a, 0xec);
-	    val = inb(0x217b) >> 4;
+	    val = port_in(0x217b) >> 4;
 	    switch (val) {
 	    case 0:
 		et4000_chiptype = CHIP_ET4000W32;
@@ -1063,7 +1063,7 @@ static int et4000_init(int force, int par1, int par2)
 	et4000_memory = par1;
     else {
 	outb(CRT_IC, 0x37);
-	value = inb(CRT_DC);
+	value = port_in(CRT_DC);
 	et4000_memory = (value & 0x08) ? 256 : 64;
 	switch (value & 3) {
 	case 2:
@@ -1076,7 +1076,7 @@ static int et4000_init(int force, int par1, int par2)
 	if (value & 0x80)
 	    et4000_memory *= 2;
 	outb(CRT_IC, 0x32);
-	if (inb(CRT_DC) & 0x80) {
+	if (port_in(CRT_DC) & 0x80) {
 	    /* Interleaved memory on ET4000/W32i/p: */
 	    /* Multiply by 2. */
 	    et4000_memory *= 2;
