@@ -1,12 +1,14 @@
-                       /*  ET6000 driver  Don Secrest  version 1.06     */
-                       /*  version using svgalib_pci_find July 9 1999   */
-#include <stdlib.h>    /*  Testing modeline update. */
+                       /*  ET6000 driver  Don Secrest  version 1.4         */
+                       /*  version using svgalib_pci_find Feb 10, 2001     */
+#include <stdlib.h>    /*  Testing modeline update. Works for linux-2.2.15 */
 #include <stdio.h>
 #include <unistd.h>
 #include <stdarg.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/types.h>
+#include <sys/mman.h>
+#include <asm/page.h>
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
@@ -22,14 +24,14 @@ FILE *outf;
 static int compareregs(void);
 static int textset = 0;
 static void prtmodeinfo(vga_modeinfo *m);
-static char *not_written = "ET6000 subroutine %s is not written yet.\n";
+/* static char *not_written = "ET6000 subroutine %s is not written yet.\n"; */
 static unsigned char *textstore = 0;
 static unsigned char *vgatextstore = 0;
 void fdumpregs(FILE *fd,unsigned char regs[],int n);
-#define DNOTW(a) fprintf(outf,not_written,a)
+/* #define DNOTW(a) fprintf(outf,not_written,a) */
 #define DPRT(a) fprintf(outf,a)
 #else
-#define DNOTW(a)
+/* #define DNOTW(a) */
 #define DPRT(a)
 #endif  /* DB6K */
 
@@ -40,7 +42,6 @@ FILE *fdm;
 #define SEG_SELECT 0x3CD
 
 static unsigned long base1;
-
 /* The following are set by init so that they may be restored to their  */
 /* state in setmode.                                                    */
 static unsigned char pci40std,pci42std,pci57std,pci58std,pci59std=0;
@@ -56,9 +57,9 @@ static int ET6000_TOTAL_REGS = VGA_MISCOUTPUT + 14;
 static char mem_type;
 static int et6000_memory;
 static int et6000_linear_base;
-static int et6000_interlaced(int mode);
+static int et6000_interlaced(int mode); 
 static unsigned comp_lmn(unsigned clock,float fac);
-static int et6000_chiptype = 0;
+static int et6000_chiptype = 0; 
 static int x3_4,x3_5,x3_8 = 0,x3_a; /* On init these get set according */
 				    /*  to color */
 union longword {
@@ -67,6 +68,13 @@ union longword {
   unsigned long l;
 };
 
+static unsigned int cursor_colors[16*2];
+static unsigned int *cursors[16];
+static int cursor_loaded[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+static int cursor_format[16];
+static int active_cursor = -1;
+static int cursor_doublescan = 0; /* Used by et6000_cursor */
+
 /* The following are in the order of apearance in struct DriverSpecs as are */
 /* the program entries.                                                     */
 
@@ -85,9 +93,11 @@ static void et6000_setdisplaystart(int address);
 static void et6000_setlogicalwidth(int width);
 static void et6000_getmodeinfo(int mode,vga_modeinfo *modeinfo);
 /* Obsolete blit functions left out here. */
-static int  et6000_ext_set(unsigned what, va_list params);
-static int  et6000_accel(unsigned operation, va_list params);
+/* static int  et6000_ext_set(unsigned what, va_list params); */
+/* static int  et6000_accel(unsigned operation, va_list params); */
 static int  et6000_linear(int op, int param);
+/* Two empty spaces here */
+static int  et6000_cursor(int cmd,int p1,int p2,int p3,int p4,void *p5);
 
 /*
 ** End of the DriverSpecs function declarations         
@@ -116,11 +126,12 @@ DriverSpecs __svgalib_et6000_driverspecs =
     0,				/* fillblt */
     0,				/* hlinelistblt */
     0,				/* bltwait */
-    et6000_ext_set,
-    et6000_accel,
+    NULL,                       /* et6000_ext_set */
+    NULL,                       /* et6000_accel */
     et6000_linear,
     NULL,			/* accelspecs */
     NULL,			/* emulation */
+    et6000_cursor,              /* cursor */
 };
 #ifdef DBG
 /* programs to debug the modeline stuff. */
@@ -149,6 +160,7 @@ static void pmodetiming(ModeTiming *Tg)
 }
 #endif  /* Modeline routine debug. */
 
+/* Called from setmode */
 static void et6000_initializemode(unsigned char *moderegs, 
 				  ModeTiming *modetiming,ModeInfo *modeinfo,
 				  int mode)
@@ -233,6 +245,7 @@ static void et6000_initializemode(unsigned char *moderegs,
     moderegs[et6ksav + 12] = 0x00;
     moderegs[et6ksav +  6] = 0x04;
     moderegs[et6ksav] &= 0xfe;
+    moderegs[VGA_GR5] = 0x00;  /* 16 color mode not 0x02 */
     break;
   }
 
@@ -258,7 +271,6 @@ static int  et6000_saveregs(unsigned char regs[]) /* Called frm vga_saveregs */
 #ifdef DB6K
   { 
     fprintf(outf,"In saveregs, regs at %x\n",(int) regs);
-    fflush(outf);
   }
 #endif
   /* Save extended CRT registers. */
@@ -276,13 +288,13 @@ static int  et6000_saveregs(unsigned char regs[]) /* Called frm vga_saveregs */
   /* Extended sequencer register 7 in EXT+9 doesen't exist on et6000. */
   /* Also EXT+10 doesn't exist. There is no 0x3c3 on et6000.          */
   regs[EXT + 11] = port_in(SEG_SELECT);
-  regs[EXT + 10] = inb(base1 | 0x13);  /* I am using EXT+10 for the   */
+  regs[EXT + 10] = port_in(base1 | 0x13);  /* I am using EXT+10 for the   */
                                        /* linear map map location.    */
                                    /* Reset flip flop. Page 137       */
                                    /* of the et6000 manual. These two */
                                    /* regs 10 & 11 are 0 for linear.  */
-  regs[EXT] = inb(base1 | 0x58);   /* Offset 58 describes 16bpp       */
-  regs[EXT + 6] = inb(base1 | 0x59); /* bit1 = 0  555 RGB             */
+  regs[EXT] = port_in(base1 | 0x58);   /* Offset 58 describes 16bpp       */
+  regs[EXT + 6] = port_in(base1 | 0x59); /* bit1 = 0  555 RGB             */
                                      /*        1  565 RGB             */
                                    /* Offset 59 bits <32> set bpp.    */
                                    /* ---------------------           */
@@ -303,9 +315,12 @@ static int  et6000_saveregs(unsigned char regs[]) /* Called frm vga_saveregs */
   __svgalib_delay();
   regs[EXT + 7] = port_in(ATT_R);
   port_out(0x06,SEQ_I);
-  regs[EXT + 5] = ((inb(base1 | 0x57) & 2) << 4); /* bit 1 of 57 to bit5  */
-  regs[EXT + 9] = inb(base1 | 0x40);   /* Sets memory mapping.            */
-  regs[EXT + 1] = inb(base1 | 0x42);   /* Saved for future use.           */
+  regs[EXT + 5] = ((port_in(base1 | 0x57) & 2) << 4); /* bit 1 of 57 to bit5  */
+  regs[EXT + 9] = port_in(base1 | 0x40);   /* Sets memory mapping.            */
+  regs[EXT + 1] = port_in(base1 | 0x42);   /* Saved for future use.           */
+#ifdef DB6K
+  fprintf(outf,"Leaving saveregs.\n");
+#endif
   return(13);                          /* ET6000 requires 13 EXT regs.    */
                                        /* We may find that it needs more! */
 }
@@ -319,8 +334,6 @@ static void et6000_setregs(const unsigned char regs[], int mode)
   et6000_linear(LINEAR_DISABLE,0);  /* Turn off linear. */
 #ifdef DB6K
   fprintf(outf,"In setregs for regs at %x, mode = %d\n",(int) regs,mode);
-  fflush(outf);
-  /* return;       Testing */
 #endif
 
   et6000_unlock();
@@ -333,6 +346,9 @@ static void et6000_setregs(const unsigned char regs[], int mode)
   port_out(0x11,__svgalib_CRT_I);
   save = port_in(__svgalib_CRT_D);
   port_out(save &0x7F,__svgalib_CRT_D);
+#ifdef DB6K
+  fprintf(outf,"After first block\n");
+#endif
 
   /* Write extended CRT regs */            
   for(i = 0;i < 3;i++) {
@@ -347,12 +363,14 @@ static void et6000_setregs(const unsigned char regs[], int mode)
   outb(base1 | 0x58,(vga_getcolors() == (1 << 16))? (regs[EXT] | 0x02):
                                                         regs[EXT]);
   outb(base1 | 0x59,regs[EXT + 6]);
-  outb(base1 | 0x57,((inb(base1 | 0x57) | ((regs[EXT + 5] & 0x20) >> 4))));
+  outb(base1 | 0x57,((port_in(base1 | 0x57) | ((regs[EXT + 5] & 0x20) >> 4))));
 
     /* For True Color modes we must devide the MCLK by 3. */
     outb(base1 | 0x42,regs[EXT + 1]);
     outb(base1 | 0x40,regs[EXT + 9]);
-
+#ifdef DB6K
+    fprintf(outf,"after second block\n");
+#endif
   /* Original value for reg 0x11. */
   port_out(0x11,__svgalib_CRT_I);
   port_out(save,__svgalib_CRT_D);
@@ -370,6 +388,9 @@ static void et6000_setregs(const unsigned char regs[], int mode)
   __svgalib_delay();
   port_out(regs[EXT + 7],ATT_IW);
   /* I don't have an external DAC so I didn't include the last 3 regs.     */
+#ifdef DB6K
+  fprintf(outf,"End of setregs\n");
+#endif
 }
 
 static void et6000_unlock(void)
@@ -385,7 +406,6 @@ static void et6000_lock(void)
   if(x3_8) {
 #ifdef DB6K
     fprintf(outf,"Lock called\n");
-    fflush(outf);
 #endif
     return;                /* et4000 does this, I don't know why! */
     outb(x3_8,0);          /* I think it is because we lose root  */
@@ -447,7 +467,7 @@ static int  et6000_init(int force, int par1, int par2)
 
   if(et6000_chiptype)
     return(1);             /* An attempt to reenter init. */
-  /* Check for ET6000 chiptype.                                             */
+  /* Check for ET6000 chiptype.                                            */
   /* Get PCI and VL configuration space.  The VL bus has not been checked. */
   /* I have a PCI bus.                                                     */
 
@@ -455,16 +475,11 @@ static int  et6000_init(int force, int par1, int par2)
     x3_4 = __svgalib_CRT_I;       /* set in vga.c */
     x3_5 = __svgalib_CRT_D;
     x3_a = __svgalib_IS1_R;
-    if(__svgalib_CRT_I == 0x3d4)
+    if(__svgalib_CRT_I == CRT_IC)
       x3_8 = 0x3d8;
     else
       x3_8 = 0x3b8;
 
-    if(iopl(3) == -1)
-      {
-          printf("No permission to use PCI ports.\n");
-          perror("PCI ports");
-      }
     et6000_linear_base = 0;
 		if(__svgalib_pci_find_vendor_vga(0x100c,buf,0) == 0 &&
 		   (buf[0] == 0x3208100c))
@@ -475,14 +490,14 @@ DPRT("Found et6000\n");
 		else 
 		  {
 #ifdef DB6K 
-		   fprintf(outf,"Can't ifnd et6000 chipset. buf[0] = 0x%8lx\n",
+		   fprintf(outf,"Can't find et6000 chipset. buf[0] = 0x%8lx\n",
 			(unsigned long) buf[0]);
 #endif
 		    return(0);
 		  }
 
 		et6000_unlock();            /* Set key. */
-		mem_type = inb(0x3c2) & 1;  /* 0 = DRAM, 1 = MDRAM */
+		mem_type = port_in(MIS_W) & 1;  /* 0 = DRAM, 1 = MDRAM */
 
                 base1 = buf[0x5] & ~0xFF; 
 		              /* 14 hex. The start of confg.  */
@@ -497,13 +512,13 @@ DPRT("Found et6000\n");
 			      */
 		if(!pci59std) {
 		  /* Saving pcibus registers so they may be restored. */
-		  pci40std = (unsigned char) buf[16] & 0xFF;
-		  pci42std = (unsigned char) ((buf[16] & 0xFF0000) >> 16);
-		  pci57std = (unsigned char) ((buf[21] & 0xFF000000) >> 24);
-		  pci58std = (unsigned char) buf[22] & 0xFF;
-		  pci59std = (unsigned char) ((buf[22] & 0xFF00) >> 8);
+		  pci40std = port_in(base1 | 0x40);
+		  pci42std = port_in(base1 | 0x42);
+		  pci57std = port_in(base1 | 0x57);
+		  pci58std = port_in(base1 | 0x58);
+		  pci59std = port_in(base1 | 0x59);
 		}
-		mem_cfg_reg = (unsigned char) ((buf[17] & 0xFF00) >> 8) & 3;
+		mem_cfg_reg = port_in(base1 | 0x45) & 3;
 #ifdef DB6K
 		{
 		  int i;
@@ -511,8 +526,9 @@ DPRT("Found et6000\n");
 			fprintf(outf,"%08x     %08lx  %08lx  %08lx  %08lx\n",
 				(i/4) << 4,buf[i+3],buf[i+2],buf[i+1],buf[i]);
 		  fprintf(outf,"\npci40std 0x%02x  pci42std 0x%02x  pci58std "
-                  "0x%02x  pci59std 0x%02x\n",pci40std,
-			  pci42std,pci58std,pci59std);
+                  "0x%02x  pci59std 0x%02x\n"
+		  "mem_cfg_reg 0x%02x mem_type 0x%02x\n",pci40std,
+			  pci42std,pci58std,pci59std,mem_cfg_reg,mem_type);
 		}
 #endif
 		if(mem_type)
@@ -538,10 +554,14 @@ DPRT("Found et6000\n");
     outb(base1 | 0x68,0xa);   /* Set clocka */
     outb(base1 | 0x69,0x24);
     outb(base1 | 0x69,0x21);
-
     if(__svgalib_driver_report)
       printf("Using Tseng ET6000 driver (%d MBytes %sDRAM)\n",
 		       et6000_memory,mem_type ? "M" : "");
+#ifdef DB6K
+    else
+      fprintf(outf,"NO driver report. %d MBytes %sDRAM\n",et6000_memory,
+	      mem_type ?  "M" : "");
+#endif
     __svgalib_driverspecs = &__svgalib_et6000_driverspecs;
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = et6000_memory << 10;
@@ -585,12 +605,15 @@ static unsigned char last_page = 0;
 /* Bank switching function.  Set 64k bank number. */
 static void et6000_setpage(int page)
 {
+#ifdef DB6KP
+  fprintf(outf,"setpage %d\n",page);
+#endif
     /* Set both read and write bank. 3cd */
     port_out(last_page = (page | ((page & 15) << 4)), SEG_SELECT);
 	/* Write page4-5 to bits 0-1 of ext. bank register, */
 	/* and to bits 4-5. */
     /* return;       Testing */
-	outb(0x3cb, ((inb(0x3cb) & ~0x33) | (page >> 4) | (page & 0x30)));
+	outb(0x3cb, ((port_in(0x3cb) & ~0x33) | (page >> 4) | (page & 0x30)));
 }
 
 static void et6000_setrdpage(int page)
@@ -599,7 +622,7 @@ static void et6000_setrdpage(int page)
   last_page &= 0x0F;
   last_page |= (page << 4);
   port_out(last_page,SEG_SELECT);
-  outb(0x3cb,(inb(0x3cb) & ~0x30) | (page & 0x30));
+  outb(0x3cb,(port_in(0x3cb) & ~0x30) | (page & 0x30));
 }
 
 static void et6000_setwrpage(int page)
@@ -608,7 +631,7 @@ static void et6000_setwrpage(int page)
   last_page &= 0xF0;
   last_page |= (page | 0x0F);
   port_out(last_page,SEG_SELECT);
-  outb(0x3cb,(inb(0x3cb) & ~0x03) | page >> 4);
+  outb(0x3cb,(port_in(0x3cb) & ~0x03) | page >> 4);
 }
 
 static int  et6000_setmode(int mode, int prv_mode)
@@ -626,28 +649,31 @@ static int  et6000_setmode(int mode, int prv_mode)
     switch (et6000_modeavailable(mode)) {
     case STDVGADRV:
 	/* Reset extended register that is set to non-VGA */
-	/* compatible value for 132-column textmodes (at */
+	/* compatible value for 132-column textodes (at */
 	/* least on some cards). */
 DPRT("STDVGADRV\n");
 	et6000_unlock();
 	outb(__svgalib_CRT_I, 0x34);            /* 3#4 */
-	i = inb(__svgalib_CRT_D);               /* 3#5 */
+	i = port_in(__svgalib_CRT_D);               /* 3#5 */
 	if ((i & 0x02) == 0x02)                 /* Clock0 select bit 2 */
 	    outb(__svgalib_CRT_D, (i & 0xFD));  /* Turn it off. */
 	                              /* Make sure pci bus is set right. */
 #ifdef DB6K
-	fprintf(outf,"reg 42 = %2x  58 = %2x before.\n",inb(base1 | 0x42)
-		,inb(base1 | 0x58));
+	fprintf(outf,"reg 42 = %2x  58 = %2x before.\n",port_in(base1 | 0x42)
+		,port_in(base1 | 0x58));
 #endif
 	outb(base1 | 0x40,pci40std);
 	outb(base1 | 0x42,pci42std);
 	outb(base1 | 0x57,pci57std);
 	outb(base1 | 0x58,pci58std);
 	outb(base1 | 0x59,pci59std);
+	outb(base1 | 0x46,(port_in(base1 | 0x46)&0xfe)); 
+	  /* make sure cursor is off */
+	if(mode == 5) cursor_doublescan = 1;
 #ifdef DB6K
 	fprintf(outf,"pci42 = %2x, pci58 = %2x, pci59 = %2x\n",pci42std,
 		pci58std,pci59std);
-	fprintf(outf,"reg 42 = %2x after.\n",inb(base1 | 0x42));
+	fprintf(outf,"reg 42 = %2x after.\n",port_in(base1 | 0x42));
 #endif
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     case SVGADRV:
@@ -666,15 +692,18 @@ DPRT("SVGADRV\n");
       }
     moderegs = malloc(ET6000_TOTAL_REGS);
     et6000_initializemode(moderegs,modetiming,modeinfo,mode);
+    cursor_doublescan = modetiming->flags & DOUBLESCAN ?1:0;
 #ifdef DBG
     fprintf(fdm,"In setmode.\n");
     pmodetiming(modetiming);
     fdumpregs(fdm,moderegs,ET6000_TOTAL_REGS);
+    fprintf(fdm,"cursor_doublescan = %1d\n",cursor_doublescan);
 #endif
 	__svgalib_setregs(moderegs);
 	et6000_setregs(moderegs,mode);
 	free(moderegs);
 	free(modeinfo);
+	free(modetiming);
 	return(0);
 }
 
@@ -685,7 +714,8 @@ static int  et6000_modeavailable(int mode)
   ModeTiming *modetiming;
   ModeInfo *modeinfo;
 
-  if(mode == G720x348x2 || mode == GPLANE16 || mode < G640x480x256 )
+
+  if (IS_IN_STANDARD_VGA_DRIVER(mode))
     return(__svgalib_vga_driverspecs.modeavailable(mode));
   if(mode <= TEXT || mode > GLASTMODE)
     return(0);
@@ -720,23 +750,26 @@ static int  et6000_modeavailable(int mode)
 
 static void et6000_setdisplaystart(int address)
 {
-  DPRT("setdisplaystart");
-  outw(x3_4, 0x0d + ((address ) & 0x00ff)*256);      /* ds0-7  */
-  outw(x3_4, 0x0c + ((address ) & 0xff00)*256);      /* ds0 15 */
-  /* inb(x3_a);                This was from the et4000 card /? ATC write 
-     outb(0x3c0,0x13 + 0x20);                                      mode.   
-  outb(0x3c0,(inb(0x3c1) & 0xf0) | ((address & 3) << 1));  ds0-1  */
+  /* DPRT("setdisplaystart\n");
+   #ifdef DB6K
+  fprintf(outf,"address = 0x%08x\n",address);
+  #endif */
+  outw(x3_4, 0x0d +(((address >>2) & 0x00ff) << 8));  /* ds0-7  */
+  outw(x3_4, 0x0c + ((address >>2) & 0xff00));        /* ds0 15 */
   outb(x3_4, 0x33);
-  outb(x3_5, (inb(x3_5) & 0xfc)  | ((address & 0xf0000) >> 16 ));
-                                                          /* ds 16-19 */
+  outb(x3_5, (port_in(x3_5) & 0xf0)  | ((address & 0xf0000) >> 18 ));
+                                                      /* ds 16-19 */
 }
 
 static void et6000_setlogicalwidth(int width)
 {
-  DPRT("setlogicalwidth");
+  DPRT("setlogicalwidth\n");
+#ifdef DB6K
+  fprintf(outf,"width = 0x%08x\n",width);
+#endif
   outw(x3_4,0x13 + (((width >> 3 ) & 0xff) << 8));       /* lw3-10 */
   outb(x3_4 ,0x3f);
-  outb(x3_5,(inb(x3_5) & 0x3f) | ((width & 0x1800) >> 3)); /* lw11-12 */
+  outb(x3_5,(port_in(x3_5) & 0x3f) | ((width & 0x1800) >> 3)); /* lw11-12 */
 }
 
 static void et6000_getmodeinfo(int mode,vga_modeinfo *modeinfo)
@@ -754,15 +787,15 @@ static void et6000_getmodeinfo(int mode,vga_modeinfo *modeinfo)
     break;
   }
   modeinfo->maxlogicalwidth = 4088;              /* Why? */
-  modeinfo->startaddressrange = 0xfffff;  /* Mask for the startaddress */
+  modeinfo->startaddressrange = 0xfffff*et6000_memory;
+           /* Mask for the startaddress */
   if(mode == G320x200x256)
     modeinfo->startaddressrange = 0;
   modeinfo->haveblit = 0;
   modeinfo->memory = et6000_memory*1024;         /* In kbytes. */
-  modeinfo->flags |= HAVE_RWPAGE | HAVE_EXT_SET;
+  modeinfo->flags |= HAVE_RWPAGE;
   if(et6000_interlaced(mode))
     modeinfo->flags |= IS_INTERLACED;
-  modeinfo->flags |= EXT_INFO_AVAILABLE;
 #ifdef DB6K
   fprintf(outf,"Getmodeinfo mode = %d, colors = %d, linearset = 0x%x " 
           ,mode,modeinfo->colors,__svgalib_modeinfo_linearset);
@@ -833,7 +866,7 @@ static int compareregs(void)
 #else
 }
 #endif
-
+/*
 static int  et6000_ext_set(unsigned what, va_list params)
 {
   DNOTW("ext_set");
@@ -845,7 +878,7 @@ static int  et6000_accel(unsigned operation, va_list params)
   DNOTW("accel");
   return(0);
 }
-
+*/
 static int  et6000_linear(int op, int param)
 {
 #ifdef DB6K
@@ -854,8 +887,9 @@ static int  et6000_linear(int op, int param)
   if(op == LINEAR_QUERY_BASE) {
 #ifdef DB6K
     fprintf(outf, "QUERY\n");
-#endif
-    if((vga_getmodeinfo(vga_getcurrentmode())->flags | CAPABLE_LINEAR))
+#endif                                    
+                              /* Possible error with & */
+    if((vga_getmodeinfo(vga_getcurrentmode())->flags & CAPABLE_LINEAR))
       return(et6000_linear_base);
     else                    /* frame_ptr was determined at init time */
       return(-1);
@@ -868,8 +902,8 @@ static int  et6000_linear(int op, int param)
       et6000_mapped_mem = param;
       port_out(0,SEG_SELECT);
       port_out(0,0x3cb);
-      port_out(5,0x3ce);
-      port_out(0x40,0x3cf);  /* Set linear graphics. Don't touch this */
+      port_out(5,GRA_I);
+      port_out(0x40,GRA_D);  /* Set linear graphics. Don't touch this */
                              /* on disable. */
       outb(base1 | 0x40, 0x09);
       /* Set memory relocation and linear. */
@@ -883,8 +917,6 @@ static int  et6000_linear(int op, int param)
       outb(base1 | 0x40, 0);
       /* Turn off memory relocation and linear. */
       __svgalib_modeinfo_linearset = 0;  /* In vga.c */
-      /*      port_out(5,0x3ce);
-      port_out(0x00,0x3cf);   See comment on LINEAR_ENABLE */
       return(0);
     }
     else if(op == LINEAR_QUERY_RANGE) {
@@ -904,7 +936,227 @@ static int  et6000_linear(int op, int param)
 #endif
     return(-1);
 }
+static int et6000_cursor(int cmd,int p1,int p2,int p3,int p4,void *p5)
+{
+  int i,j;
 
+#ifdef DB6K
+  if(cmd != 2 && cmd != 3)
+  fprintf(outf,"In et6000_cursor cmd = %d\n",cmd);
+  fflush(outf);
+#endif
+  switch(cmd)
+    {
+    case CURSOR_INIT:
+      for(i = 0;i < 16;i++)
+	cursors[i] = NULL;
+      break;
+
+    case CURSOR_HIDE:
+      outb(base1 | 0x46,port_in(base1 | 0x46)&0xfe);
+      break;
+
+    case CURSOR_SHOW:
+      if(active_cursor == -1)
+	{
+	  printf("No active cursor\n");
+	  return(-1);
+	}
+      outb(base1 | 0x46,port_in(base1 | 0x46) | 0x01);
+      break;
+
+    case CURSOR_POSITION:
+      outb(base1 | 0x84,p1&0xff);
+      outb(base1 | 0x85,p1 >> 8);
+      outb(base1 | 0x86,(p2<<cursor_doublescan)&0xff);
+      /* double for double scan */
+      outb(base1 | 0x87,(p2<<cursor_doublescan) >> 8);
+      break;
+
+    case CURSOR_IMAGE:
+      switch(p2)
+	{
+	case 0:
+	  if(cursors[p1] != NULL)
+	    free(cursors[p1]);
+	  cursors[p1] = malloc(256);
+	  memcpy(cursors[p1],p5,256);
+	  cursor_format[p1] = 0;
+	  break;
+
+	case 1:
+	  if(cursors[p1] != NULL)
+	    free(cursors[p1]);
+	  cursors[p1] = malloc(4096);
+	  memcpy(cursors[p1],p5,4096);
+	  cursor_format[p1] = 1;
+	  break;
+	}
+      cursor_colors[p1*2] = p3;
+      cursor_colors[p1*2 + 1] = p4;
+      cursor_loaded[p1] = 1;
+      break;
+
+    case CURSOR_SELECT:
+      if(active_cursor == p1)
+	return(0);
+      if(cursor_loaded[p1])
+	{
+	  int c = 64, cl[2],sptptr,sptr,k,n,l,cf,islinear,col=0;
+	  unsigned char c0=0,cr;
+	  unsigned char f,f1,cb,*cu;
+
+	  if(vga_getmodeinfo(vga_getcurrentmode())->flags & IS_LINEAR)
+	    islinear = 1;
+	  else
+	    islinear = 0;
+#ifdef DB6K
+		
+		fprintf(outf,"cursor_loaded[%d] = %d\n",p1,cursor_loaded[p1]);
+		{
+		  unsigned short pat[8] = {0x0055,0xaa55,0xff55,0x0033,0x0022,
+					   0x00aa,0xffaa,0x0000};
+		  int i,j;
+		  if(islinear)
+		    {
+		      fprintf(outf,"Islinear = %d , 16*et6000_memory = %d\n",
+			      islinear,16*et6000_memory);
+		      /* Put something here for linear. */
+		    }
+		  else
+		    {
+		      fprintf(outf,"islinear = %d\n",islinear);
+		      et6000_setpage((16*et6000_memory) - 1);
+		    }
+		  for(i = 0;i < 0x8;i++)
+		    for(j = 0;j < 0x2000;j+=2)
+		      gr_writew(pat[i],i*0x2000 + j + islinear*
+				65536*(16*et6000_memory - 1));
+		  if(!islinear) et6000_setpage(0);
+		}
+#endif
+	  for(j = 0;j < 2;j++)
+	    {
+	      cl[j] = 0;
+	      col = cursor_colors[p1*2 + j];
+	      for(i = 0;i < 3;i++)
+		{
+		  cf = (col << i*8) & 0xff0000;
+		  if(c > cf) c0 = 0;
+		  else if(2*c > cf) c0 = 1;
+		  else if(3*c > cf) c0 = 2;
+		  else c0 = 3;
+		  cl[j] |= c0;
+		  cl[j] = cl[j] << 2;
+#ifdef DB6K
+		  fprintf(outf,"f = 0x%02x \n",cf);
+#endif
+		}
+	      cl[j] = cl[j] >> 2;
+#ifdef DB6K
+		fprintf(outf,"Col = 0x%x, Color %d = 0x%x\n c0 = 0x%x\n"
+			,col,j,cl[j],c0);
+#endif
+	    }
+	  outb(base1 | 0x67,9);  /* set register 9 for colors */
+	  outb(base1 | 0x69,cl[0]); /* Sprite color 0 */
+	  outb(base1 | 0x69,cl[1]); /* Sprite color 1 */
+	  /*  sptr = sptptr = __svgalib_linear_mem_size/2 - 1024; */
+	  sptptr = 0x7ff00;
+	  sptr = 64512 - 128 - 512*(cursor_doublescan - 1) + 
+	    (islinear)*65536*(16*et6000_memory - 2); 
+	  /* 64512 = 65536 - 16*64  */
+	  if(islinear)
+	    {
+#ifdef DB6K
+	      fprintf(outf,"Linear sptr = %d\n",sptr);
+#endif
+	      /* Put something here for linear */
+	    }
+	  else
+	    {
+#ifdef DB6K
+	      fprintf(outf,"Paged sptr = %d\n",sptr);
+#endif
+
+	      et6000_setpage((16*et6000_memory) - 1);
+	    }
+	  cr = 0;
+	  cb = 0;
+	  l = 0;
+	  cu = (unsigned char *) cursors[p1];
+	  for(k = 0;k < 32;k++)
+	    {
+	      for(n = 3;n >= 0;n--)
+		{
+		  f = *(cu + (32 + k)*4 + n);
+		  f1 = *(cu + k*4 + n);
+		  for(j = 0;j < 8;j++)
+		    {
+		      if(f&0x01)
+			{
+			  if(f1&0x01) cr |= 0x01;
+			  else
+			    cr |= 0x00;
+			}
+		      else cr |= 0x02;
+		      cb = cb << 2;
+		      cb |= cr;
+		      cr = 0;
+		      f  = f >> 1;
+		      f1 = f1 >> 1;
+		      if(!((j+1) % 4))
+		      {
+			gr_writeb(cb,sptr + 16*8 + 9 + l - j/4);
+			cb = 0;
+		      }
+		    }
+		  l += 2;
+		}
+	      sptr += 8;
+	    }
+	    
+	  outb(x3_4,0x0E); /* CRTC Reg E bits 16-19 of sprite add. */
+	  outb(x3_5,(sptptr >> 16) & 0x0f);
+	  outb(x3_4,0x0F);
+	  outb(x3_5,(sptptr >> 8)&0xff);
+	  outb(base1 | 0x82,0x20);  /* set cursor size. */
+	  outb(base1 | 0x83,0x20*(cursor_doublescan-1));
+	  active_cursor = p1;
+#ifdef DB6K
+	  outb(x3_4,0x0E);
+	  fprintf(outf,"sptptr = 0x%08x  0E = 0x%x ",
+		  sptptr,port_in(x3_5));
+	  outb(x3_4,0x0F);
+	  fprintf(outf,"0F = 0x%x\n",port_in(x3_5));
+	  *(LINEAR_POINTER) = 0x26;
+	  fprintf(outf," = 0x%02x\n",
+		  *(LINEAR_POINTER));
+	  if(!islinear) et6000_setpage((16*et6000_memory) - 1);
+	  for(i = 0;i < 64;i++)
+	    {
+	      for(j = 0;j < 16;j++)
+		fprintf(outf,"%02x",
+		    gr_readb(64512 - 128 + 16*i + j + islinear*65536*
+		      ((16*et6000_memory) - 1) - 512*(cursor_doublescan-1)));
+	      fprintf(outf,"\n");
+	    }
+	  fprintf(outf,"LINEAR_POINTER %08x, GM = %08x, islinear = %d\n",
+		  (unsigned int)  LINEAR_POINTER,GM,islinear);
+	  if(!islinear)et6000_setpage(0);
+#endif		      
+	  if(!islinear)et6000_setpage(0);
+	}
+      else 
+	{
+	  printf("Cursor %d's immage has not been loaded when selected!\n",
+		 p1);
+	 return(-1);
+	}
+      break;
+    }
+  return(0);
+}
 
 /* Programs not exported. */
 
