@@ -147,16 +147,16 @@ static void gvga6400_setdisplaystart(int address)
     if (modeinfo.bytesperpixel == 0)
 	switch (modeinfo.colors) {
 	case 16:		/* planar 16-color mode */
-	    inb(0x3da);
+	    port_in(0x3da);
 	    outb(ATT_IW, 0x13 + 0x20);
-	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | (address & 7));
+	    outb(ATT_IW, (port_in(ATT_R) & 0xf0) | (address & 7));
 	    /* write sa0-2 to bits 0-2 */
 	    address >>= 3;
 	    break;
 	case 256:		/* planar 256-color mode */
-	    inb(0x3da);
+	    port_in(0x3da);
 	    outb(ATT_IW, 0x13 + 0x20);
-	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
+	    outb(ATT_IW, (port_in(ATT_R) & 0xf0) | ((address & 3) << 1));
 	    /* write sa0-1 to bits 1-2 */
 	    address >>= 2;
 	    break;
@@ -219,17 +219,17 @@ static int gvga6400_test(void)
 static int gvga6400_saveregs(unsigned char regs[])
 {
     outb(SEQ_I, 6);
-    regs[EXT + 2] = inb(SEQ_D);
+    regs[EXT + 2] = port_in(SEQ_D);
     outb(SEQ_I, 7);
-    regs[EXT + 3] = inb(SEQ_D);
+    regs[EXT + 3] = port_in(SEQ_D);
     outb(SEQ_I, 8);
-    regs[EXT + 4] = inb(SEQ_D);
+    regs[EXT + 4] = port_in(SEQ_D);
     outb(SEQ_I, 0x10);
-    regs[EXT + 5] = inb(SEQ_D);
+    regs[EXT + 5] = port_in(SEQ_D);
     outb(GRA_I, 9);
-    regs[EXT + 6] = inb(GRA_D);
+    regs[EXT + 6] = port_in(GRA_D);
     outb(CRT_IC, 0x2f);
-    regs[EXT + 10] = inb(CRT_DC);
+    regs[EXT + 10] = port_in(CRT_DC);
 
     return (11);
 }
