@@ -70,26 +70,26 @@ static CardSpecs *cardspecs;
 
 static void i740_outxr_io(int index, int val)
 {
-    outb(XRX,index);
-    outb(XRX+1,val);
+    port_out(index, XRX);
+    port_out(val, XRX+1);
 }
 
 static int i740_inxr_io(int index)
 {
-    outb(XRX,index);
-    return inb(XRX+1);
+    port_out(index, XRX);
+    return port_in(XRX+1);
 }
 
 static void i740_outxr_mm(int index, int val)
 {
-    *(VGAMap+XRX)=index;
-    *(VGAMap+XRX+1)=val;
+    *(__svgalib_vgammbase+XRX)=index;
+    *(__svgalib_vgammbase+XRX+1)=val;
 }
 
 static int i740_inxr_mm(int index)
 {
-    *(VGAMap+XRX)=index;
-    return *(VGAMap+XRX+1);
+    *(__svgalib_vgammbase+XRX)=index;
+    return *(__svgalib_vgammbase+XRX+1);
 }
 
 static void (*i740_outxr)(int,int)=i740_outxr_io;
@@ -291,10 +291,7 @@ static int i740_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
@@ -567,7 +564,7 @@ static int i740_setmode(int mode, int prv_mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    if ((mode < G640x480x256)||(mode==G720x348x2)) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
     if (!i740_modeavailable(mode))
@@ -722,10 +719,10 @@ static int i740_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
                         l2=(l2<<24)|(l2>>24)|((l2>>8)&0xff00)|((l2<<8)&0xff0000);
                         l3=(l3<<24)|(l3>>24)|((l3>>8)&0xff00)|((l3<<8)&0xff0000);
                         l4=(l4<<24)|(l4>>24)|((l4>>8)&0xff00)|((l4<<8)&0xff0000);
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j)=l2;
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+4)=l4;
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+8)=l1;
-                        *(unsigned long *)(LINEAR_MEM_POINTER+i+16*j+12)=l3;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j)=l2;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+4)=l4;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+8)=l1;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+12)=l3;
                     }
                 break;
             }
@@ -799,8 +796,8 @@ static int i740_init(int force, int par1, int par2)
        i740_mmio_base=buf[5]&0xffffff00;
     };
 
-    VGAMap=mmap(0,0x1000,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,i740_mmio_base);
-    mm_io_mapio();
+    __svgalib_vgammbase=mmap(0,0x1000,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,i740_mmio_base);
+    __svgalib_mm_io_mapio();
     i740_inxr=i740_inxr_mm;
     i740_outxr=i740_outxr_mm;
     
@@ -828,7 +825,7 @@ static int i740_init(int force, int par1, int par2)
     cardspecs->maxPixelClock24bpp = 128000;
     cardspecs->maxPixelClock32bpp = 86000;
     cardspecs->flags = CLOCK_PROGRAMMABLE;
-    cardspecs->maxHorizontalCrtc = 2040;
+    cardspecs->maxHorizontalCrtc = 4088;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->mapClock = i740_map_clock;
