@@ -27,11 +27,15 @@
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <termios.h>
-#include <linux/kd.h>
+#include <sys/kd.h>
+#if 0
 /* linux/keyboard.h defines NR_KEYS and some scancode-like constants, so it */
 /* should also be useful for svgalib programs using the keyboard. It misses */
 /* a few KERNEL ifdefs around kernel data structures though. */
 #include <linux/keyboard.h>
+#else
+#define NR_KEYS 128
+#endif
 #include <sys/vt.h>
 /* Needed to check uid of keymap files */
 #include <sys/stat.h>
@@ -150,7 +154,7 @@ static char *kbd_config_options[] = {
   "kbd_fake_mouse_event", "kbd_keymap", "kbd_only_root_keymaps", NULL
 };
 
-static char *kbd_process_option(int option, int mode) {
+static char *kbd_process_option(int option, int mode, char **nptr) {
   char *ptr;
   short type   = 0;
   short ignore_type   = 0;
@@ -164,7 +168,7 @@ static char *kbd_process_option(int option, int mode) {
   
   switch (option) {
     case 0: /* kbd_fake_mouse_event */
-	if ( (ptr = strtok(NULL, " ")) == 0) {
+	if ( (ptr = __svgalib_token(nptr)) == 0) {
   param_needed:
 	  fprintf(stderr, "svgalib: kbd-config: too few parameters for \'%s\'\n",
 	  		kbd_config_options[option]);
@@ -190,7 +194,7 @@ static char *kbd_process_option(int option, int mode) {
 #endif
         
   read_event:
-	if ( (ptr = strtok(NULL, " ")) == 0) {
+	if ( (ptr = __svgalib_token(nptr)) == 0) {
 	  if ( event_ok )
 	    break;
 	  else  
@@ -216,7 +220,7 @@ static char *kbd_process_option(int option, int mode) {
 	   goto read_event;
 	} else if ( ! strcasecmp(ptr, "button")) {
 	   type = FME_TYPE_BUTTON;
-	   if ( (ptr = strtok(NULL, " ")) == 0)
+	   if ( (ptr = __svgalib_token(nptr)) == 0)
 	     goto param_needed;
 	   if ( (data = atoi(ptr)) == 0  ||  data > 10) {
 	     fprintf(stderr, "svgalib: kbd-config: \'%s\' is not a valid mouse button\n", ptr);
@@ -243,7 +247,7 @@ static char *kbd_process_option(int option, int mode) {
 	   ignore_type = FME_TYPE_IGNOREX;
 
   process_delta:
-	   if ( (ptr = strtok(NULL, " ")) == 0)
+	   if ( (ptr = __svgalib_token(nptr)) == 0)
 	     goto param_needed;
 	   if ( ! strcasecmp(ptr, "off")) {
 	     type = ignore_type;
@@ -269,7 +273,7 @@ static char *kbd_process_option(int option, int mode) {
 	} else if ( ! strcasecmp(ptr, "button1")) {
 	   type = FME_TYPE_BUTTON1;
   process_button:
-	   if ( (ptr = strtok(NULL, " ")) == 0)
+	   if ( (ptr = __svgalib_token(nptr)) == 0)
 	     goto param_needed;
 	   if ( ! strcasecmp(ptr, "pressed")) {
 	     data = 1;
@@ -295,7 +299,7 @@ static char *kbd_process_option(int option, int mode) {
     	break;
 
     case 1: /* kbd_keymap */
-         if ( (ptr = strtok(NULL, " ")) == 0 ) {
+         if ( (ptr = __svgalib_token(nptr)) == 0 ) {
              goto param_needed;
          } else
              if(kbd_load_keymap(ptr))
@@ -311,12 +315,14 @@ static char *kbd_process_option(int option, int mode) {
       break;
 
   }
-  return strtok(NULL, " ");
+  return __svgalib_token(nptr);
 };
 
 int keyboard_init_return_fd(void) {
     char *ptr;
 
+    DTP((stderr,"keyboard_init_return_fd\n"));
+
     keyboard_translatekeys(translatemode); /* Honour 'nosigint' setting */
 
     /* Install default keyboard handler. */
@@ -362,6 +368,8 @@ int keyboard_init_return_fd(void) {
 
 int keyboard_init(void)
 {
+    DTP((stderr,"keyboard_init\n"));
+
     if (keyboard_init_return_fd() == -1)
 	return -1;
     else
@@ -370,6 +378,8 @@ int keyboard_init(void)
 
 void keyboard_close(void) {
 
+    DTP((stderr,"keyboard_close\n"));
+
     if (__svgalib_kbd_fd < 0)
 	return;
 
@@ -487,7 +497,7 @@ static int keyboard_getevents(int wait)
              	win_state = is_pressed;
 		break;
 
-	  case SCANCODE_F1...SCANCODE_F10:
+	  case SCANCODE_F1 ... SCANCODE_F10:
 		functionkey_state = (is_pressed)  ?
 		  functionkey_state |   1 << (scancode - SCANCODE_F1)   :
 		  functionkey_state & ~(1 << (scancode - SCANCODE_F1));
