@@ -7,10 +7,13 @@ Problems:
     secondary (mmio only) not tested.
     fifo threshold needs more tuning.
 */
+
+
 #include <stdlib.h>
 #include <stdio.h>		
 #include <string.h>
 #include <unistd.h>
+#include <sys/mman.h>
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
@@ -20,6 +23,8 @@ Problems:
 #include "accel.h"
 #include "vgapci.h"
 
+#include "lagunaio.c"
+
 struct lagunahw {
     unsigned char crt[5];
     unsigned char gra[3];
@@ -46,6 +51,8 @@ enum {
     L5462=0, L5464, L5465
 };
 
+static int pagemul=4;
+
 static int laguna_init(int, int, int);
 static void laguna_unlock(void);
 static void laguna_lock(void);
@@ -57,33 +64,9 @@ static int laguna_is_linear, laguna_linear_base, laguna_mmio_base;
 
 static CardSpecs *cardspecs;
 
-static inline unsigned int laguna_readb(int a) {
-    return *(MMIO_POINTER+a);
-}
-
-static inline unsigned int laguna_readw(int a) {
-    return *(unsigned short *)(MMIO_POINTER+a);
-}
-
-static inline unsigned int laguna_readl(int a) {
-    return *(unsigned int *)(MMIO_POINTER+a);
-}
-
-static inline void laguna_writeb(int a, unsigned int d) {
-    *(MMIO_POINTER+a)=d;
-}
-
-static inline void laguna_writew(int a, unsigned int d) {
-    *(unsigned short *)(MMIO_POINTER+a)=d;
-}
-
-static inline void laguna_writel(int a, unsigned int d) {
-    *(unsigned int *)(MMIO_POINTER+a)=d;
-}
-
 static void laguna_setpage(int page)
 {
-    __svgalib_outgra(0x09,page*4);
+    __svgalib_outgra(0x09,page*pagemul);
 }
 
 static int __svgalib_laguna_inlinearmode(void)
@@ -167,30 +150,37 @@ static void laguna_setregs(const unsigned char regs[], int mode)
     __svgalib_outcrtc(0x1d,l->crt[3]);
     __svgalib_outcrtc(0x1e,l->crt[4]);
 
-    __svgalib_outgra(0x09,l->gra[0]);
-    __svgalib_outgra(0x0a,l->gra[1]);
-    __svgalib_outgra(0x0b,l->gra[2]);
-
+    __svgalib_outseq(0x07,l->seq[2]);
     __svgalib_outseq(0x0e,l->seq[0]);
     __svgalib_outseq(0x1e,l->seq[1]);
-    __svgalib_outseq(0x07,l->seq[2]);
+
+    laguna_writew(0xc0,l->format);
+
+    laguna_writel(0x3fc,l->pcifc);
+
+    laguna_writew(0xea,l->disp_thresh);
 
     if(laguna_chiptype==L5465)
         laguna_writew(0x2c4,l->tiling_control);
-    laguna_writew(0x402,l->control);
+
+    laguna_writew(0x407,l->tile);
+
     if(laguna_chiptype==L5465)
         laguna_writeb(0x2c0,l->bclk);
         else laguna_writeb(0x8c,l->bclk);
-    laguna_writew(0x407,l->tile);
+
+    laguna_writew(0x402,l->control);
+
     laguna_writew(0xb0,l->pal_state);
-    laguna_writew(0xc0,l->format);
     laguna_writew(0xe0,l->cur_x);
     laguna_writew(0xe2,l->cur_y);
     laguna_writew(0xe4,l->cur_preset);
     laguna_writew(0xe6,l->cur_misc);
     laguna_writew(0xe8,l->cur_address);
-    laguna_writew(0xea,l->disp_thresh);
-    laguna_writel(0x3fc,l->pcifc);
+
+    __svgalib_outgra(0x09,l->gra[0]);
+    __svgalib_outgra(0x0a,l->gra[1]);
+    __svgalib_outgra(0x0b,l->gra[2]);
 }
 
 
@@ -202,8 +192,7 @@ static int laguna_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
@@ -233,10 +222,11 @@ static int laguna_findclock(int clock, int *on, int *om, int *op, int *opp)
     int n, m, diff, best, bestm;
     
     *opp=0;
-    while(clock<20000) {
-        clock*=2;
-        *opp=+1;
-    }
+
+//    while(clock<20000) {
+//        clock*=2;
+//        *opp=+1;
+//    }
     
     if(clock<40000) {
         clock *= 2;
@@ -247,7 +237,7 @@ static int laguna_findclock(int clock, int *on, int *om, int *op, int *opp)
     bestm=5;
     for(m=6; m<30;m++) {
         n=clock*m/REF;
-        if((n<64)||(n>127)) continue;
+        if((n<32)||(n>127)) continue;
         diff=clock-(n*REF/m);
         if(diff<0)diff=-diff;
         if(diff<best) {
@@ -309,6 +299,8 @@ static void laguna_initializemode(unsigned char *moderegs,
         l->bclk=laguna_readb(0x2c0);
         else l->bclk=laguna_readb(0x8c);
 
+    l->bclk=18;
+
     l->tile=laguna_readb(0x407) & 0x3f; /* no interleave */
 
     offset = modeinfo->lineWidth >> 3;
@@ -356,7 +348,7 @@ static void laguna_initializemode(unsigned char *moderegs,
     l->seq[2]=1;
 
     l->pal_state=0;
-    l->format=pp>>14;
+    l->format=0;
     switch(modeinfo->bitsPerPixel) {
         case 8:
             l->format|=0;
@@ -393,7 +385,7 @@ static int laguna_setmode(int mode, int prv_mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    if ((mode < G640x480x256)||(mode==G720x348x2)) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
         laguna_writeb(0xe6,0);        
         __svgalib_outseq(0x07,0);
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
@@ -511,6 +503,67 @@ return htiming;
 
 /* Function table (exported) */
 
+static unsigned char cur_colors[16*6];
+
+static int laguna_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
+    unsigned long *b3;
+    int i, j; 
+    unsigned int l1, l2;
+    
+    switch(cmd){
+        case CURSOR_INIT:
+            return 1;
+        case CURSOR_HIDE:
+            laguna_writew(0xe6,laguna_readw(0xe6)&0xfffe);
+            break;
+        case CURSOR_SHOW:
+            laguna_writew(0xe6,laguna_readw(0xe6)|1);
+            break;
+        case CURSOR_POSITION:
+            laguna_writew(0xe0,p1);
+            laguna_writew(0xe2,p2);
+            break;
+        case CURSOR_SELECT:
+            i=laguna_memory*1024-(16-p1)*1024;
+            laguna_writew(0xb0,laguna_readw(0xb0)|8);
+            laguna_writeb(0xa8,15);
+            laguna_writeb(0xac,cur_colors[p1*6]>>2);
+            laguna_writeb(0xac,cur_colors[p1*6+1]>>2);
+            laguna_writeb(0xac,cur_colors[p1*6+2]>>2);
+            laguna_writeb(0xa8,0);
+            laguna_writeb(0xac,cur_colors[p1*6+3]>>2);
+            laguna_writeb(0xac,cur_colors[p1*6+4]>>2);
+            laguna_writeb(0xac,cur_colors[p1*6+5]>>2);
+            laguna_writew(0xb0,laguna_readw(0xb0)&~8);            
+            laguna_writew(0xe8,i>>8);
+            break;
+        case CURSOR_IMAGE:
+            i=laguna_memory*1024-(16-p1)*1024;
+            b3=(unsigned long *)p5;
+            switch(p2) {
+                case 0:
+                    cur_colors[p1*6]=(p3&0xff0000)>>16;
+                    cur_colors[p1*6+1]=(p3&0xff00)>>8;
+                    cur_colors[p1*6+2]=p3&0xff;
+                    cur_colors[p1*6+3]=(p4&0xff0000)>>16;
+                    cur_colors[p1*6+4]=(p4&0xff00)>>8;
+                    cur_colors[p1*6+5]=p4&0xff;
+                    for(j=0;j<32;j++) {
+                        l1=*(b3+j);
+                        l2=*(b3+32+j);
+                        *(unsigned long *)(LINEAR_POINTER+i+j*16)=l1&l2;
+                        *(unsigned long *)(LINEAR_POINTER+i+j*16+4)=0;
+                        *(unsigned long *)(LINEAR_POINTER+i+j*16+8)=l1;
+                        *(unsigned long *)(LINEAR_POINTER+i+j*16+12)=0;
+                    }
+                    memset(LINEAR_POINTER+i+512,0,512);
+                    break;
+            }
+            break;
+    }
+    return 0;
+}       
+
 DriverSpecs __svgalib_laguna_driverspecs =
 {
     laguna_saveregs,
@@ -537,6 +590,7 @@ DriverSpecs __svgalib_laguna_driverspecs =
     laguna_linear,
     0,				/* accelspecs, filled in during init. */
     NULL,                       /* Emulation */
+    laguna_cursor,
 };
 
 /* Initialize chipset (called after detection) */
@@ -558,6 +612,7 @@ static int laguna_init(int force, int par1, int par2)
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
     laguna_linear_base=0;
     if (!found){
+       int i,j;
        laguna_mmio_base=buf[4]&0xffffff00;
        laguna_linear_base=buf[5]&0xffffff00;
        switch(buf[0]>>16) {
@@ -571,9 +626,19 @@ static int laguna_init(int force, int par1, int par2)
                laguna_chiptype=L5465;
                break;
        }
-       laguna_memory=((__svgalib_inseq(0x14)&0x07)+1)*1024;
+       if(__svgalib_secondary)laguna_mapio();
+       MMIO_POINTER = mmap(0, 16*1024, PROT_READ | PROT_WRITE, MAP_SHARED, __svgalib_mem_fd,
+       			laguna_mmio_base);
+       i=*(MMIO_POINTER + 0x200);
+       j=*(MMIO_POINTER + 0x201);
+    
+       munmap(MMIO_POINTER, 16*1024);
+       laguna_memory = 512*(1<<((j&0xc0)>>6))*((i&7)+1);
+//       laguna_memory=((__svgalib_inseq(0x14)&0x07)+1)*1024;
     } else return -1;
 
+    if(laguna_memory<4096) pagemul<<=2;
+
     if (__svgalib_driver_report) {
 	printf("Using LAGUNA driver, %s with %iKB video ram found.\n",
             	chipnames[laguna_chiptype],laguna_memory);
