@@ -13,6 +13,10 @@
 #include <sys/io.h>
 #endif
 
+typedef unsigned int CARD32;
+typedef unsigned short CARD16;
+typedef unsigned char CARD8;
+
 /* --------------------- Macro definitions shared by library modules */
 
 /* VGA index register ports */
@@ -94,38 +98,8 @@
 #define STDVGAMODE(mode) (chipset_modeavailable(mode) == STDVGADRV)
 #define SVGAMODE(mode)   (chipset_modeavailable(mode) == SVGADRV)
 
-#ifdef __alpha__
-#include <asm/io.h>
-#include <linux/autoconf.h>
-#if defined(CONFIG_ALPHA_JENSEN)
-#define DENSE_MEM_BASE  EISA_MEM
-#define SPARSE_MEM_BASE EISA_MEM
-#define MEM_SHIFT       7
-#define MEM_TYPE_BYTE   0x00
-#define MEM_TYPE_WORD   0x20
-#define MEM_TYPE_LONG   0x60
-#elif defined(CONFIG_ALPHA_CABRIOLET)
-#define DENSE_MEM_BASE  APECS_DENSE_MEM
-#define SPARSE_MEM_BASE APECS_SPARSE_MEM
-#define MEM_SHIFT       5
-#define MEM_TYPE_BYTE   0x00
-#define MEM_TYPE_WORD   0x08
-#define MEM_TYPE_LONG   0x18
-#elif defined(CONFIG_ALPHA_NONAME)
-#define DENSE_MEM_BASE  LCA_DENSE_MEM
-#define SPARSE_MEM_BASE LCA_SPARSE_MEM
-#define MEM_SHIFT       5
-#define MEM_TYPE_BYTE   0x00
-#define MEM_TYPE_WORD   0x08
-#define MEM_TYPE_LONG   0x18
-#endif
-#define SPARSE_GRAPH_BASE (SPARSE_MEM_BASE + (0xA0000 << MEM_SHIFT))
-#define GRAPH_BASE (DENSE_MEM_BASE + 0xA0000)	/* this is wrong for */
-#define FONT_BASE  (DENSE_MEM_BASE + 0xA0000)	/* jensen type hardware */
-#else
 #define GRAPH_BASE 0xA0000
 #define FONT_BASE  0xA0000
-#endif
 #define GRAPH_SIZE 0x10000
 #define FONT_SIZE  (0x2000 * 4) /* 2.0.x kernel can use 2 512 char. fonts */
 #define GPLANE16   G640x350x16
@@ -141,10 +115,15 @@ struct info {
 
 /* --------------------- Variable definitions shared by library modules */
 
+#define BANKED_POINTER __svgalib_banked_pointer
+#define LINEAR_POINTER __svgalib_linear_pointer
+#define MMIO_POINTER __svgalib_mmio_pointer
+
 extern int __svgalib_CRT_I;		/* current CRT index register address */
 extern int __svgalib_CRT_D;		/* current CRT data register address */
 extern int __svgalib_IS1_R;		/* current input status register address */
-extern unsigned char * BANKED_MEM_POINTER, * LINEAR_MEM_POINTER, * MMIO_POINTER;
+extern unsigned char * BANKED_POINTER, * LINEAR_POINTER;
+extern unsigned char * MMIO_POINTER;
 extern unsigned long int __svgalib_banked_mem_base, __svgalib_banked_mem_size;
 extern unsigned long int __svgalib_mmio_base, __svgalib_mmio_size;
 extern unsigned long int __svgalib_linear_mem_base, __svgalib_linear_mem_size;
@@ -156,9 +135,6 @@ extern struct info infotable[];
 extern int SCREENON;		/* screen visible if != 0 */
 extern unsigned long __svgalib_graph_base;
 extern unsigned char *GM;	/* graphics memory frame */
-#ifdef __alpha__
-extern unsigned char *SM;	/* sparse graphics memory frame */
-#endif
 extern int MODEX;		/* TRUE after vga_setmodeX() */
 extern int MODEFLAGS;		/* copy of flags of current modeinfo->flags */
 
@@ -173,6 +149,7 @@ extern int __svgalib_vgacolormode;
 extern unsigned char __svgalib_novga;
 extern unsigned char __svgalib_textprog;
 extern unsigned char __svgalib_secondary;
+extern unsigned char __svgalib_emulatepage;
 extern unsigned char __svgalib_novccontrol;
 extern unsigned char __svgalib_m_ignore_dx;
 extern unsigned char __svgalib_m_ignore_dy;
@@ -229,35 +206,6 @@ extern void __svgalib_releasevt_signal(int n);
 extern void __svgalib_acquirevt_signal(int n);
 #endif
 
-#ifdef __alpha__
-
-#include <asm/io.h>
-
-extern unsigned long vga_readb(unsigned long base, unsigned long off);
-extern unsigned long vga_readw(unsigned long base, unsigned long off);
-extern void vga_writeb(unsigned char b, unsigned long base, unsigned long off);
-extern void vga_writew(unsigned short b, unsigned long base, unsigned long off);
-
-#define gr_readb(off)		(vga_readb((unsigned long)SM, (off)))
-#define gr_readw(off)		(vga_readw((unsigned long)SM, (off)))
-
-#if defined(CONFIG_ALPHA_JENSEN)
-#define gr_readl(off)		(vga_readl((unsigned long)SM, (off)))
-#else
-#define gr_readl(off)		(*(((unsigned int*)GM)+(off)))
-#endif
-
-#define gr_writeb(v,off)	(vga_writeb((v), (unsigned long)SM, (off)))
-#define gr_writew(v,off)	(vga_writew((v), (unsigned long)SM, (off)))
-
-#if defined(CONFIG_ALPHA_JENSEN)
-#define gr_writel(v,off)	(vga_writel((v), (unsigned long)SM, (off)))
-#else
-#define gr_writel(v,off)	(*(((unsigned int*)GM)+(off)) = (v))
-#endif
-
-#else
-
 #define gr_readb(off)		(((volatile unsigned char *)GM)[(off)])
 #define gr_readw(off)		(*(volatile unsigned short*)((GM)+(off)))
 #define gr_readl(off)		(*(volatile unsigned long*)((GM)+(off)))
@@ -265,8 +213,6 @@ extern void vga_writew(unsigned short b, unsigned long base, unsigned long off);
 #define gr_writew(v,off)	(*(unsigned short*)((GM)+(off)) = (v))
 #define gr_writel(v,off)	(*(unsigned long*)((GM)+(off)) = (v))
 
-#endif /* alpha */
-
 extern void port_out(int value, int port);
 extern void port_outw(int value, int port);
 extern void port_outl(int value, int port);
@@ -312,6 +258,14 @@ extern unsigned char *__svgalib_give_graph_blue(void);
 
 #endif
 
+//#define DEBUG_TRACE
+
+#ifdef DEBUG_TRACE
+#define DTP(x) fprintf x
+#else
+#define DTP(x)
+#endif
+
 
 #endif /* _LIBVGA_H */
 
