@@ -84,6 +84,8 @@ static const unsigned short cflag[] = {
     CS7,       						/* IntelliMouse (PS/2) */
     (CS7 | CREAD | CLOCAL | HUPCL),			/* plug'n'pray */
     (CS8 | CREAD | CLOCAL | HUPCL),			/* Wacom Graphire tablet/mouse */
+    0,							/* DRMOUSE4DS */
+    0,							/* IntelliMouse Explorer (PS/2) */
 };
 
 static const unsigned char proto[][5] =
@@ -103,6 +105,8 @@ static const unsigned char proto[][5] =
     {0xc8, 0x08, 0x00, 0x00, 4},	/* IntelliMouse (PS/2) */
     {0x40, 0x40, 0x40, 0x00, 3},	/* pnp */
     {0x80, 0x80, 0x80, 0x00, 7},	/* Wacom Graphire */
+    {0xc8, 0x08, 0x00, 0x00, 4},	/* DRMOUSE4DS (Digital Research 2-wheel PS/2) */
+    {0xc8, 0x08, 0x00, 0x00, 4},        /* IntelliMouse Explorer (PS/2) */
 };
 
 static void ms_setspeed(const int old, const int new,
@@ -170,7 +174,7 @@ static char *mouse_config_options[] = {
   "mouse_fake_kbd_event", "mouse_wheel_steps", NULL
 };
 
-static char* mouse_process_option(int option, int mode) {
+static char* mouse_process_option(int option, int mode, char **nptr) {
 
  char *ptr;
  int newmtype;
@@ -185,7 +189,7 @@ static char* mouse_process_option(int option, int mode) {
     char  **cptr;
 
      case 0: /* mouse_accel_type */
-	if ( (ptr = strtok(NULL, " ")) == 0 ) {
+	if ( (ptr = __svgalib_token(nptr)) == 0 ) {
      param_needed:
 	  fprintf(stderr, "svgalib: mouse-config: \'%s\' requires a parameter\n",
 	  				mouse_config_options[option]);
@@ -206,7 +210,7 @@ static char* mouse_process_option(int option, int mode) {
 	fptr = &m_accel_mult;
 
    process_float:
-	if ( (ptr = strtok((NULL), " ")) == 0) {
+	if ( (ptr = __svgalib_token(nptr)) == 0) {
 	  goto param_needed;
 	} else {
 	 char *endptr;
@@ -226,7 +230,7 @@ static char* mouse_process_option(int option, int mode) {
 	iptr = &m_accel_thresh;
 
    process_int:
-	if ( (ptr = strtok((NULL), " ")) == 0) {
+	if ( (ptr = __svgalib_token(nptr)) == 0) {
 	  goto param_needed;
 	} else {
 	 char *endptr;
@@ -287,7 +291,7 @@ static char* mouse_process_option(int option, int mode) {
          cptr = &m_fake_upkeyname;
          iptr = &m_fake_upscancode;
          while(cptr) {
-             if ( (ptr = strtok(NULL, " ")) == 0 )
+             if ( (ptr = __svgalib_token(nptr)) == 0 )
                  goto param_needed;
 
 #ifdef DEBUG_WHEEL
@@ -329,7 +333,7 @@ static char* mouse_process_option(int option, int mode) {
 	goto process_int;
 
    }
-   return strtok(NULL, " ");
+   return __svgalib_token(nptr);
 }
 
 static int ms_init(void)
@@ -439,7 +443,9 @@ static int ms_init(void)
     if ((__svgalib_mouse_fd = open(m_dev, O_RDWR | O_NDELAY )) < 0)
 	return -1;
 
-    if (m_type == MOUSE_BUSMOUSE || m_type == MOUSE_PS2 || m_type == MOUSE_IMPS2 || m_type == MOUSE_GPM)
+    if (m_type == MOUSE_BUSMOUSE || m_type == MOUSE_PS2 
+    		|| m_type == MOUSE_IMPS2 || m_type == MOUSE_GPM
+		|| m_type == MOUSE_DRMOUSE4DS || m_type == MOUSE_EXPPS2)
 	m_modem_ctl = 0;
 
     /* If no signal will change there is no need to restore
@@ -497,9 +503,11 @@ static int ms_init(void)
     	usleep(75000);
     	/* Start sending coordinates */
     	write(__svgalib_mouse_fd, "\rST\r", 4);
-    } else if (m_type == MOUSE_IMPS2) {
+    } else if (m_type == MOUSE_IMPS2 || m_type == MOUSE_DRMOUSE4DS) {
 	/* Initialize the mouse into wheel mode */
 	write(__svgalib_mouse_fd, "\363\310\363\144\363\120", 6);
+    } else if (m_type == MOUSE_EXPPS2) {
+	write(__svgalib_mouse_fd, "\363\310\363\310\363\120", 6);
     } else if (m_type == MOUSE_PNP) {
     	/* Need to do this termios stuff here, by hand, ms_setspeed won't 
 	   work with pnp */
@@ -558,7 +566,17 @@ static int ms_init(void)
             MOUSE_RESETBUTTON;
         mouse_caps.axes = MOUSE_6DIM;
         break;
-        
+    
+    case MOUSE_DRMOUSE4DS:
+    	/* X, Y, RX and RY (two wheels), 3 buttons */
+	mouse_caps.axes = MOUSE_2DIM | MOUSE_RXDIM | MOUSE_RYDIM;
+	mouse_caps.buttons = MOUSE_LEFTBUTTON | MOUSE_RIGHTBUTTON | MOUSE_MIDDLEBUTTON;
+	mouse_caps.info = MOUSE_INFO_WHEEL;
+	break;
+
+    case MOUSE_EXPPS2:
+	mouse_caps.buttons = MOUSE_FIFTHBUTTON;	    
+	
     case MOUSE_INTELLIMOUSE:
     case MOUSE_IMPS2:
     case MOUSE_WACOM_GRAPHIRE:
@@ -735,7 +753,9 @@ static int get_ms_event(int wait) {
 	goto handle_packets;
     }
     /* Check whether it's a valid data packet. */
-    if ((m_type != MOUSE_PS2)&&(m_type != MOUSE_IMPS2)&&(m_type != MOUSE_SPACEBALL)&&(m_type != MOUSE_WACOM_GRAPHIRE))
+    if ((m_type != MOUSE_PS2)&&(m_type != MOUSE_IMPS2)
+	&&(m_type != MOUSE_EXPPS2)&&(m_type != MOUSE_SPACEBALL)
+    	&&(m_type != MOUSE_WACOM_GRAPHIRE) && (m_type != MOUSE_DRMOUSE4DS) )
 	for (j = 1; j < proto[m_type][4]; j++)
 	    if ((buf[i + j] & proto[m_type][2]) != proto[m_type][3]
 		|| buf[i + j] == 0x80) {
@@ -903,6 +923,58 @@ static int get_ms_event(int wait) {
 	dx = (buf[i] & 0x10) ? buf[i + 1] - 256 : buf[i + 1];
 	dy = (buf[i] & 0x20) ? -(buf[i + 2] - 256) : -buf[i + 2];
 	break;
+    case MOUSE_DRMOUSE4DS:
+    /* Digital Research 4-Axis mouse - like the PS/2 IntelliMouse, but 
+       has two wheels.  */
+        /* This bit modified from the gpm 1.13 imps2 patch by Tim Goodwin */
+        but = ((buf[i] & 1) << 2) /* left */
+            | ((buf[i] & 6) >> 1); /* middle and right */
+        dx = (buf[i] & 0x10) ? buf[i + 1] - 256 : buf[i + 1];
+        dy = (buf[i] & 0x20) ? -(buf[i + 2] - 256) : -buf[i + 2];
+
+        /* Did we turn the wheel? */
+        if((wheel = buf[i + 3]) != 0) {
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, " Wheel turned (0x%02x)", wheel);
+#endif
+	/* RX reports as 1 or F, RY reports as 2 or E */
+	/* both never report at the same time, which makes life easier */
+	if (m_wheel_delta) 
+	{
+		switch (wheel & 0x0F)
+		{
+		case 1:		drx = m_wheel_delta; break;
+		case 15:	drx = -m_wheel_delta; break;
+		case 2:		dry = -m_wheel_delta; break;
+		case 14:	dry = m_wheel_delta; break;
+		};
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; RX axis delta = %d : RY axis delta = %d", drx, dry);
+#endif
+
+            /* Are fake keyboard events enabled? */
+            if(m_fake_kbd_events && __svgalib_keyboard_eventhandler) {
+                /* Fake keypresses... */
+                int key = ((drx+dry < 0) ? m_fake_upscancode : m_fake_downscancode);
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; pressing scancode %d", key);
+#endif
+                 __svgalib_keyboard_eventhandler(key, 1); /* press */
+                /* todo: delay */
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, ", releasing scancode %d", key);
+#endif
+                __svgalib_keyboard_eventhandler(key, 0); /* release */
+            }
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, ".\n");
+#endif
+
+
+	}
+    }
+    break;
+    
     case MOUSE_IMPS2:           /* PS/2 IntelliMouse */
         /* This bit modified from the gpm 1.13 imps2 patch by Tim Goodwin */
         but = ((buf[i] & 1) << 2) /* left */
@@ -942,6 +1014,47 @@ static int get_ms_event(int wait) {
 #endif
         }
         break;
+    
+    case MOUSE_EXPPS2:           /* PS/2 IntelliMouse Explorer */
+        /* This bit modified from the gpm 1.13 imps2 patch by Tim Goodwin */
+        but = ((buf[i] & 1) << 2) /* left */
+            | ((buf[i] & 6) >> 1) /* middle and right */
+	    | ((buf[i+3] & 48) >> 1);
+        dx = (buf[i] & 0x10) ? buf[i + 1] - 256 : buf[i + 1];
+        dy = (buf[i] & 0x20) ? -(buf[i + 2] - 256) : -buf[i + 2];
+
+        /* Did we turn the wheel? */
+        if(((wheel = buf[i + 3]) & 15) != 0) {
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, " Wheel turned (0x%02x)", wheel);
+#endif
+            /* RX-axis */
+            if(m_wheel_delta) {
+                drx = ((wheel > 7) ? (-m_wheel_delta) : m_wheel_delta);
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; RX axis delta = %d", drx);
+#endif
+            }
+            /* Are fake keyboard events enabled? */
+            if(m_fake_kbd_events && __svgalib_keyboard_eventhandler) {
+                /* Fake keypresses... */
+                int key = ((wheel > 7) ? m_fake_upscancode : m_fake_downscancode);
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; pressing scancode %d", key);
+#endif
+                 __svgalib_keyboard_eventhandler(key, 1); /* press */
+                /* todo: delay */
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, ", releasing scancode %d", key);
+#endif
+                __svgalib_keyboard_eventhandler(key, 0); /* release */
+            }
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, ".\n");
+#endif
+        }
+        break;
+
     case MOUSE_SPACEBALL:
 
 	switch (buf[i]) {
