@@ -55,10 +55,10 @@ enum {
    	MX86250 = 0, MX86251
 };
 
-static int pal=0, palette[768];
+static int pal=1, palette[768];
 
 static int findcolor(int rgb) {
-   int i,j,k,l;
+   int i,j,k,l=0;
 
    if(pal)vga_getpalvec(0,256,palette);
    pal=0;
@@ -188,10 +188,7 @@ static int mx_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
@@ -360,8 +357,7 @@ static int mx_setmode(int mode, int prv_mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    if ((mode < G640x480x256 /*&& mode != G320x200x256*/)
-	|| mode == G720x348x2) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
 
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
@@ -400,9 +396,9 @@ static void mx_unlock(void)
 {
     int vgaIOBase, temp;
 
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
+    vgaIOBase = (port_in(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
     outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
+    temp = port_in(vgaIOBase + 5);
     outb(vgaIOBase + 5, temp & 0x7F);
     
     __svgalib_outCR(0x19, 0x88);
@@ -412,9 +408,9 @@ static void mx_lock(void)
 {
     int vgaIOBase, temp;
 
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
+    vgaIOBase = (port_in(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
     outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
+    temp = port_in(vgaIOBase + 5);
     outb(vgaIOBase + 5, temp & 0x7F);
     
     __svgalib_outCR(0x19, 0x0);
@@ -577,7 +573,7 @@ static int mx_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
                                 l1<<=1;
                                 l2<<=1;
                             }
-                            *(LINEAR_MEM_POINTER+i+8*j+k)=c;
+                            *(LINEAR_POINTER+i+8*j+k)=c;
                         }
                     }
                     break;
@@ -673,7 +669,7 @@ static int mx_init(int force, int par1, int par2)
     cardspecs->maxPixelClock24bpp = 160000;
     cardspecs->maxPixelClock32bpp = 160000;
     cardspecs->flags = INTERLACE_DIVIDE_VERT | CLOCK_PROGRAMMABLE;
-    cardspecs->maxHorizontalCrtc = 2040;
+    cardspecs->maxHorizontalCrtc = 4088;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->mapClock = mx_map_clock;
