@@ -91,8 +91,8 @@ Note:
 
 #define DACDelay \
 	{ \
-		unsigned char temp = inb(vgaIOBase + 0x0A); \
-		temp = inb(vgaIOBase + 0x0A); \
+		unsigned char temp = port_in(vgaIOBase + 0x0A); \
+		temp = port_in(vgaIOBase + 0x0A); \
 	}
 
 
@@ -114,7 +114,7 @@ static void neo_setpage(int page)
   fprintf(stderr, "neo_setpage: %d\n", page);
 #endif
   outb(GRAX, 0x11);
-  outw(GRAX, ((inb(GRAX+1) & 0xfc) << 8) | 0x11);
+  outw(GRAX, ((port_in(GRAX+1) & 0xfc) << 8) | 0x11);
   outw(GRAX, (page << 10) | 0x15); /* set read/write bank */
 }
 
@@ -124,7 +124,7 @@ static void neo_setrdpage(int page)
   fprintf(stderr, "neo_setrdpage: %d\n", page);
 #endif
   outb(GRAX, 0x11);
-  outw(GRAX, (((inb(GRAX+1) & 0xfc) | 0x01) << 8) | 0x11);
+  outw(GRAX, (((port_in(GRAX+1) & 0xfc) | 0x01) << 8) | 0x11);
   outw(GRAX, (page << 10) | 0x15); /* set read bank */
 }
 
@@ -134,7 +134,7 @@ static void neo_setwrpage(int page)
   fprintf(stderr, "neo_setwrpage: %d\n", page);
 #endif
   outb(GRAX, 0x11);
-  outw(GRAX, (((inb(GRAX+1) & 0xfc) | 0x01) << 8) | 0x11);
+  outw(GRAX, (((port_in(GRAX+1) & 0xfc) | 0x01) << 8) | 0x11);
   outw(GRAX, (page << 10) | 0x16); /* set write bank */
 }
 
@@ -186,62 +186,62 @@ static int neo_saveregs(unsigned char regs[])
   neo_unlock();
   outw(GRAX, 0x0015);   /* bank#0 */
 
-  outb(GRAX, 0x0a); regs[GeneralLockReg] = inb(GRAX + 1);
-  outb(GRAX, 0x0e); regs[ExtCRTDispAddr] = inb(GRAX + 1);
-  outb(GRAX, 0x0f); regs[ExtCRTOffset] = inb(GRAX + 1);
-  outb(GRAX, 0x10); regs[SysIfaceCntl1] = inb(GRAX + 1);
-  outb(GRAX, 0x11); regs[SysIfaceCntl2] = inb(GRAX + 1);
-  outb(GRAX, 0x15); regs[SingleAddrPage] = inb(GRAX + 1);
-  outb(GRAX, 0x16); regs[DualAddrPage] = inb(GRAX+1);
-  outb(GRAX, 0x20); regs[PanelDispCntlReg1] = inb(GRAX+1);
-  outb(GRAX, 0x25); regs[PanelDispCntlReg2] = inb(GRAX+1);
-  outb(GRAX, 0x30); regs[PanelDispCntlReg3] = inb(GRAX+1);
-  outb(GRAX, 0x28); regs[PanelVertCenterReg1] = inb(GRAX+1);
-  outb(GRAX, 0x29); regs[PanelVertCenterReg2] = inb(GRAX+1);
-  outb(GRAX, 0x2a); regs[PanelVertCenterReg3] = inb(GRAX+1);
+  outb(GRAX, 0x0a); regs[GeneralLockReg] = port_in(GRAX + 1);
+  outb(GRAX, 0x0e); regs[ExtCRTDispAddr] = port_in(GRAX + 1);
+  outb(GRAX, 0x0f); regs[ExtCRTOffset] = port_in(GRAX + 1);
+  outb(GRAX, 0x10); regs[SysIfaceCntl1] = port_in(GRAX + 1);
+  outb(GRAX, 0x11); regs[SysIfaceCntl2] = port_in(GRAX + 1);
+  outb(GRAX, 0x15); regs[SingleAddrPage] = port_in(GRAX + 1);
+  outb(GRAX, 0x16); regs[DualAddrPage] = port_in(GRAX+1);
+  outb(GRAX, 0x20); regs[PanelDispCntlReg1] = port_in(GRAX+1);
+  outb(GRAX, 0x25); regs[PanelDispCntlReg2] = port_in(GRAX+1);
+  outb(GRAX, 0x30); regs[PanelDispCntlReg3] = port_in(GRAX+1);
+  outb(GRAX, 0x28); regs[PanelVertCenterReg1] = port_in(GRAX+1);
+  outb(GRAX, 0x29); regs[PanelVertCenterReg2] = port_in(GRAX+1);
+  outb(GRAX, 0x2a); regs[PanelVertCenterReg3] = port_in(GRAX+1);
   if (NeoChipset != PCI_CHIP_NM2070){
-    outb(GRAX, 0x32); regs[PanelVertCenterReg4] = inb(GRAX+1);
-    outb(GRAX, 0x33); regs[PanelHorizCenterReg1] = inb(GRAX+1);
-    outb(GRAX, 0x34); regs[PanelHorizCenterReg2] = inb(GRAX+1);
-    outb(GRAX, 0x35); regs[PanelHorizCenterReg3] = inb(GRAX+1);
+    outb(GRAX, 0x32); regs[PanelVertCenterReg4] = port_in(GRAX+1);
+    outb(GRAX, 0x33); regs[PanelHorizCenterReg1] = port_in(GRAX+1);
+    outb(GRAX, 0x34); regs[PanelHorizCenterReg2] = port_in(GRAX+1);
+    outb(GRAX, 0x35); regs[PanelHorizCenterReg3] = port_in(GRAX+1);
   }
   if (NeoChipset == PCI_CHIP_NM2160){
-    outb(GRAX, 0x36); regs[PanelHorizCenterReg4] = inb(GRAX+1);
+    outb(GRAX, 0x36); regs[PanelHorizCenterReg4] = port_in(GRAX+1);
   }
   if (NeoChipset == PCI_CHIP_NM2200){
-    outb(GRAX, 0x36); regs[PanelHorizCenterReg4] = inb(GRAX+1);
-    outb(GRAX, 0x37); regs[PanelVertCenterReg5]  = inb(GRAX+1);
-    outb(GRAX, 0x38); regs[PanelHorizCenterReg5] = inb(GRAX+1);
+    outb(GRAX, 0x36); regs[PanelHorizCenterReg4] = port_in(GRAX+1);
+    outb(GRAX, 0x37); regs[PanelVertCenterReg5]  = port_in(GRAX+1);
+    outb(GRAX, 0x38); regs[PanelHorizCenterReg5] = port_in(GRAX+1);
   }
-  outb(GRAX, 0x90); regs[ExtColorModeSelect] = inb(GRAX+1);
-  outb(GRAX, 0x9B); regs[VCLK3NumeratorLow]  = inb(GRAX+1);
+  outb(GRAX, 0x90); regs[ExtColorModeSelect] = port_in(GRAX+1);
+  outb(GRAX, 0x9B); regs[VCLK3NumeratorLow]  = port_in(GRAX+1);
   if (NeoChipset == PCI_CHIP_NM2200){
-    outb(GRAX, 0x8F); regs[VCLK3NumeratorHigh] = inb(GRAX+1);
+    outb(GRAX, 0x8F); regs[VCLK3NumeratorHigh] = port_in(GRAX+1);
   }
-  outb(GRAX, 0x9F); regs[VCLK3Denominator] = inb(GRAX+1);
+  outb(GRAX, 0x9F); regs[VCLK3Denominator] = port_in(GRAX+1);
 
   regs[EXT_SAVED] = TRUE;
-  outb(vgaIOBase + 4, 0x25); regs[EXTCR + 0x25] = inb(vgaIOBase + 5);
-  outb(vgaIOBase + 4, 0x2F); regs[EXTCR + 0x2F] = inb(vgaIOBase + 5);
+  outb(vgaIOBase + 4, 0x25); regs[EXTCR + 0x25] = port_in(vgaIOBase + 5);
+  outb(vgaIOBase + 4, 0x2F); regs[EXTCR + 0x2F] = port_in(vgaIOBase + 5);
   for (i = 0x40; i <= 0x59; i++){
-    outb(vgaIOBase + 4, i); regs[EXTCR + i] = inb(vgaIOBase + 5);
+    outb(vgaIOBase + 4, i); regs[EXTCR + i] = port_in(vgaIOBase + 5);
   }
   for (i = 0x60; i <= 0x69; i++){
-    outb(vgaIOBase + 4, i); regs[EXTCR + i] = inb(vgaIOBase + 5);
+    outb(vgaIOBase + 4, i); regs[EXTCR + i] = port_in(vgaIOBase + 5);
   }
   for (i = 0x70; i <= NEO_EXT_CR_MAX; i++){
-    outb(vgaIOBase + 4, i); regs[EXTCR + i] = inb(vgaIOBase + 5);
+    outb(vgaIOBase + 4, i); regs[EXTCR + i] = port_in(vgaIOBase + 5);
   }
 
   for (i = 0x0A; i <= NEO_EXT_GR_MAX; i++){
-    outb(GRAX, i); regs[EXTGR + i] = inb(GRAX+1);
+    outb(GRAX, i); regs[EXTGR + i] = port_in(GRAX+1);
   }
 
   /* DAC */
   outb(0x3C6,0xFF); /* mask */
   outb(0x3C7,0x00); /* read address */
   for (i = 0; i < 768; i++){
-    regs[DAC + i] = inb(0x3C9);
+    regs[DAC + i] = port_in(0x3C9);
     DACDelay;
   }
 
@@ -264,7 +264,7 @@ static void neo_setregs(const unsigned char regs[], int mode)
   outb(GRAX, 0x0a); outb(GRAX+1, regs[GeneralLockReg]);
 
   /* set color mode first */
-  outb(GRAX, 0x90); temp = inb(GRAX+1);
+  outb(GRAX, 0x90); temp = port_in(GRAX+1);
   switch (NeoChipset){
   case PCI_CHIP_NM2070:
     temp &= 0xF0; /* Save bits 7:4 */
@@ -280,7 +280,7 @@ static void neo_setregs(const unsigned char regs[], int mode)
 
   /* Disable horizontal and vertical graphics and text expansions */
   outb(GRAX, 0x25);
-  temp = inb(GRAX+1);
+  temp = port_in(GRAX+1);
   outb(GRAX, 0x25);
   temp &= 0x39;
   outb(GRAX+1, temp);
@@ -297,7 +297,7 @@ static void neo_setregs(const unsigned char regs[], int mode)
 
   outb(GRAX, 0x0E); outb(GRAX+1, regs[ExtCRTDispAddr]);
   outb(GRAX, 0x0F); outb(GRAX+1, regs[ExtCRTOffset]);
-  outb(GRAX, 0x10); temp = inb(GRAX+1);
+  outb(GRAX, 0x10); temp = port_in(GRAX+1);
   temp &= 0x0F; /* Save bits 3:0 */
   temp |= (regs[SysIfaceCntl1] & ~0x0F);
   outb(GRAX, 0x10); outb(GRAX+1, temp);
@@ -305,7 +305,7 @@ static void neo_setregs(const unsigned char regs[], int mode)
   outb(GRAX, 0x11); outb(GRAX+1, regs[SysIfaceCntl2]);
   outb(GRAX, 0x15); outb(GRAX+1, regs[SingleAddrPage]);
   outb(GRAX, 0x16); outb(GRAX+1, regs[DualAddrPage]);
-  outb(GRAX, 0x20); temp = inb(GRAX+1);
+  outb(GRAX, 0x20); temp = port_in(GRAX+1);
   switch (NeoChipset){
   case PCI_CHIP_NM2070:
     temp &= 0xFC; /* Save bits 7:2 */
@@ -322,13 +322,13 @@ static void neo_setregs(const unsigned char regs[], int mode)
     break;
   }
   outb(GRAX, 0x20); outb(GRAX+1, temp);
-  outb(GRAX, 0x25); temp = inb(GRAX+1);
+  outb(GRAX, 0x25); temp = port_in(GRAX+1);
   temp &= 0x38; /* Save bits 5:3 */
   temp |= (regs[PanelDispCntlReg2] & ~0x38);
   outb(GRAX, 0x25); outb(GRAX+1, temp);
 
   if (NeoChipset != PCI_CHIP_NM2070){
-    outb(GRAX, 0x30); temp = inb(GRAX+1);
+    outb(GRAX, 0x30); temp = port_in(GRAX+1);
     temp &= 0xEF; /* Save bits 7:5 and bits 3:0 */
     temp |= (regs[PanelDispCntlReg3] & ~0xEF);
     outb(GRAX, 0x30); outb(GRAX+1, temp);
@@ -358,7 +358,7 @@ static void neo_setregs(const unsigned char regs[], int mode)
 #if 0
   outb(GRAX, 0x9B); outb(GRAX+1, regs[VCLK3NumeratorLow]);
   if (NeoChipset == PCI_CHIP_NM2200){
-    outb(GRAX, 0x8F); temp = inb(GRAX+1);
+    outb(GRAX, 0x8F); temp = port_in(GRAX+1);
     temp &= 0x0F; /* Save bits 3:0 */
     temp |= (regs[VCLK3NumeratorHigh] & ~0x0F);
     outb(GRAX, 0x8F); outb(GRAX+1, temp);
@@ -450,10 +450,8 @@ static int neo_modeavailable(int mode)
 #ifdef DEBUG
     fprintf(stderr, "neo_modeavaailable: %d\n", mode);
 #endif
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
@@ -664,7 +662,7 @@ static int neo_setmode(int mode, int prv_mode)
 #ifdef DEBUG
     fprintf(stderr, "neo_setmode: %d\n", mode);
 #endif
-    if ((mode < G640x480x256)||(mode==G720x348x2)) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
 
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
@@ -707,7 +705,7 @@ static void neo_unlock(void)
     fprintf(stderr, "neo_unlock\n");
 #endif
     outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
+    temp = port_in(vgaIOBase + 5);
     outb(vgaIOBase + 5, temp & 0x7F);
     outw(GRAX, 0x2609); /* Unlock NeoMagic registers */
 }
@@ -720,7 +718,7 @@ static void neo_lock(void)
     fprintf(stderr, "neo_lock\n");
 #endif
     outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
+    temp = port_in(vgaIOBase + 5);
     outb(vgaIOBase + 5, temp | 0x80);
     outw(GRAX, 0x0009); /* Lock NeoMagic registers */
 }
@@ -773,7 +771,7 @@ static void neo_setdisplaystart(int address)
   outw(vgaIOBase + 4, ((address & 0x00FF) << 8) | 0x0D);
 
   outb(GRAX, 0x0E);
-  oldExtCRTDispAddr = inb(GRAX+1);
+  oldExtCRTDispAddr = port_in(GRAX+1);
   outw(GRAX,
        ((((address >> 16) & 0x07) | (oldExtCRTDispAddr & 0xf8)) << 8) | 0x0E);
 }
@@ -866,7 +864,7 @@ static int neo_init(int force, int par1, int par2)
     int _ioperm=0;
     int w;
 
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
+    vgaIOBase = (port_in(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
     neo_unlock();
     if (force) {
 	neo_memory = par1;
@@ -946,7 +944,7 @@ static int neo_init(int force, int par1, int par2)
 
     /* Determine panel width -- used in NeoValidMode. */
     outb(GRAX, 0x20);
-    w = inb(GRAX+1);
+    w = port_in(GRAX+1);
     switch ((w & 0x18) >> 3){
     case 0x00:
 	NeoPanelWidth  = 640;
