@@ -28,13 +28,12 @@ there are still the following problems:
 #include "timing.h"
 #include "vgaregs.h"
 #include "interface.h"
-#include "accel.h"
 #include "nv3ref.h"
 #include "nvreg.h"
 #include "vgapci.h"
 
 #define NV3REG_SAVE(i) (VGA_TOTAL_REGS+i)
-#define NV3_TOTAL_REGS (VGA_TOTAL_REGS + 24 + 33 )
+#define NV3_TOTAL_REGS (VGA_TOTAL_REGS + 24 + 36 )
 #define M_MIN 7
 #define M_MAX 13
 
@@ -59,8 +58,6 @@ static volatile unsigned  *nvPVGA2Port;
 static int nv3_init(int, int, int);
 static void nv3_unlock(void);
 
-void __svgalib_nv3accel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
-
 static int nv3_memory,nv3_chiptype;
 static int nv3_is_linear;
 static CardSpecs *cardspecs;
@@ -89,7 +86,7 @@ static void nv3_getmodeinfo(int mode, vga_modeinfo *modeinfo)
 {
    
    if(modeinfo->colors==16)return;
-   
+
    modeinfo->maxpixels = nv3_memory*1024/modeinfo->bytesperpixel;
    modeinfo->maxlogicalwidth = 4088;
    modeinfo->startaddressrange = nv3_memory * 1024 - 1;
@@ -162,7 +159,7 @@ static void nv3_setregs(const unsigned char regs[], int mode)
    __svgalib_outcrtc(NV_PCRTC_PIXEL,regs[NV3REG_SAVE(3)]);
    __svgalib_outcrtc(NV_PCRTC_HORIZ_EXTRA,regs[NV3REG_SAVE(4)]); 
    __svgalib_outcrtc(NV_PCRTC_FIFO_CONTROL,regs[NV3REG_SAVE(5)]); 
-/*  PCRTC_Write(FIFO,regs[NV3REG_SAVE(6)]); */
+   __svgalib_outcrtc(NV_PCRTC_FIFO,regs[NV3REG_SAVE(6)]); 
 
    __svgalib_outcrtc(0x1c,regs[88]); /* this enables banking at 0xa0000 */
    __svgalib_outcrtc(0x1d,regs[89]); 
@@ -192,12 +189,7 @@ static int nv3_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
-    if(modeinfo->bitsPerPixel==24)return 0;
-    
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
@@ -205,8 +197,11 @@ static int nv3_modeavailable(int mode)
 	return 0;
     
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-   
-/*    if(modeinfo->greenWeight==5)return 0; */
+
+    if(modeinfo->bitsPerPixel==24) {
+        free(modeinfo);
+        return 0;   
+    }
 
     modetiming = malloc(sizeof(ModeTiming));
     if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
@@ -214,9 +209,9 @@ static int nv3_modeavailable(int mode)
 	free(modeinfo);
 	return 0;
     }
+
     free(modetiming);
     free(modeinfo);
-
     return SVGADRV;
 }
 
@@ -290,63 +285,81 @@ static int CalculateCRTC(ModeTiming *mode, ModeInfo *modeinfo, unsigned char *mo
 
 static void nv3_initializemode(unsigned char *moderegs,
 			    ModeTiming * modetiming, ModeInfo * modeinfo, int mode)
-{ long k;
-  int m,n,p;
-  float clockIn=(float)modetiming->pixelClock;
-  float clockOut;
-  int pixelDepth;
+{ 
+    long k;
+    unsigned int config0=0;
+    
+    int m,n,p;
+    float clockIn=(float)modetiming->pixelClock;
+    float clockOut;
+    int pixelDepth;
   
-   nv3_saveregs(moderegs);
-   __svgalib_setup_VGA_registers(moderegs, modetiming, modeinfo);
-   if(!NV3ClockSelect(clockIn,&clockOut,&m,&n,&p)) {return ;}
-   k=PRAMDAC_Val(VPLL_COEFF_NDIV,n) | PRAMDAC_Val(VPLL_COEFF_MDIV,m) | PRAMDAC_Val(VPLL_COEFF_PDIV,p);
-   moderegs[NV3REG_SAVE(12)] = k&0xff;
-   moderegs[NV3REG_SAVE(13)] = (k >> 8)&0xff;
-   moderegs[NV3REG_SAVE(14)] = (k >> 16) & 0xff ;
-   moderegs[NV3REG_SAVE(15)] = ( k >> 24 ) & 0xff ;
+    nv3_saveregs(moderegs);
+    __svgalib_setup_VGA_registers(moderegs, modetiming, modeinfo);
+    if(!NV3ClockSelect(clockIn,&clockOut,&m,&n,&p)) {return ;}
+    k=PRAMDAC_Val(VPLL_COEFF_NDIV,n) | PRAMDAC_Val(VPLL_COEFF_MDIV,m) | PRAMDAC_Val(VPLL_COEFF_PDIV,p);
+    moderegs[NV3REG_SAVE(12)] = k&0xff;
+    moderegs[NV3REG_SAVE(13)] = (k >> 8)&0xff;
+    moderegs[NV3REG_SAVE(14)] = (k >> 16) & 0xff ;
+    moderegs[NV3REG_SAVE(15)] = ( k >> 24 ) & 0xff ;
 
-   CalculateCRTC(modetiming,modeinfo,moderegs);
+    CalculateCRTC(modetiming,modeinfo,moderegs);
 
-   moderegs[NV3REG_SAVE(1)]=
-    PCRTC_Val(REPAINT1_LARGE_SCREEN,modetiming->CrtcHDisplay<1280) |
-    PCRTC_Def(REPAINT1_PALETTE_WIDTH,6BITS);
+    moderegs[NV3REG_SAVE(1)]=
+        	PCRTC_Val(REPAINT1_LARGE_SCREEN,modetiming->CrtcHDisplay<1280) |
+                PCRTC_Def(REPAINT1_PALETTE_WIDTH,6BITS);
 
 /* The new xfree driver (from nVidia) calculates those in some
    twisted way, but I leave it for now */
-  moderegs[NV3REG_SAVE(5)]=0x83;
-  moderegs[NV3REG_SAVE(6)]=0x22;
+    moderegs[NV3REG_SAVE(5)]=0x83;
+    if(modetiming->pixelClock*modeinfo->bytesPerPixel>720000) {
+        moderegs[NV3REG_SAVE(6)]=0x2f;
+    }
 
   /* PixelFormat controls how many bits per pixel. 
    * There is another register in the 
    * DAC which controls if mode is 5:5:5 or 5:6:5
    */
-  pixelDepth=(modeinfo->bitsPerPixel+1)/8;
-  if(pixelDepth>3) pixelDepth=3;
-  moderegs[NV3REG_SAVE(3)]=pixelDepth;
+    pixelDepth=(modeinfo->bitsPerPixel+1)/8;
+    if(pixelDepth>3) pixelDepth=3;
+    moderegs[NV3REG_SAVE(3)]=pixelDepth;
+    if(modetiming->flags & TVMODE) {
+        moderegs[NV3REG_SAVE(3)]|=0x80;
+        if(modetiming->flags & TVPAL) moderegs[NV3REG_SAVE(3)]|=0x40;
+    };
 
-  k=
+    k=
      PRAMDAC_Def(GENERAL_CONTROL_IDC_MODE,GAMMA)|
      PRAMDAC_Val(GENERAL_CONTROL_565_MODE,modeinfo->greenWeight==6)|
      PRAMDAC_Def(GENERAL_CONTROL_TERMINATION,37OHM)|
      PRAMDAC_Def(GENERAL_CONTROL_BPC,6BITS)|  
      PRAMDAC_Def(GENERAL_CONTROL_VGA_STATE,SEL); /* Not sure about this */
 
-   if(modeinfo->bitsPerPixel>8)k|=PRAMDAC_Def(GENERAL_CONTROL_BPC,8BITS);
+    if(modeinfo->bitsPerPixel>8)k|=PRAMDAC_Def(GENERAL_CONTROL_BPC,8BITS);
 
-   moderegs[NV3REG_SAVE(20)] = k&0xff;
-   moderegs[NV3REG_SAVE(21)] = (k >> 8)&0xff;
-   moderegs[NV3REG_SAVE(22)] = (k >> 16) & 0xff ;
-   moderegs[NV3REG_SAVE(23)] = ( k >> 24 ) & 0xff ;
+    moderegs[NV3REG_SAVE(20)] = k&0xff;
+    moderegs[NV3REG_SAVE(21)] = (k >> 8)&0xff;
+    moderegs[NV3REG_SAVE(22)] = (k >> 16) & 0xff ;
+    moderegs[NV3REG_SAVE(23)] = ( k >> 24 ) & 0xff ;
 
-   switch(nv3_chiptype){
+    switch(nv3_chiptype){
       case Riva128:
+          config0=PFB_Val(CONFIG_0_RESOLUTION,((modeinfo->lineWidth+31)/32))|
+                  PFB_Val(CONFIG_0_PIXEL_DEPTH,pixelDepth)|
+                  PFB_Def(CONFIG_0_TILING,ENABLED); 
           k=PRAMDAC_Def(PLL_COEFF_SELECT_MPLL_SOURCE,PROG)|
             PRAMDAC_Def(PLL_COEFF_SELECT_VPLL_SOURCE,PROG)|
             PRAMDAC_Def(PLL_COEFF_SELECT_VCLK_RATIO,DB2);
-          k=0x10010100;
           break;
       case RivaTNT:
+          config0=0x1114;
+          k=0x10000700;
+          break;
       case GEFORCE:
+          config0=moderegs[NV3REG_SAVE(8)]+
+          (moderegs[NV3REG_SAVE(9)]<<8)+
+          (moderegs[NV3REG_SAVE(10)]<<16)+
+          (moderegs[NV3REG_SAVE(11)]<<24);
           k=0x10000700;
           break;
    };
@@ -355,14 +368,10 @@ static void nv3_initializemode(unsigned char *moderegs,
    moderegs[NV3REG_SAVE(18)] = (k >> 16) & 0xff ;
    moderegs[NV3REG_SAVE(19)] = ( k >> 24 ) & 0xff ;
 
-   k=
-      PFB_Val(CONFIG_0_RESOLUTION,((modeinfo->lineWidth+31)/32))|
-      PFB_Val(CONFIG_0_PIXEL_DEPTH,pixelDepth)|
-      PFB_Def(CONFIG_0_TILING,ENABLED); 
-   moderegs[NV3REG_SAVE(8)] = k&0xff;
-   moderegs[NV3REG_SAVE(9)] = (k >> 8)&0xff;
-   moderegs[NV3REG_SAVE(10)] = (k >> 16) & 0xff ;
-   moderegs[NV3REG_SAVE(11)] = ( k >> 24 ) & 0xff ;
+   moderegs[NV3REG_SAVE(8)] = config0&0xff;
+   moderegs[NV3REG_SAVE(9)] = (config0 >> 8)&0xff;
+   moderegs[NV3REG_SAVE(10)] = (config0 >> 16) & 0xff ;
+   moderegs[NV3REG_SAVE(11)] = (config0 >> 24 ) & 0xff ;
 
    moderegs[88]=28;
    moderegs[109]&=0xfe; /* hide cursor */
@@ -381,11 +390,29 @@ static int nv3_setmode(int mode, int prv_mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
     int i;
-    if ((mode < G640x480x256 /*&& mode != G320x200x256*/)
-	|| mode == G720x348x2) {
-        if(nv3_chiptype==Riva128)
+
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
+        
+	unsigned int k;
+
+	if(nv3_chiptype==Riva128)
             PRAMDAC_Write(PLL_COEFF_SELECT,0x00000100);
             else PRAMDAC_Write(PLL_COEFF_SELECT,0x00000500);
+        __svgalib_outcrtc(NV_PCRTC_REPAINT0,0);
+        __svgalib_outcrtc(NV_PCRTC_REPAINT1,0x3d);
+        __svgalib_outcrtc(NV_PCRTC_EXTRA,0);
+        __svgalib_outcrtc(NV_PCRTC_PIXEL,0);
+        __svgalib_outcrtc(NV_PCRTC_HORIZ_EXTRA,0);
+        __svgalib_outcrtc(NV_PCRTC_FIFO_CONTROL,0x83);
+        __svgalib_outcrtc(0x1c,0x18);
+        __svgalib_outcrtc(0x1d,0);
+        __svgalib_outcrtc(0x1e,0);
+        __svgalib_outcrtc(0x30,0);
+        __svgalib_outcrtc(0x31,0);
+        k =  PRAMDAC_Read(GENERAL_CONTROL);
+        k &= ~0x00100000;
+        PRAMDAC_Write(GENERAL_CONTROL,k);
+
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
 
@@ -451,28 +478,24 @@ static int nv3_test(void)
       found=__svgalib_pci_find_vendor_vga(0x10de,buf,0);
       if(found)return 0;
    };
-   if ((((buf[0]>>16)&0xffff)!=0x18)&&
-       (((buf[0]>>16)&0xffff)!=0x20)&&
-       (((buf[0]>>16)&0xffff)!=0x29)&&
-       (((buf[0]>>16)&0xffff)!=0x2C)&&
-       (((buf[0]>>16)&0xffff)!=0x2D)&&
-       (((buf[0]>>16)&0xffff)!=0xA0)&&
-       (((buf[0]>>16)&0xffff)!=0x28)&&
-       (((buf[0]>>16)&0xffff)!=0x100)&&
-       (((buf[0]>>16)&0xffff)!=0x101)&&
-       (((buf[0]>>16)&0xffff)!=0x103)
-       )return 0;
+
    switch(buf[0]>>16){
       case 0x18: nv3_chiptype=Riva128; break;
       case 0x20:
-      case 0x29: case 0x2C: case 0x2D: case 0xA0:
+      case 0x29: case 0x2A: case 0x2B: case 0x2C: case 0x2D: case 0x2E: case 0x2F:
+      case 0xA0:
       case 0x28: nv3_chiptype=RivaTNT; break;
+      case 0x200: case 0x201: case 0x202: case 0x203:
+      case 0x150: case 0x151: case 0x152: case 0x153:
+      case 0x110: case 0x111: case 0x113:
       case 0x101: case 0x103:
       case 0x100: nv3_chiptype=GEFORCE; break;
-      default: nv3_chiptype=Riva128;
+      default: nv3_chiptype=-1;
    };
-   MMIOBASE=buf[4]&0xffffff00;
-   LINEARBASE=buf[5]&0xffffff00;
+
+   if(nv3_chiptype==-1)return 0;
+   MMIOBASE=0; /* let nv3_init() find those */
+   LINEARBASE=0;
    nv3_init(0,0,0);
    return 1;
 }
@@ -489,12 +512,19 @@ static void nv3_setwrpage(int page)
 
 static void nv3_setdisplaystart(int address)
 {  unsigned char byte;
+    int pan;
+
+  pan=(address&3)<<1;
   address=address >> 2;
   __svgalib_outcrtc(0x0d,address&0xff);
   __svgalib_outcrtc(0x0c,(address>>8)&0xff);
   byte=__svgalib_incrtc(NV_PCRTC_REPAINT0) & 0xe0;
   __svgalib_outcrtc(NV_PCRTC_REPAINT0,((address>>16)&0x1f)|byte);
-  
+  byte=__svgalib_incrtc(0x2D) & ~0x60;
+  __svgalib_outcrtc(0x2D,((address>>16)&0x60)|byte);
+
+  byte = __svgalib_inis1();
+  __svgalib_outatt(0x13, pan);
 }
 
 /* Set logical scanline length (usually multiple of 8) */
@@ -569,11 +599,11 @@ static int nv3_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
                             cc1<<=1;
                         }
                     }
-                    memcpy(LINEAR_MEM_POINTER+i,b3,2048);
+                    memcpy(LINEAR_POINTER+i,b3,2048);
                     free(b3);
                     break;
                 case 1: /* nvidia 1555 format  32x32 */
-                    memcpy(LINEAR_MEM_POINTER+i,p5,2048);
+                    memcpy(LINEAR_POINTER+i,p5,2048);
                     break;
             }
             break;
@@ -649,11 +679,20 @@ static int nv3_init(int force, int par1, int par2)
        switch(buf[0]>>16){
           case 0x18: nv3_chiptype=Riva128; break;
           case 0x20:
-          case 0x29: case 0x2C: case 0x2D: case 0xA0:
-          case 0x28: nv3_chiptype=RivaTNT; break;
+          case 0x28:
+          case 0x29: case 0x2A: case 0x2B: case 0x2C: case 0x2D: case 0x2E: case 0x2F:
+          case 0xA0: nv3_chiptype=RivaTNT; break;
+          case 0x250: case 0x258:
+          case 0x200: case 0x201: case 0x202: case 0x203:
+          case 0x1A0:
+          case 0x17B: case 0x17C:
+          case 0x174: case 0x175: case 0x178: case 0x17A:
+          case 0x170: case 0x171: case 0x172: case 0x173:
+          case 0x150: case 0x151: case 0x152: case 0x153:
+          case 0x110: case 0x111: case 0x112: case 0x113:
           case 0x101: case 0x103:
           case 0x100: nv3_chiptype=GEFORCE; break;
-          default: nv3_chiptype=Riva128;
+          default: nv3_chiptype=GEFORCE;
        };
        MMIOBASE=buf[4]&0xffffff00;
        LINEARBASE=buf[5]&0xffffff00;
@@ -682,7 +721,8 @@ static int nv3_init(int force, int par1, int par2)
                  if(nv3_memory==2048)nv3_memory=32768;
                  break;
           case GEFORCE:
-             	 nv3_memory=8192; /* do this later */
+              	 nv3_memory=PFB_Read(BOOT_10)*1024;
+             	 if(nv3_memory<8192)nv3_memory=8192; /* do this later */
                  break;
        }
     }
