@@ -232,7 +232,7 @@ static void oak_setregs(const unsigned char regs[], int mode)
 	OAKSetByte(OTI87_XWRITESEGMENT, regs[OTI87_XWRITESEGMENT_DATA]);
 
 	outb(SEQ_I, 0);		/* reset sync. */
-	junk = inb(SEQ_D);
+	junk = port_in(SEQ_D);
 	outw(SEQ_I, 0x00 + ((junk & 0xfd) << 8));
 
 	OAKSetByte(OTI87_CLOCK, regs[EXT + 8]);
@@ -250,7 +250,7 @@ static void oak_setregs(const unsigned char regs[], int mode)
 
 	/* doc says don't OTI-MISC unless sync. reset is off */
 	outb(SEQ_I, 0);
-	junk = inb(SEQ_D);
+	junk = port_in(SEQ_D);
 	outw(SEQ_I, 0x00 + ((junk & 0xFD) << 8));	/* now disable the timing sequencer */
 
 	OAKSetByte(OTI_MISC, regs[OTI_MISC_DATA]);
@@ -392,7 +392,7 @@ static void oak_unlock(void)
     int temp;
     /* Unprotect CRTC[0-7] */
     outb(__svgalib_CRT_I, 0x11);
-    temp = inb(__svgalib_CRT_D);
+    temp = port_in(__svgalib_CRT_D);
     outb(__svgalib_CRT_D, temp & 0x7F);
     temp = OAKGetByte(OTI_CRT_CNTL);
     OAKSetByte(OTI_CRT_CNTL, temp & 0xf0);
@@ -413,7 +413,7 @@ static void oak_lock(void)
     OAKSetByte(OTI_CRT_CNTL, oak_lockreg_save & 0xf7);
     /* Protect CRTC[0-7] */
     outb(__svgalib_CRT_I, 0x11);
-    temp = inb(__svgalib_CRT_D);
+    temp = port_in(__svgalib_CRT_D);
     outb(__svgalib_CRT_D, (temp & 0x7F) | 0x80);
 }
 
@@ -510,9 +510,9 @@ static void oak_setdisplaystart(int address)
 {
     outw(CRT_IC, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
     outw(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
-    inb(0x3da);			/* set ATC to addressing mode */
+    port_in(0x3da);			/* set ATC to addressing mode */
     outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
-    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
+    outb(ATT_IW, (port_in(ATT_R) & 0xf0) | ((address & 3) << 1));
     /* write sa0-1 to bits 1-2 */
 
     if (oak_chiptype == 87) {
@@ -622,7 +622,7 @@ static int oak_chipdetect(void)
 {
     int temp1;
 
-    temp1 = inb(OTI_INDEX);
+    temp1 = port_in(OTI_INDEX);
     temp1 &= 0xe0;
     if (temp1 == 0x40)
 	return 67;
@@ -680,7 +680,7 @@ static void oak_setlinearmode(void)
 					       __svgalib_mem_fd,
 					       0xe00000);
 */
-    linearframebuffer=LINEAR_MEM_POINTER;
+    linearframebuffer=LINEAR_POINTER;
     VGAMemory_Save = __svgalib_graph_mem;
     __svgalib_modeinfo_linearset |= IS_LINEAR;
 }
@@ -709,13 +709,13 @@ static void oak_clearlinearmode(void)
 
 static void OAK_WriteDAC(int dac_value)
 {
-    inb(DAC_REVERT);		/* reset DAC r/w pointer */
-    inb(DAC_SC11487);		/* do this four times to set register A */
-    inb(DAC_SC11487);
-    inb(DAC_SC11487);
-    inb(DAC_SC11487);
+    port_in(DAC_REVERT);		/* reset DAC r/w pointer */
+    port_in(DAC_SC11487);		/* do this four times to set register A */
+    port_in(DAC_SC11487);
+    port_in(DAC_SC11487);
+    port_in(DAC_SC11487);
     outb(DAC_SC11487, dac_value);
-    inb(DAC_REVERT);		/* and reset DAC ... */
+    port_in(DAC_REVERT);		/* and reset DAC ... */
 }
 
 /* OAKSetByte ( index, value)                           Set extended register
@@ -740,7 +740,7 @@ static void OAKSetByte(int OTI_index, int OTI_value)
 static int OAKGetByte(int OTI_index)
 {
     outb(OTI_INDEX, OTI_index);
-    return inb(OTI_R_W);
+    return port_in(OTI_R_W);
 }
 
 /* Function table (exported) */
