@@ -38,7 +38,7 @@ static void nothing(void)
 {
 }
 
-void inline _outb(unsigned port,unsigned value)
+static void inline _outb(unsigned port,unsigned value)
 {
 #ifdef DEBUG
  printf ("0x%x, 0x%x\n",port,value);
@@ -101,7 +101,7 @@ b=0;
 for (a=0x03;a<=0x15;)
  { 
  outb(SEQ_I,a);
- regs[EXT+b]=inb(SEQ_D);
+ regs[EXT+b]=port_in(SEQ_D);
  b++;
  if (a==0x03)a=0x06;
  else if (a==0x07)a=0x10;
@@ -111,13 +111,13 @@ for (a=0x03;a<=0x15;)
 for (a=0x09;a<=0x0f;a++)
  {
  outb(GRA_I,a);
- regs[EXT+b]=inb(GRA_D);
+ regs[EXT+b]=port_in(GRA_D);
  b++;
  }  
 for (a=0x29;a<=0x3e;)
  {
  outb(CRT_IC,a);
- regs[EXT+b]=inb(CRT_DC);
+ regs[EXT+b]=port_in(CRT_DC);
  b++;
  if (a==0x30)a=0x32;
  else if (a==0x32)a=0x34;
@@ -126,10 +126,10 @@ for (a=0x29;a<=0x3e;)
  }  
 /* VGA registers, not set by shitty svgalib */
 outb(CRT_IC,0x18);
-regs[EXT+b+1]=inb(CRT_DC);
+regs[EXT+b+1]=port_in(CRT_DC);
 outb(ATT_IW,0x20);
 __svgalib_delay();
-regs[EXT+b+2]=inb(ATT_IW);
+regs[EXT+b+2]=port_in(ATT_IW);
 return 28;			/* Paradise requires 28 additional registers */
 }
 
@@ -144,18 +144,18 @@ outw(CRT_IC,0x8529);
 outw(SEQ_I,0x4806);
 /* let's unlock sync, timing ... */
 outb(GRA_I,0xd);
-b=inb (GRA_D);
+b=port_in (GRA_D);
 b&=30;
 b|=2;
 outb(GRA_I,0xd);
 outb(GRA_D,b);
 outb(GRA_I,0xe);
-b=inb(GRA_D);
+b=port_in(GRA_D);
 b&=251;
 outb(GRA_I,0xe);
 outb(GRA_D,b);
 outb(CRT_IC,0x2a);
-b=inb(CRT_DC);
+b=port_in(CRT_DC);
 b&=248;
 outb(CRT_IC,0x2a);
 outb(CRT_DC,b);
@@ -278,11 +278,11 @@ static int paradise_test(void)
  printf("paradise_test\n");
 #endif
     txt[5]=0;
-    old_value=inb(CRT_IC);
+    old_value=port_in(CRT_IC);
     for (a=0;a<5;a++)
      {
      outb(CRT_IC,0x31+a);
-     txt[a]=inb(CRT_DC);
+     txt[a]=port_in(CRT_DC);
      }
     if (strcmp(txt,"WD90C"))
     {
@@ -291,11 +291,11 @@ static int paradise_test(void)
     }
     /* check version */
     outb(CRT_IC, 0x36);
-    switch (inb(CRT_DC))
+    switch (port_in(CRT_DC))
     {
     case 0x32:  /* WD90C2x */
     outb(CRT_IC, 0x37);
-    switch (inb(CRT_DC))
+    switch (port_in(CRT_DC))
      {
      case 0x34:  /* WD90C24 */
      case 0x36:  /* WD90C26 */
@@ -307,7 +307,7 @@ static int paradise_test(void)
     case 0x33:  /* WD90C3x */
     outb(CRT_IC, 0x37);
     outb(CRT_IC, 0x37);
-    switch (inb(CRT_DC))
+    switch (port_in(CRT_DC))
      {
      case 0x30:  /* WD90C30 */
      case 0x31:  /* WD90C31 */
@@ -336,9 +336,9 @@ static void paradise_setpage(int page)
 paradise_unlock();
 /* set read-write paging mode */
 outb(SEQ_I,0x11);
-outb(SEQ_D,inb(SEQ_D)|0x80);
+outb(SEQ_D,port_in(SEQ_D)|0x80);
 outb(GRA_I,0x0b);
-outb(GRA_D,inb(GRA_D)|0x80);
+outb(GRA_D,port_in(GRA_D)|0x80);
 /* set bank number */
 outb(GRA_I,0x09);
 outb(GRA_D,page<<4);
@@ -357,7 +357,7 @@ static void paradise_setdisplaystart(int address)
 paradise_unlock();
 outb(CRT_IC,0x2f);
 bits=address>>13;
-orig=inb(CRT_DC);
+orig=port_in(CRT_DC);
 orig^=bits;
 orig&=0xe7;
 orig^=bits;
@@ -420,7 +420,7 @@ static int paradise_init(int force, int par1, int par2)
 	paradise_memory = par1;
     } else {
 	outb(GRA_I,0x0b);
-	paradise_memory=(inb(GRA_D)>>6);
+	paradise_memory=(port_in(GRA_D)>>6);
 	switch (paradise_memory)
 	 {
 	 case 0:
