@@ -1,5 +1,5 @@
 /*
-Rage 128 chipset driver 
+Rage 128 chipset driver
 */
 
 #include <stdlib.h>
@@ -16,10 +16,17 @@ Rage 128 chipset driver
 #include "interface.h"
 #include "accel.h"
 #include "vgapci.h"
+#include "endianess.h"
 #include "r128_reg.h"
 
-typedef unsigned int CARD32;
-typedef unsigned short CARD16;
+#include "r128io.c"
+
+#ifdef __PPC
+#define NO_BIOS
+#else
+#undef NO_BIOS
+#endif
+
 typedef int Bool;
 
 static int r128_ramtype;
@@ -138,15 +145,15 @@ static R128RAMRec ram[] = {        /* Memory Specifications
     { 4, 4, 3, 3, 2, 3, 1, 16, 12, "64-bit DDR SGRAM" },
 };
 
-unsigned R128INPLL(int addr)
+static unsigned R128INPLL(int addr)
 {
     OUTREG8(R128_CLOCK_CNTL_INDEX, addr & 0x1f);
     return INREG(R128_CLOCK_CNTL_DATA);
 }
 
-void R128WaitForVerticalSync(void)
+static void R128WaitForVerticalSync(void)
 {
-    int           i;
+    volatile int i;
 
     OUTREG(R128_GEN_INT_STATUS, R128_VSYNC_INT_AK);
     for (i = 0; i < R128_TIMEOUT; i++) {
@@ -208,6 +215,7 @@ static void R128RestoreCrtcRegisters(R128SavePtr restore)
 }
 
 /* Write flat panel registers */
+#if 0
 static void R128RestoreFPRegisters(R128SavePtr restore)
 {
     CARD32        tmp;
@@ -239,7 +247,7 @@ static void R128RestoreFPRegisters(R128SavePtr restore)
 	}
     }
 }
-
+#endif
 static void R128PLLWaitForReadUpdateComplete(void)
 {
     while (INPLL(R128_PPLL_REF_DIV) & R128_PPLL_ATOMIC_UPDATE_R);
@@ -313,7 +321,9 @@ static void R128RestoreMode(R128SavePtr restore)
     OUTREG(R128_AMCGPIO_MASK,     restore->amcgpio_mask);
     OUTREG(R128_AMCGPIO_EN_REG,   restore->amcgpio_en_reg);
     OUTREG(R128_CLOCK_CNTL_INDEX, restore->clock_cntl_index);
+#if 0 /* works without, and it causes problems with it */
     OUTREG(R128_GEN_RESET_CNTL,   restore->gen_reset_cntl);
+#endif
     OUTREG(R128_DP_DATATYPE,      restore->dp_datatype);
 
     R128RestoreCommonRegisters( restore);
@@ -321,7 +331,7 @@ static void R128RestoreMode(R128SavePtr restore)
 //    if (info->HasPanelRegs)
 //	R128RestoreFPRegisters(restore);
 //    if (!info->HasPanelRegs || info->CRTOnly)
-	R128RestorePLLRegisters(restore);
+    R128RestorePLLRegisters(restore);
     R128RestoreDDARegisters(restore);
     R128RestorePalette(restore);
 }
@@ -362,6 +372,7 @@ static void R128SaveCrtcRegisters(R128SavePtr save)
     save->crtc_pitch           = INREG(R128_CRTC_PITCH);
 }
 
+#if 0
 /* Read flat panel registers */
 static void R128SaveFPRegisters(R128SavePtr save)
 {
@@ -377,6 +388,7 @@ static void R128SaveFPRegisters(R128SavePtr save)
     save->lvds_gen_cntl        = INREG(R128_LVDS_GEN_CNTL);
     save->tmds_crc             = INREG(R128_TMDS_CRC);
 }
+#endif
 
 /* Read PLL registers. */
 static void R128SavePLLRegisters(R128SavePtr save)
@@ -442,7 +454,12 @@ static void R128InitCommonRegisters(R128SavePtr save)
     save->cap1_trig_cntl     = 0;
     save->mem_vga_wp_sel     = 0;
     save->mem_vga_rp_sel     = 0;
+    save->config_cntl        = INREG(R128_CONFIG_CNTL);
     save->bus_cntl           = BusCntl;
+
+    save->amcgpio_en_reg   = INREG(R128_AMCGPIO_EN_REG);
+    save->amcgpio_mask     = INREG(R128_AMCGPIO_MASK);
+
     /*
      * If bursts are enabled, turn on discards and aborts
      */
@@ -545,7 +562,7 @@ static Bool R128InitCrtcRegisters(R128SavePtr save,
 
     save->config_cntl |= R128_CFG_VGA_RAM_EN;
 
-#if 0
+#ifdef __PPC
     /* Change the endianness of the aperture */
     switch (info->bitsPerPixel) {
     case 15:
@@ -558,6 +575,7 @@ static Bool R128InitCrtcRegisters(R128SavePtr save,
     return 1;
 }
 
+#if 0
 /* Define CRTC registers for requested video mode. */
 static void R128InitFPRegisters(R128SavePtr orig, R128SavePtr save,
  				ModeTiming *mode, ModeInfo *info)
@@ -652,6 +670,7 @@ static void R128InitFPRegisters(R128SavePtr orig, R128SavePtr save,
     save->fp_v_sync_strt_wid   = save->crtc_v_sync_strt_wid;
 #endif
 }
+#endif
 
 /* Define PLL registers for requested video mode. */
 static void R128InitPLLRegisters(R128SavePtr save, R128PLLPtr pll,
@@ -815,6 +834,7 @@ static void r128_getmodeinfo(int mode, vga_modeinfo *modeinfo)
     modeinfo->startaddressrange = r128_memory * 1024 - 1;
     modeinfo->haveblit = 0;
     modeinfo->flags &= ~HAVE_RWPAGE;
+    modeinfo->flags |= HAVE_EXT_SET;
 
     if (modeinfo->bytesperpixel >= 1) {
 	if(r128_linear_base)modeinfo->flags |= CAPABLE_LINEAR;
@@ -827,8 +847,6 @@ static void r128_getmodeinfo(int mode, vga_modeinfo *modeinfo)
 
 static int r128_saveregs(unsigned char regs[])
 { 
-  int i;
-
     r128_unlock();		
     R128SaveMode((R128SavePtr)(regs+VGA_TOTAL_REGS));
     return R128_TOTAL_REGS - VGA_TOTAL_REGS;
@@ -930,23 +948,11 @@ static int r128_setmode(int mode, int prv_mode)
 
 static void r128_unlock(void)
 {
-    int vgaIOBase, temp;
-
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
-    outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
-    outb(vgaIOBase + 5, temp & 0x7F);
-    
+    __svgalib_outcrtc(0x11, __svgalib_incrtc(0x11) & 0x7f);    
 }
 
 static void r128_lock(void)
 {
-    int vgaIOBase, temp;
-
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
-    outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
-    outb(vgaIOBase + 5, temp & 0x7F);   
 }
 
 
@@ -1019,6 +1025,98 @@ static int r128_map_horizontal_crtc(int bpp, int pixelclock, int htiming)
 return htiming;
 }
 
+static unsigned int cur_colors[16*2];
+
+static int r128_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
+    int i, j;
+    unsigned long *b3;
+    unsigned long l1, l2;
+    
+    switch(cmd){
+        case CURSOR_INIT:
+            return 1;
+        case CURSOR_HIDE:
+            OUTREG(R128_CRTC_GEN_CNTL,INREG(R128_CRTC_GEN_CNTL) & ~R128_CRTC_CUR_EN );
+            break;
+        case CURSOR_SHOW:
+            OUTREG(R128_CRTC_GEN_CNTL,INREG(R128_CRTC_GEN_CNTL) | R128_CRTC_CUR_EN );
+            break;
+        case CURSOR_POSITION:
+            OUTREG(R128_CUR_HORZ_VERT_OFF, R128_CUR_LOCK | 0);
+            OUTREG(R128_CUR_HORZ_VERT_POSN, p2|(p1<<16));
+            break;
+        case CURSOR_SELECT:
+            i=r128_memory*1024-(p1+1)*4096;
+            OUTREG(R128_CUR_OFFSET,i);
+            OUTREG(R128_CUR_CLR0,cur_colors[p1*2]);
+            OUTREG(R128_CUR_CLR1,cur_colors[p1*2+1]);
+            break;
+        case CURSOR_IMAGE:
+            i=r128_memory*1024-(p1+1)*4096;
+            b3=(unsigned long *)p5;
+            switch(p2) {
+                case 0:
+                    cur_colors[p1*2]=p3;
+                    cur_colors[p1*2+1]=p4;
+
+                    for(j=0;j<32;j++) {
+                        l1=*(b3+j); /* source */
+                        l2=*(b3+32+j); /* mask */
+                        l1=BE32(l1);
+                        l2=BE32(l2);
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j)=~l2;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+4)=0xffffffff;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+8)=l1&l2;
+                        *(unsigned long *)(LINEAR_POINTER+i+16*j+12)=0;
+                        *(unsigned long *)(LINEAR_POINTER+i+512+16*j)=0xffffffff;
+                        *(unsigned long *)(LINEAR_POINTER+i+512+16*j+4)=0xffffffff;
+                        *(unsigned long *)(LINEAR_POINTER+i+512+16*j+8)=0;
+                        *(unsigned long *)(LINEAR_POINTER+i+512+16*j+12)=0;
+                    }
+                break;
+            }
+            break;
+    }
+    return 0;
+}       
+
+static Emulation r128_emul = 
+{
+	0,
+	0,
+	0,
+	0,
+	0,
+        0,
+	0,				/* screenoff */
+	0,				/* screenon */
+	R128WaitForVerticalSync,	/* waitretrace */
+};
+
+static int r128_ext_set( unsigned what, va_list params )
+{
+    int param2;
+    param2 = va_arg(params, int);
+
+    switch (what) {
+        case VGA_EXT_AVAILABLE:
+	    switch (param2) {
+                case VGA_AVAIL_SET:
+	            return VGA_EXT_AVAILABLE | VGA_EXT_SET | VGA_EXT_CLEAR | VGA_EXT_RESET;
+	        case VGA_AVAIL_ACCEL:
+	            return 0;
+	        case VGA_AVAIL_FLAGS:
+	            return VGA_CLUT8;
+	    }
+	    break;
+        case VGA_EXT_SET:
+	    if (param2 & VGA_CLUT8) OUTREGP( R128_DAC_CNTL, R128_DAC_8BIT_EN, R128_DAC_MASK_ALL );
+            return 1;
+            break;
+    }
+    return 0;
+}
+
 /* Function table (exported) */
 
 DriverSpecs __svgalib_r128_driverspecs =
@@ -1042,11 +1140,12 @@ DriverSpecs __svgalib_r128_driverspecs =
     0,
     0,
     0,
-    0,				/* ext_set */
+    r128_ext_set,		/* ext_set */
     0,				/* accel */
     r128_linear,
     0,				/* accelspecs, filled in during init. */
-    NULL,                       /* Emulation */
+    &r128_emul,			/* Emulation */
+    r128_cursor,
 };
 
 /* Initialize chipset (called after detection) */
@@ -1055,9 +1154,10 @@ static int r128_init(int force, int par1, int par2)
 {
     unsigned long buf[64];
     int found=0;
+#ifndef NO_BIOS
     unsigned char *BIOS_POINTER;
+#endif
 
-    r128_unlock();
     if (force) {
 	r128_memory = par1;
         r128_chiptype = par2;
@@ -1076,20 +1176,33 @@ static int r128_init(int force, int par1, int par2)
 
     MMIO_POINTER = mmap(0, 16*1024, PROT_READ | PROT_WRITE, MAP_SHARED, __svgalib_mem_fd,
        			r128_mmio_base);
+#ifndef NO_BIOS
     BIOS_POINTER = mmap(0, 64*1024, PROT_READ | PROT_WRITE, MAP_SHARED, __svgalib_mem_fd,
        			0xc0000);
+#endif
 
     r128_memory      = INREG(R128_CONFIG_MEMSIZE) / 1024;
     BusCntl            = INREG(R128_BUS_CNTL);
     HasPanelRegs	= 0;
     CRTOnly		= 1;
     r128_ramtype	= 1;
-#if 0
-    pll.reference_freq = 2700;
+#ifdef NO_BIOS
+    {
+    int x_mpll_ref_fb_div, xclk_cntl, Nx, M;
+    int PostDivSet[] = {0, 1, 2, 4, 8, 3, 6, 12};
+    
+    pll.reference_freq = 2950;
     pll.min_pll_freq   = 12500;
-    pll.max_pll_freq   = 40000;
+    pll.max_pll_freq   = 25000;
     pll.reference_div =	INPLL(R128_PPLL_REF_DIV) & R128_PPLL_REF_DIV_MASK;
-    pll.xclk           = 14000;
+
+    x_mpll_ref_fb_div = INPLL(R128_X_MPLL_REF_FB_DIV);
+    xclk_cntl = INPLL(R128_XCLK_CNTL) & 0x7;
+    Nx = (x_mpll_ref_fb_div & 0x00FF00) >> 8;
+    M =  (x_mpll_ref_fb_div & 0x0000FF);
+    pll.xclk =  R128Div((2 * Nx * pll.reference_freq),
+                          (M * PostDivSet[xclk_cntl]));
+    }
 #else
 #define R128_BIOS16(x) (*(unsigned short *)(BIOS_POINTER + x))
 #define R128_BIOS32(x) (*(unsigned int *)(BIOS_POINTER + x))
@@ -1101,15 +1214,19 @@ static int r128_init(int force, int par1, int par2)
     pll.min_pll_freq   = R128_BIOS32(pll_info_block + 0x12);
     pll.max_pll_freq   = R128_BIOS32(pll_info_block + 0x16);
     pll.xclk           = R128_BIOS16(pll_info_block + 0x08);
+    }
+#endif
 #if 0
 printf("pll: %i %i %i %i %i\n",pll.reference_freq,pll.reference_div,
     pll.min_pll_freq,    pll.max_pll_freq, pll.xclk);
 #endif
-    }
-#endif
 
     munmap(MMIO_POINTER, 16*1024);
+#ifndef NO_BIOS
     munmap(BIOS_POINTER, 64*1024);
+#endif
+
+    r128_mapio();
     
     if (__svgalib_driver_report) {
 	printf("Using Rage 128 driver, %iKB.\n",r128_memory);
@@ -1118,12 +1235,12 @@ printf("pll: %i %i %i %i %i\n",pll.reference_freq,pll.reference_div,
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = r128_memory;
     cardspecs->maxPixelClock4bpp = 75000;	
-    cardspecs->maxPixelClock8bpp = 160000;	
-    cardspecs->maxPixelClock16bpp = 160000;	
-    cardspecs->maxPixelClock24bpp = 160000;
-    cardspecs->maxPixelClock32bpp = 160000;
+    cardspecs->maxPixelClock8bpp = 250000;	
+    cardspecs->maxPixelClock16bpp = 250000;	
+    cardspecs->maxPixelClock24bpp = 250000;
+    cardspecs->maxPixelClock32bpp = 250000;
     cardspecs->flags = INTERLACE_DIVIDE_VERT | CLOCK_PROGRAMMABLE;
-    cardspecs->maxHorizontalCrtc = 2040;
+    cardspecs->maxHorizontalCrtc = 4080;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->mapClock = r128_map_clock;
