@@ -39,11 +39,10 @@ Copyright 1994 through 1998 by Marc Aurele La France (TSI @ UQV), tsi@ualberta.c
 #include "timing.h"
 #include "vgaregs.h"
 #include "interface.h"
-#include "accel.h"
 #include "vgapci.h"
 #include "rage.h"
 
-int           ATIIOPortCRTC_H_TOTAL_DISP, ATIIOPortCRTC_H_SYNC_STRT_WID,
+static int    ATIIOPortCRTC_H_TOTAL_DISP, ATIIOPortCRTC_H_SYNC_STRT_WID,
               ATIIOPortCRTC_V_TOTAL_DISP, ATIIOPortCRTC_V_SYNC_STRT_WID,
               ATIIOPortCRTC_OFF_PITCH,ATIIOPortCRTC_INT_CNTL,
               ATIIOPortCRTC_GEN_CNTL, ATIIOPortDSP_CONFIG, ATIIOPortDSP_ON_OFF,
@@ -58,9 +57,11 @@ int           ATIIOPortCRTC_H_TOTAL_DISP, ATIIOPortCRTC_H_SYNC_STRT_WID,
               ATIIOPortCUR_HORZ_VERT_POSN, ATIIOPortCUR_CLR0,
               ATIIOPortCUR_CLR1, ATIIOPortCUR_OFFSET;
 
-void ATIAccessMach64PLLReg(const int Index, const int Write)
+#include "rageio.c"
+
+static void ATIAccessMach64PLLReg(const int Index, const int Write)
 {
-    int clock_cntl1 = inb(ATIIOPortCLOCK_CNTL + 1) &
+    int clock_cntl1 = port_in(ATIIOPortCLOCK_CNTL + 1) &
         ~GetByte(PLL_WR_EN | PLL_ADDR, 1);
 
     /* Set PLL register to be read or written */
@@ -90,6 +91,7 @@ typedef struct {
         unsigned char DAC[DAC_SIZE];
         unsigned char extdac[16];
         int ics2595;
+        unsigned char atib[16]; 
        } ATIHWRec, *ATIHWPtr;
 
 #define RAGE_TOTAL_REGS (VGA_TOTAL_REGS + sizeof(ATIHWRec))
@@ -98,20 +100,17 @@ extern unsigned char __svgalib_ragedoubleclock;
 static int rage_init(int, int, int);
 static void rage_unlock(void);
 static void rage_lock(void);
-
-void __svgalib_rageaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
-
 static int rage_memory,rage_chiptyperev;
 static int rage_is_linear=0 , rage_linear_base;
 static int ATIIODecoding;
 static int ATIIOBase;
 static int postdiv[8]={1,2,4,8,3,0,6,12};
 static int rage_bpp;
-static int M, minN, maxN, Nadj;
+static int maxM, minM, Madj, minN, maxN, Nadj;
 static double fref;
 static int ATIClockToProgram;
 static int ATIChip, ATIMemoryType;
-static int rage_dac=0, rage_clock=0;
+static int rage_dac, rage_clock;
 
 static CardSpecs *cardspecs;
 
@@ -383,7 +382,7 @@ static void rage_getmodeinfo(int mode, vga_modeinfo *modeinfo)
 
 /* Read and save chipset-specific registers */
 
-int rage_saveregs(unsigned char regs[])
+static int rage_saveregs(unsigned char regs[])
 { 
   ATIHWPtr save;
   int i;
@@ -419,19 +418,6 @@ int rage_saveregs(unsigned char regs[])
   save->crtc_gen_cntl=inl(ATIIOPortCRTC_GEN_CNTL);
   save->gen_test_cntl=inl(ATIIOPortGEN_TEST_CNTL);  
 
-  save->cur_clr0=inl(ATIIOPortCUR_CLR0);
-  save->cur_clr1=inl(ATIIOPortCUR_CLR1);
-  save->cur_offset=inl(ATIIOPortCUR_OFFSET);
-  save->cur_horz_vert_posn=inl(ATIIOPortCUR_HORZ_VERT_POSN);
-  save->cur_horz_vert_off=inl(ATIIOPortCUR_HORZ_VERT_OFF);
-
-  switch(rage_clock){
-     case 0: /* internal clock */
-        for(i=6;i<12;i++)
-           save->PLL[i]=ATIGetMach64PLLReg(i); /* internal Clock */
-        break;
-  };
-
   if((ATIChip>=ATI_CHIP_264VTB)&&(ATIIODecoding==BLOCK_IO)) {
     save->dsp_on_off=inl(ATIIOPortDSP_ON_OFF);
     save->dsp_config=inl(ATIIOPortDSP_CONFIG);
@@ -441,27 +427,68 @@ int rage_saveregs(unsigned char regs[])
   ATIIOPortDAC_MASK = ATIIOPortDAC_REGS + 2;
   ATIIOPortDAC_READ = ATIIOPortDAC_REGS + 3;
   ATIIOPortDAC_WRITE = ATIIOPortDAC_REGS + 0;
-  save->dac_read = inb(ATIIOPortDAC_READ);
-  save->dac_write = inb(ATIIOPortDAC_WRITE);
-  save->dac_mask = inb(ATIIOPortDAC_MASK);
+  save->dac_read = port_in(ATIIOPortDAC_READ);
+  save->dac_write = port_in(ATIIOPortDAC_WRITE);
+  save->dac_mask = port_in(ATIIOPortDAC_MASK);
+
+  outl(ATIIOPortCRTC_GEN_CNTL, save->crtc_gen_cntl | 0x1000000);
+  
+  switch(rage_clock){
+     case -1: /* internal clock */
+        for(i=6;i<12;i++)
+           save->PLL[i]=ATIGetMach64PLLReg(i); /* internal Clock */
+        break;
+     case 4: /* CH8398 */        
+        i=inl(ATIIOPortDAC_CNTL);
+        outl(ATIIOPortDAC_CNTL,(i&~3)|3);
+        outb(ATIIOPortDAC_READ, ATIClockToProgram);
+        save->PLL[0]=port_in(ATIIOPortDAC_DATA);
+        save->PLL[1]=port_in(ATIIOPortDAC_DATA);
+        outl(ATIIOPortDAC_CNTL,i);        
+        break;
+  };
 
   outb(ATIIOPortDAC_MASK, 0xFFU);
   outb(ATIIOPortDAC_READ, 0x00U);
+
   for (i = 0;  i<DAC_SIZE;  i++)
-      save->DAC[i] = inb(ATIIOPortDAC_DATA);
+      save->DAC[i] = port_in(ATIIOPortDAC_DATA);
 
   switch(rage_dac){
+      case -1:
+          save->cur_clr0=inl(ATIIOPortCUR_CLR0);
+          save->cur_clr1=inl(ATIIOPortCUR_CLR1);
+          save->cur_offset=inl(ATIIOPortCUR_OFFSET);
+          save->cur_horz_vert_posn=inl(ATIIOPortCUR_HORZ_VERT_POSN);
+          save->cur_horz_vert_off=inl(ATIIOPortCUR_HORZ_VERT_OFF);
+          break;
+      case 4: /* CH8398 */
+          i=inl(ATIIOPortDAC_CNTL);
+          outl(ATIIOPortDAC_CNTL,(i&~3)|3);
+          save->extdac[4]=port_in(ATIIOPortDAC_WRITE);
+          save->extdac[5]=port_in(ATIIOPortDAC_DATA);
+          save->extdac[7]=port_in(ATIIOPortDAC_READ);
+          save->extdac[6]=port_in(ATIIOPortDAC_MASK); 
+          outl(ATIIOPortDAC_CNTL,i);          
+          break;
        case 5:
           i=inl(ATIIOPortDAC_CNTL);
-          outl(ATIIOPortDAC_CNTL,(i&0xfffffffc)|2);
-          save->extdac[8]=inb(ATIIOPortDAC_WRITE);
-          save->extdac[10]=inb(ATIIOPortDAC_MASK);
-          save->extdac[11]=inb(ATIIOPortDAC_READ);
-          outl(ATIIOPortDAC_CNTL,(i&0xfffffffc)|3);
-          save->extdac[12]=inb(ATIIOPortDAC_WRITE);
+          outl(ATIIOPortDAC_CNTL,(i&~3)|2);
+          save->extdac[8]=port_in(ATIIOPortDAC_WRITE);
+          save->extdac[10]=port_in(ATIIOPortDAC_MASK);
+          save->extdac[11]=port_in(ATIIOPortDAC_READ);
+          outl(ATIIOPortDAC_CNTL,(i&~3)|3);
+          save->extdac[12]=port_in(ATIIOPortDAC_WRITE);
           outl(ATIIOPortDAC_CNTL,i);
        break;
   };
+  
+  for(i=6;i<7;i++) {
+     outb(0x1ce, 0xb0+i);
+     save->atib[i]=inb(0x1cf);
+  }
+
+  outl(ATIIOPortCRTC_GEN_CNTL, save->crtc_gen_cntl);
 
   return RAGE_TOTAL_REGS - VGA_TOTAL_REGS;
 }
@@ -513,14 +540,10 @@ static void rage_setregs(const unsigned char regs[], int mode)
     outl(ATIIOPortCRTC_GEN_CNTL,restore->crtc_gen_cntl);
     outl(ATIIOPortGEN_TEST_CNTL,restore->gen_test_cntl);
 
-    outl(ATIIOPortCUR_CLR0, restore->cur_clr0);
-    outl(ATIIOPortCUR_CLR1, restore->cur_clr1);
-    outl(ATIIOPortCUR_OFFSET, restore->cur_offset);
-    outl(ATIIOPortCUR_HORZ_VERT_POSN, restore->cur_horz_vert_posn);
-    outl(ATIIOPortCUR_HORZ_VERT_OFF, restore->cur_horz_vert_off);
+    outl(ATIIOPortCRTC_GEN_CNTL, restore->crtc_gen_cntl | 0x1000000);
 
     switch(rage_clock){
-       case 0: /* internal clock */
+       case -1: /* internal clock */
           i=ATIGetMach64PLLReg(PLL_VCLK_CNTL) | PLL_VCLK_RESET;
           ATIPutMach64PLLReg(PLL_VCLK_CNTL,i);
     
@@ -530,7 +553,15 @@ static void rage_setregs(const unsigned char regs[], int mode)
     
           i&= ~PLL_VCLK_RESET;
           ATIPutMach64PLLReg(PLL_VCLK_CNTL,i);
-       break;
+          break;
+       case 4: /* CH8398 */
+          i=inl(ATIIOPortDAC_CNTL);
+          outl(ATIIOPortDAC_CNTL,(i&~3)|3);
+          outb(ATIIOPortDAC_WRITE, ATIClockToProgram);
+          outb(ATIIOPortDAC_DATA, restore->PLL[0]);
+          outb(ATIIOPortDAC_DATA, restore->PLL[1]);
+          outl(ATIIOPortDAC_CNTL,i);          
+          break;
     };
 
     /* make sure the dac is in 8 bit or 6 bit mode, as needed */
@@ -542,6 +573,19 @@ static void rage_setregs(const unsigned char regs[], int mode)
       outb(ATIIOPortDAC_DATA, restore->DAC[Index]);
 
     switch(rage_dac){
+      case -1:
+          outl(ATIIOPortCUR_CLR0, restore->cur_clr0);
+          outl(ATIIOPortCUR_CLR1, restore->cur_clr1);
+          outl(ATIIOPortCUR_OFFSET, restore->cur_offset);
+          outl(ATIIOPortCUR_HORZ_VERT_POSN, restore->cur_horz_vert_posn);
+          outl(ATIIOPortCUR_HORZ_VERT_OFF, restore->cur_horz_vert_off);
+          break;
+      case 4: /* CH8398 */
+          i=inl(ATIIOPortDAC_CNTL);
+          outl(ATIIOPortDAC_CNTL,(i&~3)|3);
+          outb(ATIIOPortDAC_MASK, restore->extdac[6]);
+          outl(ATIIOPortDAC_CNTL,i);          
+          break;
        case 5:
           i=inl(ATIIOPortDAC_CNTL);
           outl(ATIIOPortDAC_CNTL,(i&0xfffffffc)|2);
@@ -549,13 +593,19 @@ static void rage_setregs(const unsigned char regs[], int mode)
           outb(ATIIOPortDAC_MASK,restore->extdac[10]);
           outb(ATIIOPortDAC_READ,restore->extdac[11]);
           outl(ATIIOPortDAC_CNTL,(i&0xfffffffc)|3);
-          outb(ATIIOPortDAC_WRITE,(inb(ATIIOPortDAC_WRITE)&0x80)|restore->extdac[12]);
+          outb(ATIIOPortDAC_WRITE,(port_in(ATIIOPortDAC_WRITE)&0x80)|restore->extdac[12]);
        break;
     };
 
     outb(ATIIOPortDAC_READ, restore->dac_read);
     outb(ATIIOPortDAC_WRITE, restore->dac_write);
     outl(ATIIOPortDAC_CNTL, restore->dac_cntl);
+    
+    for(i=6;i<7;i++) {
+       outb(0x1ce, 0xb0 + i);
+       outb(0x1cf, restore->atib[i]);
+    }
+    outl(ATIIOPortCRTC_GEN_CNTL, restore->crtc_gen_cntl);
 
 };
 /* Return nonzero if mode is available */
@@ -566,10 +616,7 @@ static int rage_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
@@ -590,6 +637,7 @@ static int rage_modeavailable(int mode)
 
 static unsigned
 comp_lmn(unsigned clock, int *n, int *mp, int *lp);
+static int rage_map_clock(int bpp, int clock);
 
 /* Local, called by rage_setmode(). */
 static void rage_initializemode(unsigned char *moderegs,
@@ -598,7 +646,7 @@ static void rage_initializemode(unsigned char *moderegs,
     int m,n,l,i;
     ATIHWPtr ATINewHWPtr;
     int ATIDisplayFIFODepth = 32 ;
-    int ReferenceDivider = M ;
+    int ReferenceDivider = maxM;
     
     ATINewHWPtr=(ATIHWPtr)(moderegs+VGA_TOTAL_REGS); 
 
@@ -660,7 +708,7 @@ static void rage_initializemode(unsigned char *moderegs,
       else
             ATINewHWPtr->dac_cntl &= ~DAC_8BIT_EN;
 
-    ATINewHWPtr->config_cntl= inl(ATIIOPortCONFIG_CNTL) | CFG_MEM_VGA_AP_EN;
+    ATINewHWPtr->config_cntl= inl(ATIIOPortCONFIG_CNTL) | CFG_MEM_VGA_AP_EN | 2;
 
     ATINewHWPtr->ovr_clr=0;
 
@@ -674,13 +722,20 @@ static void rage_initializemode(unsigned char *moderegs,
     modetiming->CrtcHSyncStart  = (modetiming->CrtcHSyncStart >> 3) -1;
     modetiming->CrtcHSyncEnd = (modetiming->CrtcHSyncEnd >> 3) -1;
 
-    comp_lmn(modetiming->pixelClock,&n,&m,&l);
+    comp_lmn(rage_map_clock(modeinfo->bitsPerPixel, modetiming->pixelClock),&n,&m,&l);
     
     switch(rage_clock) {
-       case 1:
+       case 5:
              ATINewHWPtr->ics2595=n|(l<<9);
          break;
-       default:
+       case 4: /* CH8398 */
+           for(i=0;i<16;i++)ATINewHWPtr->atib[i]=0;
+           ATINewHWPtr->atib[6]=0x04;           
+           ATINewHWPtr->PLL[0]=n;
+           ATINewHWPtr->PLL[1]=m | (l<<6);
+           ATINewHWPtr->clock_cntl = 0x40 + ATIClockToProgram;
+           break;
+       case -1:
             ATINewHWPtr->PLL[PLL_VCLK0_FB_DIV+ATIClockToProgram]=n;
             ATINewHWPtr->PLL[PLL_VCLK_POST_DIV]&=0xfc<<(ATIClockToProgram<<1);
             ATINewHWPtr->PLL[PLL_VCLK_POST_DIV]|=(l&3)<<(ATIClockToProgram<<1);
@@ -757,6 +812,17 @@ static void rage_initializemode(unsigned char *moderegs,
             ATINewHWPtr->crtc_gen_cntl |= CRTC_INTERLACE_EN;
 
     switch(rage_dac){
+       case 4: /* CH8398 */
+          moderegs[VGA_MISCOUTPUT]=0x6f;
+          switch(modeinfo->bitsPerPixel) {
+             case 8: ATINewHWPtr->extdac[6]=0x04; break;
+             case 16: 
+                if(modeinfo->colorBits==15) ATINewHWPtr->extdac[6]=0x14; 
+                   else ATINewHWPtr->extdac[6]=0x34; 
+                break;
+             case 24: ATINewHWPtr->extdac[6]=0xb4; break;
+          };
+          break;
        case 5: /* 68860/68880 */
           ATINewHWPtr->extdac[8]=2;
           ATINewHWPtr->extdac[10]=0x1d;
@@ -775,8 +841,6 @@ static void rage_initializemode(unsigned char *moderegs,
        break;       
     };
 
-
-
     rage_bpp = modeinfo->bytesPerPixel; 
 
 if((ATIChip>=ATI_CHIP_264VTB)&&(ATIIODecoding==BLOCK_IO))
@@ -796,7 +860,7 @@ if((ATIChip>=ATI_CHIP_264VTB)&&(ATIIODecoding==BLOCK_IO))
 
     IO_Value = ATIGetMach64PLLReg(PLL_XCLK_CNTL);
     ATIXCLKPostDivider = GetBits(IO_Value, PLL_XCLK_SRC_SEL);
-    ATIXCLKReferenceDivider = M ;
+    ATIXCLKReferenceDivider = maxM ;
     switch (ATIXCLKPostDivider)
     {
         case 0: case 1: case 2: case 3:
@@ -933,8 +997,7 @@ static int rage_setmode(int mode, int prv_mode)
     ModeInfo *modeinfo;
     ATIHWPtr ATINewHWPtr;
 
-    if ((mode < G640x480x256 /*&& mode != G320x200x256*/)
-	|| mode == G720x348x2) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
 
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
@@ -961,8 +1024,6 @@ static int rage_setmode(int mode, int prv_mode)
     rage_setregs(moderegs, mode);	
     free(moderegs);
 
-    __svgalib_InitializeAcceleratorInterface(modeinfo);
-
     free(modeinfo);
 
     return 0;
@@ -1017,7 +1078,6 @@ static int rage_test(void)
 }
 
 /* Set display start address (not for 16 color modes) */
-/* Cirrus supports any address in video memory (up to 2Mb) */
 
 static void rage_setdisplaystart(int address)
 { 
@@ -1054,7 +1114,8 @@ return clock ;
 
 static int rage_map_clock(int bpp, int clock)
 {
-return clock ;
+   if((bpp==24)&&(rage_clock==4))clock += (clock>>1);
+   return clock ;
 }
 
 static int rage_map_horizontal_crtc(int bpp, int pixelclock, int htiming)
@@ -1062,6 +1123,29 @@ static int rage_map_horizontal_crtc(int bpp, int pixelclock, int htiming)
 return htiming;
 }
 
+static int pal=1, palette[768];
+
+static int findcolor(int rgb) {
+   int i,j,k,l=0;
+
+   if(pal)vga_getpalvec(0,256,palette);
+   pal=0;
+   k=0xffffff;
+   for(i=0;i<256;i++) {
+      j=((rgb&0xff)-(palette[i*3+2]<<2))*((rgb&0xff)-(palette[i*3+2]<<2))+
+        (((rgb>>8)&0xff)-(palette[i*3+1]<<2))*(((rgb>>8)&0xff)-(palette[i*3+1]<<2))+
+        (((rgb>>16)&0xff)-(palette[i*3]<<2))*(((rgb>>16)&0xff)-(palette[i*3]<<2));
+      if(j==0) {
+         return i;
+      }
+      if(j<k) {
+         k=j;
+         l=i;
+      }
+   }
+   return l;
+}
+
 static unsigned int cur_colors[16*2];
 
 static int rage_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
@@ -1095,8 +1179,8 @@ static int rage_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
             b3=(unsigned long *)p5;
             switch(p2) {
                 case 0:
-                    cur_colors[p1*2]=p3<<8;
-                    cur_colors[p1*2+1]=p4<<8;
+                    cur_colors[p1*2]=p3<<8 | findcolor(p3);
+                    cur_colors[p1*2+1]=p4<<8 | findcolor(p4);
                     for(j=0;j<32;j++) {
                         l1=*(b3+j);
                         l2=~(*(b3+32+j));
@@ -1111,11 +1195,11 @@ static int rage_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
                                 l1<<=1;
                                 l2<<=1;
                             }
-                            *(LINEAR_MEM_POINTER+i+16*j+k)=c;
-                            *(LINEAR_MEM_POINTER+i+16*j+k+8)=0xaa;
+                            *(LINEAR_POINTER+i+16*j+k)=c;
+                            *(LINEAR_POINTER+i+16*j+k+8)=0xaa;
                         }
                     }
-                    memset(LINEAR_MEM_POINTER+i+512,0xaa,512);
+                    memset(LINEAR_POINTER+i+512,0xaa,512);
                 break;
             }
             break;
@@ -1251,26 +1335,7 @@ static int rage_init(int force, int par1, int par2)
      rage_memory = videoRamSizes[i+2];
    };
 
-   if(ATIChip>=ATI_CHIP_264CT){
-      M=ATIGetMach64PLLReg(PLL_REF_DIV);
-      minN=2;
-      maxN=255;
-      fref=28636;
-      Nadj=0;
-      if(__svgalib_ragedoubleclock)fref*=2;
-   } else {
-      rage_dac=(j>>9)&7;
-      if(rage_dac==5) {
-         rage_clock=1;
-         M=46;
-         minN=257;
-         maxN=512;
-         Nadj=257;
-         for(i=0;i<4;i++)postdiv[i]=1<<i;
-         for(i=4;i<8;i++)postdiv[i]=0;
-         fref=14318;
-      };
-   };
+   rage_mapio();
 
    if (__svgalib_driver_report) {
         printf("Using RAGE driver, %iKB.   ChipID:%i MemType:%i\n",rage_memory,ATIChip,ATIMemoryType);
@@ -1306,12 +1371,13 @@ static int rage_init(int force, int par1, int par2)
          break;
       default:
          cardspecs->maxPixelClock8bpp = 135000;	
-         cardspecs->maxPixelClock16bpp = 80000;	
-         cardspecs->maxPixelClock24bpp = 40000;
-         cardspecs->maxPixelClock32bpp = 40000;
+         cardspecs->maxPixelClock16bpp =135000;	
+         cardspecs->maxPixelClock24bpp = 60000;
+         cardspecs->maxPixelClock32bpp = 60000;
    };
    cardspecs->flags = CLOCK_PROGRAMMABLE;
-   cardspecs->maxHorizontalCrtc = 2040;
+   cardspecs->maxHorizontalCrtc = 4088;
+   cardspecs->nClocks = 0;
    cardspecs->maxPixelClock4bpp = 0;
    cardspecs->mapClock = rage_map_clock;
    cardspecs->mapHorizontalCrtc = rage_map_horizontal_crtc;
@@ -1322,13 +1388,72 @@ static int rage_init(int force, int par1, int par2)
    __svgalib_linear_mem_base=rage_linear_base;
    __svgalib_linear_mem_size=rage_memory*0x400;
 
+#define BIOSWord(x) ((int)BIOS[x]+256*(int)BIOS[x+1])
+
    BIOS=mmap(0,32*1024,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,0xc0000);
 
-   i=(int)BIOS[0x48]+256*(int)BIOS[0x49];
-   j=(int)BIOS[i+0x10]+256*(int)BIOS[i+0x11];
+   i=BIOSWord(0x48);
+   j=BIOSWord(i+0x10);
 
    ATIClockToProgram=BIOS[j+6];
 
+   if(ATIChip>=ATI_CHIP_264CT){
+      rage_dac = -1;
+      rage_clock = -1;
+      maxM=minM=ATIGetMach64PLLReg(PLL_REF_DIV);
+      minN=2;
+      maxN=255;
+      switch (BIOSWord(j+0x08)/10) {
+         case 143:
+            fref=14318; 
+            break;
+         case 286:
+            fref=28636;
+            break;
+         default:
+            fref=BIOSWord(j+0x08)*10;
+            break;
+      }
+      Nadj=0;
+      Madj=0;
+      fref*=2; /* X says double for all chips */
+      if(__svgalib_ragedoubleclock)fref/=2; /* just in case */
+      if (__svgalib_driver_report) {
+          printf("Rage: BIOS reports base frequency=%.3fMHz  Denominator=%3i\n",fref/1000,maxM);
+      }
+   } else {
+      rage_dac=(j>>9)&7;
+      switch(rage_dac) {
+          case 4:
+              rage_clock=4;
+              Nadj=8;
+              minN=64;
+              maxN=263;
+              maxM=34;
+              minM=23;
+              Madj=2;
+              for(i=0;i<3;i++)postdiv[i]=1<<i;
+              for(i=3;i<8;i++)postdiv[i]=0;
+              fref=14318;
+              cardspecs->maxPixelClock32bpp = 0;
+              break;
+          case 5:
+              rage_clock=5;
+              maxM=minM=46;
+              Madj=0;
+              minN=257;
+              maxN=512;
+              Nadj=257;
+              for(i=0;i<4;i++)postdiv[i]=1<<i;
+              for(i=4;i<8;i++)postdiv[i]=0;
+              fref=14318;
+              break;
+      };
+      if (__svgalib_driver_report) {
+          printf("Rage: BIOS reports RamDAC=%i\n",rage_dac);
+      }
+   };
+
    munmap(BIOS,32768);
 
    return 0;
@@ -1344,7 +1469,7 @@ comp_lmn(unsigned clock, int *np, int *mp, int *lp)
   double  fout;
   double  fvco_goal;
   
-  for (m = M; m <= M; m++)
+  for (m = minM; m <= maxM; m++)
   {
     for (l = 0;(l < 8); l++) if(postdiv[l])
     {
@@ -1357,13 +1482,13 @@ comp_lmn(unsigned clock, int *np, int *mp, int *lp)
           continue;
         *lp=l;
         *np=n-Nadj;
-        *mp=m;
+        *mp=m-Madj;
         return 1 ;
       }
     }
   }
 printf("Can't do clock=%i\n",clock);
-printf("fref=%f, M=%i, N in %i - %i\n",fref,M,minN,maxN);
+printf("fref=%f, M= in %i - %i, N in %i - %i\n",fref,minM, maxM, minN,maxN);
 {int i; for (i=0;i<8;i++)printf("%i ",postdiv[i]); printf("\n");};
   return 0;
 }
