@@ -76,7 +76,7 @@ static void att20c490_init(void)
 #endif
 }
 
-int __svgalib_att20c490_map_clock(int bpp, int pixelclock)
+static int __svgalib_att20c490_map_clock(int bpp, int pixelclock)
 {
     if (bpp == 16)
 	return pixelclock * 2;
@@ -85,7 +85,7 @@ int __svgalib_att20c490_map_clock(int bpp, int pixelclock)
     return pixelclock;
 }
 
-int __svgalib_att20c490_map_horizontal_crtc(int bpp, int pixelclock, int htiming)
+static int __svgalib_att20c490_map_horizontal_crtc(int bpp, int pixelclock, int htiming)
 {
     if (bpp == 16)
 	return htiming * 2;
