@@ -114,6 +114,9 @@ DacMethods *__svgalib_all_dacs[] =
 #endif
 #ifdef INCLUDE_SC1148X_DAC
     &__svgalib_SC1148X_methods,
+#endif
+#ifdef INCLUDE_ICS_GENDAC_DAC
+    &__svgalib_ICS_GENDAC_methods,
 #endif
     NULL
 };
