@@ -517,7 +517,7 @@ DacMethods __svgalib_S3_GENDAC_methods =
 
 /* Note: s3.c also defines CR67, but doesn't use it for the Trio64. */
 
-extern int s3Mclk;
+extern int __svgalib_s3_s3Mclk;
 
 static int Trio64_get_mclk(void)
 {
@@ -583,7 +583,7 @@ static void Trio64_init(void)
     if (__svgalib_driver_report)
 	printf("svgalib: RAMDAC: Trio64: MCLK = %0.3f MHz\n",
 	       mclk / 1000.0);
-    s3Mclk = mclk;
+    __svgalib_s3_s3Mclk = mclk;
 }
 
 static int Trio64_map_clock(int bpp, int pixelclock)
