@@ -253,7 +253,7 @@ static DacMethods *dac_used;
 static ClockChipMethods *clk_used;
 static int dac_speed = 0;
 
-int s3Mclk = 0;
+int __svgalib_s3_s3Mclk = 0;
 
 /* forward declaration. */
 extern DriverSpecs __svgalib_s3_driverspecs;
@@ -374,7 +374,7 @@ static void s3_getmodeinfo(int mode, vga_modeinfo * modeinfo)
     }
 
     /* Adjust line width (only for SVGA modes) */
-    if (!(mode < G640x480x256 || mode == G720x348x2))
+    if (!IS_IN_STANDARD_VGA_DRIVER(mode))
 	modeinfo->linewidth = s3_adjlinewidth(modeinfo->linewidth);
 
     modeinfo->maxlogicalwidth = 8184;
@@ -475,6 +475,7 @@ static int s3_ext_set(unsigned what, va_list params)
 	    return old_values;
 	}
     default:
+        break;
     }
     return 0;
 }
@@ -487,7 +488,7 @@ static int s3_modeavailable(int mode)
     ModeInfo *modeinfo;
     ModeTiming *modetiming;
 
-    if (mode < G640x480x256 || mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     /* Enough memory? */
@@ -641,16 +642,16 @@ static void s3_setregs(const unsigned char regs[], int mode)
 	CR1 = __svgalib_inCR(0x01);
 	__svgalib_outCR(0x01, CR1 | 0x20);
 
-	__svgalib_outbCR(0x55, __svgalib_inCR(0x55) | 1);
-	__svgalib_outCR(0x66, regs[S3_CR66]);
-	__svgalib_outCR(0x67, regs[S3_CR67]);	/* S3 pixmux. */
+	__svgalib_outcrtc(0x55, __svgalib_inCR(0x55) | 1);
+	__svgalib_outcrtc(0x66, regs[S3_CR66]);
+	__svgalib_outcrtc(0x67, regs[S3_CR67]);	/* S3 pixmux. */
 
 	dac_used->restoreState(regs + S3_DAC_OFFSET);
 
-	__svgalib_outCR(0x6D, regs[S3_CR6D]);
-	__svgalib_outbCR(0x55, __svgalib_inCR(0x55) & ~1);
+	__svgalib_outcrtc(0x6D, regs[S3_CR6D]);
+	__svgalib_outcrtc(0x55, __svgalib_inCR(0x55) & ~1);
 
-	__svgalib_outCR(0x01, CR1);	/* Unblank screen. */
+	__svgalib_outcrtc(0x01, CR1);	/* Unblank screen. */
     }
 #ifdef S3_LINEAR_SUPPORT
     if (mode == TEXT && s3_linear_addr)
@@ -852,8 +853,8 @@ static void s3_initializemode(unsigned char *moderegs,
 	    }
 	    if (s3_memory < 2048 || s3_chiptype == S3_TRIO32)
 		clock *= 2;
-	    if (s3Mclk > 0)
-	    	mclk = s3Mclk;
+	    if (__svgalib_s3_s3Mclk > 0)
+	    	mclk = __svgalib_s3_s3Mclk;
 	    else if (s3_chiptype == S3_801 || s3_chiptype == S3_805)
 		mclk = 50000;	/* Assumption. */
 	    else
@@ -1092,7 +1093,7 @@ static int s3_setmode(int mode, int prv_mode)
     unsigned char moderegs[S3_TOTAL_REGS];
     int res;
 
-    if (mode < G640x480x256 || mode == G720x348x2) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
 	/* Let the standard VGA driver set standard VGA modes. */
 	res = __svgalib_vga_driverspecs.setmode(mode, prv_mode);
 	if (res == 0) {
@@ -1141,7 +1142,7 @@ static int s3_setmode(int mode, int prv_mode)
 static unsigned char rdinx(int port, unsigned char index)
 {
     outb(port, index);
-    return inb(port + 1);
+    return port_in(port + 1);
 }
 
 static void wrinx(int port, unsigned char index, unsigned char val)
@@ -1171,7 +1172,7 @@ static int s3_test(void)
 {
     int vgaIOBase, vgaCRIndex, vgaCRReg;
 
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
+    vgaIOBase = (port_in(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
     vgaCRIndex = vgaIOBase + 4;
     vgaCRReg = vgaIOBase + 5;
 
@@ -1222,12 +1223,12 @@ static void s3_setpage(int page)
 #endif
     s3_unlock();
     outb(CRT_IC, 0x35);
-    outb(CRT_DC, (inb(CRT_DC) & 0xF0) | (page & 0x0F));
+    outb(CRT_DC, (port_in(CRT_DC) & 0xF0) | (page & 0x0F));
     if (s3_chiptype >= S3_801) {
 	outb(CRT_IC, 0x51);
-	outb(CRT_DC, (inb(CRT_DC) & ~0x0C) | ((page & 0x30) >> 2));
+	outb(CRT_DC, (port_in(CRT_DC) & ~0x0C) | ((page & 0x30) >> 2));
     }
-    inb(CRT_DC);			/* ARI: Ferraro says: required for first generation 911 only */
+    port_in(CRT_DC);			/* ARI: Ferraro says: required for first generation 911 only */
     s3_lock();
 }
 
@@ -1244,7 +1245,7 @@ static void s3_setpage864(int page)
     s3_unlock();
     /* "Linear" mode banking. */
     outb(CRT_IC, 0x6A);
-    outb(CRT_DC, (inb(CRT_DC) & ~0x3F) | page);
+    outb(CRT_DC, (port_in(CRT_DC) & ~0x3F) | page);
     s3_lock();
 }
 
@@ -1267,16 +1268,16 @@ static void s3_setdisplaystart(int address)
     s3_unlock();
     outw(CRT_IC, 0x0d | ((address << 6) & 0xff00));	/* sa2-sa9 */
     outw(CRT_IC, 0x0c | ((address >> 2) & 0xff00));	/* sa10-sa17 */
-    inb(0x3da);			/* set ATC to addressing mode */
+    port_in(0x3da);			/* set ATC to addressing mode */
     outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
-    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
+    outb(ATT_IW, (port_in(ATT_R) & 0xf0) | ((address & 3) << 1));
     /* write sa0-1 to bits 1-2 */
 
     outb(CRT_IC, 0x31);
-    outb(CRT_DC, (inb(CRT_DC) & ~0x30) | ((address & 0xc0000) >> 14));
+    outb(CRT_DC, (port_in(CRT_DC) & ~0x30) | ((address & 0xc0000) >> 14));
     if (s3_chiptype >= S3_801) {
 	outb(CRT_IC, 0x51);
-	outb(CRT_DC, (inb(CRT_DC) & ~0x03) | ((address & 0x300000) >> 20));
+	outb(CRT_DC, (port_in(CRT_DC) & ~0x03) | ((address & 0x300000) >> 20));
     }
     s3_lock();
 }
@@ -1410,7 +1411,7 @@ static char *s3_config_options[] =
     "clocks", "ramdac", "dacspeed", "clockchip", NULL
 };
 
-static char *s3_process_option(int option, int mode)
+static char *s3_process_option(int option, int mode, char **nptr)
 {
 /*
  * option is the number of the option string in s3_config_options,
@@ -1423,7 +1424,7 @@ static char *s3_process_option(int option, int mode)
 	for (;;) {
 	    char *ptr;
 	    int freq;
-	    ptr = strtok(NULL, " ");
+	    ptr = __svgalib_token(nptr);
 	    if (ptr == NULL)
 		break;
 	    /*
@@ -1439,7 +1440,7 @@ static char *s3_process_option(int option, int mode)
     }
     if (option == 1) {		/* "Ramdac" */
 	char *ptr;
-	ptr = strtok(NULL, " ");
+	ptr = __svgalib_token(nptr);
 #ifdef INCLUDE_IBMRGB52x_DAC
 	if (strcasecmp(ptr, "IBMRGB52x") == 0)
 	    dac_used = &__svgalib_IBMRGB52x_methods;
@@ -1478,7 +1479,7 @@ static char *s3_process_option(int option, int mode)
     }
     if (option == 2) {		/* "Dacspeed" */
 	char *ptr;
-	ptr = strtok(NULL, " ");
+	ptr = __svgalib_token(nptr);
 	/*
 	 * This doesn't protect against bad characters
 	 * (atoi() doesn't detect errors).
@@ -1503,7 +1504,7 @@ static char *s3_process_option(int option, int mode)
 	}
 	return ptr;
     }
-    return strtok(NULL, " ");
+    return __svgalib_token(nptr);
 }
 
 
