@@ -49,7 +49,10 @@ chipset companies still do this).
 #define PCI_CHIP_SIS5598        0x0200
 #define PCI_CHIP_SIS5597        0x0200
 #define PCI_CHIP_SIS6326        0x6326
-#define PCI_CHIP_SIS530        0x6306
+#define PCI_CHIP_SIS530         0x6306
+#define PCI_CHIP_SIS300		0x0300
+#define PCI_CHIP_SIS540		0x5300
+#define PCI_CHIP_SIS630		0x6300
 
 #define ClockReg	XR(0x07) 
 #define DualBanks	XR(0x0B)
@@ -86,8 +89,8 @@ static CardSpecs *cardspecs;
 
 static void sis_setpage(int page)
 {
-        outb(0x3cb+io_reloc,page);
-        outb(0x3cd+io_reloc,page);
+        outb(0x3cb+__svgalib_io_reloc,page);
+        outb(0x3cd+__svgalib_io_reloc,page);
 }
 
 static int __svgalib_sis_inlinearmode(void)
@@ -168,10 +171,7 @@ static int sis_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
@@ -233,7 +233,7 @@ static void sis_initializemode(unsigned char *moderegs,
     moderegs[BankReg] = 0x02;
     moderegs[DualBanks] = 0x00; 
 
-    if ( sis_is_linear ) {
+    if ( sis_is_linear || __svgalib_emulatepage) {
 	    moderegs[BankReg] |= 0x80;  	/* enable linear mode addressing */
 	    moderegs[LinearAddr0] = (sis_linear_base & 0x07f80000) >> 19 ; 
             moderegs[LinearAddr1] = ((sis_linear_base & 0xf8000000) >> 27) |
@@ -315,7 +315,7 @@ static int sis_setmode(int mode, int prv_mode)
     ModeInfo *modeinfo;
     int tmp;
 
-    if ((mode < G640x480x256)||(mode==G720x348x2)) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
         if(sis_chiptype >= SIS_5597) { 
 		/* Work Arrout for vesafb case */
                 read_xr(0x06,tmp);
@@ -389,6 +389,9 @@ static int sis_test(void)
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS5597)||
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS5598)||
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS530)||
+       (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS540)||
+       (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS630)||
+       (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS300)||
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS6326)
        )){
        sis_init(0,0,0);
@@ -444,7 +447,7 @@ static int sis_linear(int op, int param)
         write_xr(6,temp);
         return 0;
     };
-    if (op==LINEAR_DISABLE){
+    if (op==LINEAR_DISABLE && !__svgalib_emulatepage){
         int temp;
 
         sis_is_linear=0; 
@@ -485,7 +488,11 @@ static int sis_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
             write_xr(0x1e,(in_xr(0x1e)&0xf8)|((p2>>8)&0x07));
             break;
         case CURSOR_SELECT:
+#if 1
+            i=sis_memory*1024-(16-p1)*1024;
+#else
             i=sis_memory*1024-16384;
+#endif
             if((sis_chiptype==SIS_6326)&&(sis_memory>4096))
                i-=4096*1024; /* 6326 can't put cursor beyond 4MB */
             for(j=0;j<6;j++)write_xr(0x14+j,cur_colors[p1*6+j]>>2);
@@ -524,11 +531,11 @@ static int sis_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
                                 l1<<=1;
                                 l2<<=1;
                             }
-                            *(LINEAR_MEM_POINTER+i+16*j+k)=c;
-                            *(LINEAR_MEM_POINTER+i+16*j+k+8)=0xaa;
+                            *(LINEAR_POINTER+i+16*j+k)=c;
+                            *(LINEAR_POINTER+i+16*j+k+8)=0xaa;
                         }
                     }
-                    memset(LINEAR_MEM_POINTER+i+512,0xaa,512);
+                    memset(LINEAR_POINTER+i+512,0xaa,512);
                     break;
             }
             break;
@@ -609,6 +616,9 @@ static int sis_init(int force, int par1, int par2)
             case PCI_CHIP_SG86C215: sis_chiptype=SIS_86C215; break;
             case PCI_CHIP_SG86C225: sis_chiptype=SIS_86C225; break;
             case PCI_CHIP_SIS5597: sis_chiptype=SIS_5597; break;
+            case PCI_CHIP_SIS300:
+            case PCI_CHIP_SIS540:
+            case PCI_CHIP_SIS630:
             case PCI_CHIP_SIS530: sis_chiptype=SIS_530; break;
 /*            case PCI_CHIP_SIS5598: sis_chiptype=SIS_5598; break;*/
             case PCI_CHIP_SIS6326: sis_chiptype=SIS_6326; break;
@@ -616,8 +626,8 @@ static int sis_init(int force, int par1, int par2)
     };
     
     if(sis_chiptype>=SIS_5597) {
-       io_reloc=(buf[6]&0xffffff80)-0x380; /* should work at least for 6326 and 530 */
-       rel_io_mapio();
+       __svgalib_io_reloc=(buf[6]&0xffffff80)-0x380; /* should work at least for 6326 and 530 */
+       __svgalib_rel_io_mapio();
     }
     
     if(sis_memory==0){
