@@ -1,4 +1,4 @@
-#define GTF
+#undef GTF
 /* 
  * Generic mode timing module.
  */
@@ -406,7 +406,8 @@ int vga_getcurrenttiming(int *pixelClock,
    return 1;
 };
 
-int vga_changetiming(int pixelClock,       		                       int HDisplay,
+int vga_changetiming(int pixelClock,
+   		     int HDisplay,
    		     int HSyncStart,
                      int HSyncEnd,
                      int HTotal,
@@ -417,6 +418,7 @@ int vga_changetiming(int pixelClock,       		                       int HDisplay
                      int flags) {
   if(current_timing){
      new_timing=*current_timing;
+     new_timing.pixelClock+=pixelClock;
      new_timing.HDisplay+=HDisplay;
      new_timing.HSyncStart+=HSyncStart;
      new_timing.HSyncEnd+=HSyncEnd;
