@@ -555,13 +555,13 @@ static int tvga8900_test(void)
      * Check first that we have a Trident card.
      */
     outb(SEQ_I, 0x0b);
-    save0b = inb(SEQ_D);
+    save0b = port_in(SEQ_D);
     outw(SEQ_I, 0x000B);	/* Switch to Old Mode */
-    inb(SEQ_D);			/* Now to New Mode */
+    port_in(SEQ_D);			/* Now to New Mode */
     outb(SEQ_I, 0x0E);
-    origVal = inb(SEQ_D);
+    origVal = port_in(SEQ_D);
     outb(SEQ_D, 0x00);
-    newVal = inb(SEQ_D) & 0x0F;
+    newVal = port_in(SEQ_D) & 0x0F;
     outb(SEQ_D, (origVal ^ 0x02));
 
     if (newVal != 2) {
@@ -613,27 +613,27 @@ static void tvga8900_setdisplaystart(int address)
 	outw(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
     }
     if (__svgalib_cur_mode != G320x200x256) {
-	inb(0x3da);		/* set ATC to addressing mode */
+	port_in(0x3da);		/* set ATC to addressing mode */
 	outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
 	if (tvga8900_memory == 1024) {
-	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | (address & 7));
+	    outb(ATT_IW, (port_in(ATT_R) & 0xf0) | (address & 7));
 	    /* write sa0-2 to bits 0-2 */
 	    address >>= 1;
 	} else
-	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
+	    outb(ATT_IW, (port_in(ATT_R) & 0xf0) | ((address & 3) << 1));
 	/* write sa0-1 to bits 1-2 */
     }
   setextendedbits:
     outb(CRT_IC, 0x1e);
-    outb(CRT_DC, (inb(CRT_DC) & 0x5f) | 0x80	/* set bit 7 */
+    outb(CRT_DC, (port_in(CRT_DC) & 0x5f) | 0x80	/* set bit 7 */
 	 | ((address & 0x40000) >> 13));	/* sa18: write to bit 5 */
     outb(SEQ_I, 0x0b);
     outb(SEQ_D, 0);		/* select 'old mode' */
     outb(SEQ_I, 0x0e);
-    outb(SEQ_D, (inb(SEQ_D) & 0xfe)
+    outb(SEQ_D, (port_in(SEQ_D) & 0xfe)
 	 | ((address & 0x80000) >> 19));	/* sa19: write to bit 0 */
     outb(SEQ_I, 0x0b);
-    inb(SEQ_D);			/* return to 'new mode' */
+    port_in(SEQ_D);			/* return to 'new mode' */
 }
 
 
@@ -719,7 +719,7 @@ bit 0:		force noninterlaced if set
 
 	/* check version */
 	outw(SEQ_I, 0x000b);
-	switch (inb(SEQ_D)) {
+	switch (port_in(SEQ_D)) {
 	case 0x02:			/* 8800cs */
 	case 0x03:			/* 8900b */
 	case 0x04:			/* 8900c */
