@@ -69,6 +69,8 @@ return vesa_is_linear;
 
 static void vesa_getmodeinfo(int mode, vga_modeinfo *modeinfo)
 {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
+	return __svgalib_vga_driverspecs.getmodeinfo(mode, modeinfo);
 
     if(modeinfo->colors==16)return;
 
@@ -87,6 +89,19 @@ static void vesa_getmodeinfo(int mode, vga_modeinfo *modeinfo)
 	    modeinfo->flags |= IS_LINEAR;
       }
       
+    /* to get the logical scanline width */
+    memset(&vesa_r, 0, sizeof(vesa_r));
+
+    vesa_r.eax = 0x4f01;
+    vesa_r.ecx = SVGALIB_VESA[mode];
+    vesa_r.es = (unsigned int)vesa_data.mode >> 4;
+    vesa_r.edi = (unsigned int)vesa_data.mode & 0xf;
+
+    if (!__svgalib_LRMI_int(0x10, &vesa_r)) {
+       fprintf(stderr, "Can't get mode info (vm86 failure)\n");
+       return;
+    }
+    modeinfo->linewidth = vesa_data.mode->bytes_per_scanline;
 }
 
 /* Read and save chipset-specific registers */
@@ -130,10 +145,7 @@ static int vesa_modeavailable(int mode)
     ModeInfo *modeinfo;
 
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
+    if (IS_IN_STANDARD_VGA_DRIVER(mode))
 	return __svgalib_vga_driverspecs.modeavailable(mode);
 
     info = &__svgalib_infotable[mode];
@@ -157,7 +169,7 @@ static int vesa_setmode(int mode, int prv_mode)
 {
     vesa_bpp=1;
     vesa_granularity=1;
-    if ((mode < G640x480x256)|| (mode == G720x348x2)) {
+    if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
 
         if(__svgalib_vesatext){
             vesa_r.eax=0x4f02; /* make sure we are in a regular VGA mode before we start */
@@ -369,7 +381,7 @@ static int vesa_init(int force, int par1, int par2)
     };
 
     __svgalib_LRMI_init();
-    for(i=0;i<__GLASTMODE;i++)SVGALIB_VESA[i]=((i<G640x480x256)||(i==G720x348x2));
+    for(i=0;i<__GLASTMODE;i++)SVGALIB_VESA[i]=IS_IN_STANDARD_VGA_DRIVER(i);
 
     vesa_data.info = __svgalib_LRMI_alloc_real(sizeof(struct vbe_info_block)
 	 + sizeof(struct vbe_mode_info_block));
@@ -392,7 +404,7 @@ static int vesa_init(int force, int par1, int par2)
     if(vesa_data.info->vbe_version>=0x0300)vesa_chiptype=2 ; 
 
     vesa_memory = vesa_data.info->total_memory*64;
-        
+
     mode_list = (short int *)(vesa_data.info->video_mode_list_seg * 16 + vesa_data.info->video_mode_list_off);
 
     while (*mode_list != -1) {
@@ -537,13 +549,13 @@ static int vesa_init(int force, int par1, int par2)
 
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = vesa_memory;
-    cardspecs->maxPixelClock4bpp = 75000;	
-    cardspecs->maxPixelClock8bpp = 175500;	
-    cardspecs->maxPixelClock16bpp = 144000;	
-    cardspecs->maxPixelClock24bpp = 75000;
-    cardspecs->maxPixelClock32bpp = 94500;
+    cardspecs->maxPixelClock4bpp = 300000;	
+    cardspecs->maxPixelClock8bpp = 300000;	
+    cardspecs->maxPixelClock16bpp = 300000;	
+    cardspecs->maxPixelClock24bpp = 300000;
+    cardspecs->maxPixelClock32bpp = 300000;
     cardspecs->flags = CLOCK_PROGRAMMABLE;
-    cardspecs->maxHorizontalCrtc = 2040;
+    cardspecs->maxHorizontalCrtc = 4088;
     cardspecs->nClocks =1;
     cardspecs->clocks = NULL;
     cardspecs->mapClock = vesa_map_clock;
