@@ -26,13 +26,26 @@
 #include "vgabg.h"
 /*#endif*/
 
+vga_cardinfo *vga_getcardinfo(void) {
+    vga_cardinfo *vci;
+    
+    vci = malloc(sizeof(vga_cardinfo));
+    if(vci==NULL) return vci;
+    vci->version = 0x100;
+    vci->size = sizeof(vga_cardinfo);
+    vci->chipset = __svgalib_chipset;
+    vci->physmem = __svgalib_linear_mem_base;
+    
+    return vci;
+}
+
 void vga_waitretrace(void)
 {
     if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->waitretrace) {
         __svgalib_driverspecs->emul->waitretrace();
     } else {
-	while (!(inb(0x3da) & 8));
-	while (inb(0x3da) & 8);
+	while (!(__svgalib_inis1() & 8));
+	while (__svgalib_inis1() & 8);
     }
 }
 
@@ -52,6 +65,9 @@ static void *__svgalib_linearframebuffer;
 unsigned char *
  vga_getgraphmem(void)
 {
+
+    DTP((stderr,"getgraphmem\n"));
+
     if (vga_getmodeinfo(__svgalib_cur_mode)->flags & IS_LINEAR)
 	return __svgalib_linearframebuffer;
     return __svgalib_graph_mem;
@@ -62,21 +78,16 @@ unsigned char *
 
 int __svgalib_physmem(void)
 {
-#ifdef __alpha__
-    printf("__svgalib_physmem: are you sure you wanna do this??\n");
-    return -1;
-#else
     struct sysinfo si;
     si.totalram = 0;
     syscall(SYS_sysinfo, &si);
     return si.totalram;
-#endif
 }
 
 static unsigned char *
  map_framebuffer(unsigned address, int size)
 {
-    return( LINEAR_MEM_POINTER );
+    return( LINEAR_POINTER );
 }
 
 /*
@@ -150,6 +161,9 @@ int vga_setlinearaddressing(void)
     int (*lfn) (int op, int param) = __svgalib_driverspecs->linear;
     int memory, mappedMemory;
     vga_modeinfo *modeinfo;
+
+    DTP((stderr,"setlinearaddressing\n"));
+
     modeinfo = vga_getmodeinfo(__svgalib_cur_mode);
     if (modeinfo->flags & EXT_INFO_AVAILABLE)
 	memory = modeinfo->memory * 1024;
@@ -164,26 +178,21 @@ int vga_setlinearaddressing(void)
 	/* 'Linear' driver function available. */
 	unsigned long mapaddr;
 	int i;
-#ifndef __alpha__
 	unsigned long gran, maxmap;
 	int range;
-#endif
 	/* First try the fixed bases that the driver reports. */
 	i = 0;
 	for (;; i++) {
 	    mapaddr = (*lfn) (LINEAR_QUERY_BASE, i);
 	    if ((int) mapaddr == -1)
 		break;
-#ifndef __alpha__
 	    if (mapaddr <= __svgalib_physmem())
 		continue;
-#endif
 	    /* Check that the linear framebuffer is writable. */
 	    if ((mappedMemory = verify_linear_mapping(
 						 mapaddr, memory)) != -1)
 		goto foundlinearframebuffer;
 	}
-#ifndef __alpha__
 	/* Attempt mapping for device that maps in low memory range. */
 	gran = (*lfn) (LINEAR_QUERY_GRANULARITY, 0);
 	range = (*lfn) (LINEAR_QUERY_RANGE, 0);
@@ -198,7 +207,6 @@ int vga_setlinearaddressing(void)
 	}
 	if ((mappedMemory = verify_linear_mapping(mapaddr, memory)) == -1)
 	    return -1;
-#endif
 
       foundlinearframebuffer:
 	(*lfn) (LINEAR_ENABLE, mapaddr);
@@ -211,6 +219,8 @@ int vga_setlinearaddressing(void)
 	}
 	__svgalib_modeinfo_linearset |= IS_LINEAR;
 
+        graph_mem = LINEAR_POINTER;
+
 	if (memory != mappedMemory)
 	    printf("svgalib: Warning, card has %dK, only %dK available in linear mode.\n",
 		   memory >> 10, mappedMemory >> 10);
