@@ -12,13 +12,11 @@
 #include <stdlib.h>
 
 #include "vga.h"
+#include "vgaio.h"
 #include "vgabg.h"
 #include "libvga.h"
 #include "driver.h"
 
-static int current_index = -1;
-static int setget = -1;		/* flag var to indicate last lut operation */
-
 /*
  * In grayscale mode, we convert RGB colors to a Y component on the
  * green-channel (the Y component is used in grayscale TV sets for the
@@ -56,9 +54,9 @@ static int get_lut(int index, int *red, int *green, int *blue)
     /* prevents lockups on mach64 */
     if ((__svgalib_chipset == MACH64)) {
         outb(0x02ec+0x5c00,index);
-        *red=inb(0x02ec+0x5c01);
-        *green=inb(0x02ec+0x5c01);
-        *blue=inb(0x02ec+0x5c01);
+        *red=port_in(0x02ec+0x5c01);
+        *green=port_in(0x02ec+0x5c01);
+        *blue=port_in(0x02ec+0x5c01);
         return 0;
     }
 
@@ -69,6 +67,9 @@ static int get_lut(int index, int *red, int *green, int *blue)
 
 int vga_setpalette(int index, int red, int green, int blue)
 {
+
+    DTP((stderr,"setpalette %i %i %i %i\n",index,red,green,blue));
+
     if (__svgalib_grayscale) {
 	if ((unsigned) index >= sizeof(__svgalib_green_backup) / sizeof(__svgalib_green_backup[0])) {
 	    printf("vga_setpalette: color index %d out of range\n", index);
@@ -108,7 +109,10 @@ int vga_setpalvec(int start, int num, int *pal)
 {
     int i;
 
-    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->setpalette) {
+    DTP((stderr,"setpalvec %i %i %x\n",start,num,pal));
+
+    if ((__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->setpalette) ||
+	(__svgalib_outpal!=__svgalib_vga_outpal)) {
         for (i = start; i < start + num; ++i) {
 	    vga_setpalette(i, pal[0], pal[1], pal[2]);
 	    pal += 3;
@@ -123,7 +127,11 @@ int vga_setpalvec(int start, int num, int *pal)
             string[i] = pal[i];
 
         port_out ( start, 0x3c8 );
+#if 0
         port_rep_outb( string, num * 3, 0x3c9 );
+#else
+        for(i=0;i<num*3;i++)port_out(string[i],0x3c9);
+#endif
     }
 
     return num;
